namespace Brjnk.Common
{
    using System;
    using System.Diagnostics;
    using System.Threading;

    internal class AsyncTask<T>
    {
        private Func<T> func;
        private Action<T, object> callback;
        private SynchronizationContext context;
        private object userObject;
        private Action<Exception, object> exceptionCallback;

        public AsyncTask(Func<T> asyncFunc, SynchronizationContext context, Action<T, object> callback, object userObject, Action<Exception, object> exceptionCallback)
        {
            if (asyncFunc == null) throw new ArgumentNullException("asyncFunc");
            func = asyncFunc;
            this.callback = callback;
            this.context = context;
            this.userObject = userObject;
            this.exceptionCallback = exceptionCallback;
        }

        public void Start()
        {
            T result = default(T);
            try
            {
                result = func();
                if (callback != null)
                    Invoke(new SendOrPostCallback((o) => callback(result, userObject)), null);
            }
            catch (Exception ex)
            {
                Debug.Fail("Byla zachycena vyjmka pi vykonvn asynchron operace tdou AsyncProcessing.", ex.ToString());
                if (exceptionCallback != null)
                    Invoke(new SendOrPostCallback((o) => exceptionCallback(ex, userObject)), null);
            }
        }

        private void Invoke(SendOrPostCallback toInvoke, object state)
        {
            if (context == null)
            {
                toInvoke(state);
            }
            else
            {
                context.Post(toInvoke, state);
            }
        }
    }
}