namespace Brjnk.Common
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;

    public class DeepClone
    {
        //Inspirace: http://www.codeproject.com/Articles/38270/Deep-copy-of-objects-in-C

        public static T Clone<T>(T obj)
        {
            return (T)Clone(obj, new List<Tuple<object, object>>());
        }

        //Z principu nemus fungovat na vechny tdy.
        //Problm mohou zpsobit napklad udlosti
        //Je vhodn pouze pro datov tdy.
        private static object Clone(object obj, List<Tuple<object, object>> circular)
        {
            if (obj == null)
                return null;
            Type type = obj.GetType();

            //Hodnotov typ - bude pouze vrcen
            if (type.IsValueType || type == typeof(string))
            {
                return obj;
            }

            //Pole - bude vytvoeno nov pole a prvky budou hlubok kopie.
            if (type.IsArray)
            {
                var retVal = GetCloned(circular, obj);
                if (retVal != null) return retVal;

                string typeNoArray = type.FullName.Replace("[]", string.Empty);
                Type elementType = Type.GetType(typeNoArray + ", " + type.Assembly.FullName);
                var array = obj as Array;
                Array copied = Array.CreateInstance(elementType, array.Length);
                AddCloned(circular, obj, copied);
                for (int i = 0; i < array.Length; i++)
                {
                    object element = array.GetValue(i);

                    if (element != null)
                    {
                        object copy = Clone(element, circular);
                        copied.SetValue(copy, i);
                    }
                }
                return Convert.ChangeType(copied, obj.GetType());
            }
            //Objekt - bude vytvoen nov, do kterho budou zkoprovny vechny hodnoty vnitnch field
            if (type.IsClass)
            {
                var retVal = GetCloned(circular, obj);
                if (retVal != null) return retVal;
                object instance = Activator.CreateInstance(obj.GetType());
                AddCloned(circular, obj, instance);
                FieldInfo[] fields = type.GetFields(BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance);
                foreach (FieldInfo field in fields)
                {
                    object fieldValue = field.GetValue(obj);
                    if (fieldValue == null)
                        continue;
                    object copy = GetCloned(circular, fieldValue);
                    if (copy == null) copy = Clone(fieldValue, circular);
                    field.SetValue(instance, copy);
                }
                return instance;
            }
            else
                throw new ArgumentException("Unknown type");
        }

        private static void AddCloned(List<Tuple<object, object>> circular, object obj, object clone)
        {
            circular.Add(new Tuple<object, object>(obj, clone));
        }

        private static object GetCloned(List<Tuple<object, object>> circular, object obj)
        {
            foreach (var item in circular)
            {
                if (object.ReferenceEquals(item.Item1, obj)) return item.Item2;
            }
            return null;
        }
    }
}