namespace Brjnk.Common
{
    using System;
    using System.Runtime.InteropServices;
    using System.Windows.Interop;
    using System.Windows.Media;
    using System.Windows.Media.Imaging;

    /// <summary>
    /// Umouje zskat ikonu (ImageSource class) pro soubor.
    /// </summary>
    public static class IconsReader
    {
        private const uint SHGFI_ICON = 0x100;

        private const int SHELLICONSIZE = 0x00004;

        [DllImport("shell32.dll", CharSet = CharSet.Auto)]
        public static extern IntPtr SHGetFileInfo(string pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, uint cbFileInfo, uint uFlags);

        public static ImageSource GetIcon(string filePath)
        {
            if (!System.IO.File.Exists(filePath)) return null;

            SHFILEINFO fileInfo = new SHFILEINFO();
            SHGetFileInfo(filePath, (uint)0, ref fileInfo, (uint)Marshal.SizeOf(fileInfo), SHGFI_ICON | SHELLICONSIZE);
            if (fileInfo.hIcon != IntPtr.Zero)
            {
                return ConvertIcon(fileInfo);
            }
            else
            {
                System.Diagnostics.Debug.WriteLine(filePath, "Ikonu se nepodailo nalzt");
                return null;
            }
        }

        private static ImageSource ConvertIcon(SHFILEINFO fileInfo)
        {
            var retVal = Imaging.CreateBitmapSourceFromHIcon(fileInfo.hIcon, System.Windows.Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());

            if (!DestroyIcon(fileInfo.hIcon)) throw new Exception("DestroyIcon failed");
            return retVal;
        }

        [DllImport("user32.dll", SetLastError = true)]
        private static extern bool DestroyIcon(IntPtr hIcon);
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
    public struct SHFILEINFO
    {
        public IntPtr hIcon;
        public int iIcon;
        public uint dwAttributes;

        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
        public string szDisplayName;

        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        public string szTypeName;
    };
}