namespace Brjnk.MVVM
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Linq;

    /// <summary>
    /// </summary>
    public class ChangedEventArgs : EventArgs
    {
        private List<object> objectNodes = new List<object>();

        private ReadOnlyCollection<object> objectNodesRO;

        private List<PropertyInfo> propertyNodes = new List<PropertyInfo>();

        private ReadOnlyCollection<PropertyInfo> propertyNodesRO;

        private ChangedEventArgs(object sender, ChangeType type)
        {
            this.Type = type;

            objectNodesRO = new ReadOnlyCollection<object>(objectNodes);
            propertyNodesRO = new ReadOnlyCollection<PropertyInfo>(propertyNodes);
            AddNodeObject(sender);
        }

        /// <summary>
        /// Vytvo ChangedEventArgs. Pokud dolo na objektu INotifyPropertyChanged ke zmn vlastnosti
        /// </summary>
        public ChangedEventArgs(object sender, PropertyInfo property)
            : this(sender, ChangeType.PropertyChanged)
        {
            AddNodePropertyInfo(property);
        }

        /// <summary>
        /// Vytvo ChangedEventArgs. Pokud dolo na sledovanm objektu DataItem ke zmn vlastnosti
        /// </summary>
        public ChangedEventArgs(object sender, PropertyInfo property, object oldValue, object newValue)
            : this(sender, ChangeType.PropertyChanged)
        {
            AddNodePropertyInfo(property);
            this.OldValue = oldValue;
            this.NewValue = newValue;
            this.IsOldNewValueSpecified = true;
        }

        /// <summary>
        /// Vytvo ChangedEventArgs. Pokud dolo k pidn nebo odebrn potomka.
        /// </summary>
        public ChangedEventArgs(object sender, ChangeType type, object child)
            : this(sender, type)
        {
            if (type != ChangeType.ChildAdded && type != ChangeType.ChildRemoved) return;
            this.Child = child;
        }

        /// <summary>
        /// Vytvo ChangedEventArgs pro udlost typu Reset
        /// </summary>
        public ChangedEventArgs(object sender)
            : this(sender, ChangeType.Reset)
        { }

        public void AddNodeObject(object instance)
        {
            objectNodes.Add(instance);
        }

        public void AddNodePropertyInfo(PropertyInfo property)
        {
            propertyNodes.Add(property);
        }

        public bool BeginsPathWith(params string[] p)
        {
            if (propertyNodes.Count < p.Length) return false;
            for (int i = 0; i < p.Length; i++)
            {
                if (p[i] != propertyNodes[propertyNodes.Count - 1 - i].Name) return false;
            }
            return true;
        }

        public bool PathEquals(params string[] p)
        {
            if (propertyNodes.Count != p.Length) return false;
            for (int i = 0; i < p.Length; i++)
            {
                if (p[i] != propertyNodes[propertyNodes.Count - 1 - i].Name) return false;
            }
            return true;
        }

        public ChangeType Type { get; private set; }

        public object Child { get; private set; }

        public object OldValue { get; private set; }

        /// <summary>
        /// Indikuje, zda je OldValue a NewValue dostupn. Hodnota OldValue je dostupn pouze v ppad e dolo ke zmn vlastnosti na potomku tdy DataItem.
        /// </summary>
        public bool IsOldNewValueSpecified { get; private set; }

        public object NewValue { get; private set; }

        public object Sender { get { return objectNodes.First(); } }

        public bool IsSenderDataItem { get { return Sender is DataItem; } }

        public IEnumerable<object> Nodes { get { return objectNodes.Reverse<object>(); } }

        public IEnumerable<PropertyInfo> PropertyNodes { get { return propertyNodes.Reverse<PropertyInfo>(); } }

        public string PropertyPath
        {
            get { return string.Join("/", PropertyNodes.Select(p => p.Name)); }
        }

        public string NodesPath
        {
            get { return string.Join("/", Nodes); }
        }
    }

    public enum ChangeType
    {
        ChildAdded,
        ChildRemoved,
        PropertyChanged,
        Reset,
    }
}