namespace Brjnk.MVVM
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Linq;

    /// <summary>
    /// </summary>
    public class Wrapper<T> : DataItem, IWrapper<T> where T : INotifyPropertyChanged
    {
        private static List<PropertyWrapperInfo<T>> wrappersInfo = new List<PropertyWrapperInfo<T>>();

        protected static void WrapProperty(PropertyWrapperInfo<T> wrapperInfo)
        {
            wrappersInfo.Add(wrapperInfo);
        }

        private Dictionary<string, object> wrappers = new Dictionary<string, object>();

        public Wrapper(T sourceObject)
        {
            this.source = sourceObject;
            sourceObject.PropertyChanged += new PropertyChangedEventHandler(sourceObject_PropertyChanged);
            foreach (var info in wrappersInfo)
            {
                wrappers.Add(info.PropertyName, CreateWrapper(info));
            }
            base.StartWatchingObject("Source", sourceObject);
        }

        protected object GetWrapper(string propertyName)
        {
            if (disposed) throw new InvalidOperationException("Metoda Release ji byla zavolna, tento objekt byl uvolnn.");
            return wrappers[propertyName];
        }

        private string propertyUpdating; //uchovv nzev prv upravovan vlastnosti zabalenho objektu, aby pi jej zmn nebyl znovuvytvoen wrapper

        protected void SetWrapper(string name, IWrapper wrapper)
        {
            propertyUpdating = name;
            if (disposed) throw new InvalidOperationException("Metoda Release ji byla zavolna, tento objekt byl uvolnn.");
            var oldWrapper = wrappers[name];

            ReleaseWrapper(oldWrapper);

            wrappers[name] = wrapper;
            StartWatchingObject(name, wrapper);
            var info = wrappersInfo.Where(w => w.PropertyName == name).First();
            info.Setter(source, wrapper.GetWrappedObjectUntyped());
            OnChanged(new ChangedEventArgs(this, GetPropertyInfo(name), oldWrapper, wrapper));
            OnPropertyChanged(name);
            propertyUpdating = null;
        }

        private void sourceObject_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (propertyUpdating == e.PropertyName) return;
            object wrapper;
            if (wrappers.TryGetValue(e.PropertyName, out wrapper))
            {
                ReleaseWrapper(wrapper);
                wrappers.Remove(e.PropertyName);
                wrappers.Add(e.PropertyName, CreateWrapper(e.PropertyName));
            }
        }

        private void ReleaseWrapper(object wrapper)
        {
            if (wrapper is IWrapper) (wrapper as IWrapper).Release();
            base.StopWatchingObject(wrapper);
        }

        private object CreateWrapper(string p)
        {
            return CreateWrapper(wrappersInfo.Where(w => w.PropertyName == p).First());
        }

        private object CreateWrapper(PropertyWrapperInfo<T> propertyWrapperInfo)
        {
            var value = propertyWrapperInfo.Getter(source);
            object newWrapper;
            if ((object)value == null && propertyWrapperInfo.WrapNull == false)
            {
                newWrapper = null;
            }
            else
            {
                newWrapper = propertyWrapperInfo.WrapperFactory(value);
            }
            base.StartWatchingObject(propertyWrapperInfo.PropertyName, newWrapper);
            return newWrapper;
        }

        protected override void ProcessPropertyChanged(string propertyName)
        {
            if (propertyUpdating == propertyName) return;
            base.ProcessPropertyChanged(propertyName);
        }

        private T source;

        public T Source { get { return source; } }

        public T GetWrappedObject()
        {
            return source;
        }

        public object GetWrappedObjectUntyped()
        {
            return source;
        }

        private bool disposed;

        public void Release()
        {
            source.PropertyChanged -= new PropertyChangedEventHandler(sourceObject_PropertyChanged);
            base.StopWatchingObject(source);
            foreach (var wrapper in wrappers)
            {
                ReleaseWrapper(wrapper.Value);
            }
            wrappers.Clear();
            disposed = true;
        }
    }
}