namespace Brjnk.Common
{
    using System;
    using System.Windows;

    /// <summary>
    /// </summary>
    public static class LineSegmentHelper
    {
        /// <summary>
        /// Vrt prsek dvou pmek. Kad z pmek je definovna dvma body.
        /// Pmky nesm bt rovnobn
        /// </summary>
        public static Point GetIntersection(Point startPoint1, Point endPoint1, Point startPoint2, Point endPoint2)
        {
            if (startPoint1 == endPoint1 || startPoint2 == endPoint2)
                throw new ArgumentException("Body definujc pmku jsou toton");

            //http://en.wikipedia.org/wiki/Line-line_intersection
            double x1 = startPoint1.X;
            double y1 = startPoint1.Y;
            double x2 = endPoint1.X;
            double y2 = endPoint1.Y;
            double x3 = startPoint2.X;
            double y3 = startPoint2.Y;
            double x4 = endPoint2.X;
            double y4 = endPoint2.Y;

            double x = ((x1 * y2 - y1 * x2) * (x3 - x4) - (x1 - x2) * (x3 * y4 - y3 * x4)) /
                       ((x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4));

            double y = ((x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2) * (x3 * y4 - y3 * x4)) /
                       ((x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4));
            return new Point(x, y);
        }

        /// <summary>
        /// Vrt bod pmky (definovan startPoint1 a endPOint1), kter je nejbli pedanmu bodu (point)
        /// </summary>
        public static GetNearestPointResult GetNearestPoint(Point startPoint1, Point endPoint1, Point point)
        {
            Vector v = endPoint1 - startPoint1;
            v = new Vector(v.Y, -v.X);
            Point secondPoint = point + v;
            var intersection = GetIntersection(startPoint1, endPoint1, point, secondPoint);

            double d = v.Length;
            double d1 = (startPoint1 - intersection).Length;
            double d2 = (endPoint1 - intersection).Length;
            if (d2 > d)
                return new GetNearestPointResult(startPoint1, 0, (point - startPoint1).Length);
            else if (d1 > d)
                return new GetNearestPointResult(endPoint1, 1, (point - endPoint1).Length);
            else
                return new GetNearestPointResult(intersection, d1 / d, (point - intersection).Length);
        }

        public static Point GetPointByParameter(Point startPoint, Point endPoint, double parameter)
        {
            return startPoint + (endPoint - startPoint) * parameter;
        }

        public static double GetParameter(Point startPoint, Point endPoint, Point point)
        {
            return (point - startPoint).Length / (startPoint - endPoint).Length;
        }
    }

    public class GetNearestPointResult
    {
        public GetNearestPointResult(Point nearest, double parameter, double distance)
        {
            this.NearestPoint = nearest;
            this.Distance = distance;
            this.Parameter = parameter;
        }

        public Point NearestPoint { get; private set; }

        public double Distance { get; private set; }

        public double Parameter { get; private set; }
    }
}