namespace Brjnk.Common.UI
{
    using System;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;

    /// <summary>
    /// Textblock, kter automaticky mn velikost psma tak, aby se cel text veel do dostupnho prostoru.
    /// Velikost psma je urena "experimentln" opakovanm volnm TextBlock.Measure() pro rzn velikosti psma.
    /// Z tohoto dvodu mohou nastat vkonostn problmy pi dlouhch textech nebo opakovanm pouit tto komponenty!
    /// </summary>
    public class AutoScaleTextBlock : System.Windows.FrameworkElement
    {
        private TextBlock textBlock = new TextBlock();

        public AutoScaleTextBlock()
        {
            textBlock.TextWrapping = System.Windows.TextWrapping.Wrap;
            textBlock.SetBinding(TextBlock.FontSizeProperty, new Binding("RealFontSize") { Source = this });
            textBlock.SetBinding(TextBlock.TextProperty, new Binding("Text") { Source = this });
            this.AddVisualChild(textBlock);
        }

        protected override System.Windows.Media.Visual GetVisualChild(int index)
        {
            if (index != 0) throw new ArgumentOutOfRangeException();
            return textBlock;
        }

        protected override int VisualChildrenCount
        {
            get
            {
                return 1;
            }
        }

        //JIN ALGORITMUS:
        //Velikost psma je postupn zvtovna o stanoven krok
        //protected override System.Windows.Size MeasureOverride(System.Windows.Size availableSize)
        //{
        //    var stopky = new System.Diagnostics.Stopwatch();
        //    stopky.Start();
        //    const int stepsCount = 100;
        //    double min = MinFontSize;
        //    double step = (MaxFontSize - min) / stepsCount;

        //    for (int i = 1; i <= stepsCount; i++)
        //    {
        //        SetFontSize(min + (step) * i);
        //        textBlock.Measure(availableSize);
        //        var size = textBlock.DesiredSize;
        //        if (size.Height >= availableSize.Height)
        //        {
        //            SetFontSize(min + (step) * (i - 1));
        //            break;
        //        }
        //    }
        //    textBlock.Measure(availableSize);
        //    stopky.Stop();
        //    System.Diagnostics.Debug.WriteLine(stopky.ElapsedMilliseconds, "");
        //    return textBlock.DesiredSize;
        //}

        //Algoritmus:
        //Vyhledv vhodnou velikost pomoc binrnho vyhledvn
        protected override Size MeasureOverride(Size availableSize)
        {
            double min = MinFontSize; double max = MaxFontSize;
            SetFontSize(min);
            textBlock.Measure(availableSize);
            if (textBlock.DesiredSize.Height >= availableSize.Height) return textBlock.DesiredSize;

            while (max - min > 5)
            {
                SetFontSize(Middle(min, max));
                textBlock.Measure(availableSize);
                if (textBlock.DesiredSize.Height >= availableSize.Height)
                {
                    max = RealFontSize;
                }
                else
                { min = RealFontSize; }
            }

            SetFontSize(min);
            textBlock.Measure(availableSize);

            return textBlock.DesiredSize;
        }

        private double Middle(double min, double max)
        {
            return (max + min) / 2;
        }

        private void SetFontSize(double p)
        {
            SetValue(RealFontSizePropertyKey, p);
        }

        protected override System.Windows.Size ArrangeOverride(System.Windows.Size finalSize)
        {
            textBlock.Arrange(new Rect(finalSize));
            return new System.Windows.Size(textBlock.ActualWidth, textBlock.ActualHeight);
        }

        public double MaxFontSize
        {
            get { return (double)GetValue(MaxFontSizeProperty); }
            set { SetValue(MaxFontSizeProperty, value); }
        }

        public static readonly DependencyProperty MaxFontSizeProperty =
            DependencyProperty.Register("MaxFontSize", typeof(double), typeof(AutoScaleTextBlock), new FrameworkPropertyMetadata(100.0, FrameworkPropertyMetadataOptions.AffectsMeasure));

        public double MinFontSize
        {
            get { return (double)GetValue(MinFontSizeProperty); }
            set { SetValue(MinFontSizeProperty, value); }
        }

        public static readonly DependencyProperty MinFontSizeProperty =
            DependencyProperty.Register("MinFontSize", typeof(double), typeof(AutoScaleTextBlock), new UIPropertyMetadata(10.0));

        public double RealFontSize
        {
            get { return (double)GetValue(RealFontSizePropertyKey.DependencyProperty); }
            set { SetValue(RealFontSizePropertyKey.DependencyProperty, value); }
        }

        private static readonly DependencyPropertyKey RealFontSizePropertyKey =
            DependencyProperty.RegisterReadOnly("RealFontSize", typeof(double), typeof(AutoScaleTextBlock), new UIPropertyMetadata(0.0));

        public static readonly DependencyProperty RealFontSizeProperty = RealFontSizePropertyKey.DependencyProperty;

        public string Text
        {
            get { return (string)GetValue(TextProperty); }
            set { SetValue(TextProperty, value); }
        }

        public static readonly DependencyProperty TextProperty = TextBlock.TextProperty.AddOwner(typeof(AutoScaleTextBlock));
    }
}