namespace Brjnk.Common.UI
{
    using System.Windows;
    using System.Windows.Controls;

    /// <summary>
    /// ozen canvas s odlinm zpsobem urovn pozice element.
    /// </summary>
    public class Canvas : Panel
    {
        protected override System.Windows.Size ArrangeOverride(System.Windows.Size finalSize)
        {
            foreach (UIElement ch in this.InternalChildren)
            {
                var desired = ch.DesiredSize;
                double centerX = CalculateCenter(desired.Width, GetCenterX(ch));
                double centerY = CalculateCenter(desired.Height, GetCenterY(ch));
                ch.Arrange(new Rect(GetX(ch) - centerX, GetY(ch) - centerY,
                    desired.Width, desired.Height));
            }
            return finalSize;
        }

        protected override System.Windows.Size MeasureOverride(System.Windows.Size availableSize)
        {
            foreach (UIElement ch in this.InternalChildren)
            {
                ch.Measure(new Size(double.PositiveInfinity, double.PositiveInfinity));
            }
            return new System.Windows.Size(0, 0);
        }

        private double CalculateCenter(double length, ElementCenter elementCenter)
        {
            double value = elementCenter.Value;
            if (elementCenter.Relative) value = value * length;
            if (elementCenter.FromOppositeSide) value = length - value;
            return value;
        }

        #region AttachedProperties

        public static readonly DependencyProperty CenterXProperty =
              DependencyProperty.RegisterAttached("CenterX", typeof(ElementCenter), typeof(Canvas),
              new FrameworkPropertyMetadata() { AffectsParentArrange = true });

        public static readonly DependencyProperty CenterYProperty =
             DependencyProperty.RegisterAttached("CenterY", typeof(ElementCenter), typeof(Canvas),
             new FrameworkPropertyMetadata() { AffectsParentArrange = true });

        public static readonly DependencyProperty XProperty =
             DependencyProperty.RegisterAttached("X", typeof(double), typeof(Canvas),
             new FrameworkPropertyMetadata() { AffectsParentArrange = true });

        public static readonly DependencyProperty YProperty =
            DependencyProperty.RegisterAttached("Y", typeof(double), typeof(Canvas),
            new FrameworkPropertyMetadata() { AffectsParentArrange = true });

        public static ElementCenter GetCenterX(DependencyObject obj)
        {
            return (ElementCenter)obj.GetValue(CenterXProperty);
        }

        public static ElementCenter GetCenterY(DependencyObject obj)
        {
            return (ElementCenter)obj.GetValue(CenterYProperty);
        }

        public static double GetX(DependencyObject obj)
        {
            return (double)obj.GetValue(XProperty);
        }

        public static double GetY(DependencyObject obj)
        {
            return (double)obj.GetValue(YProperty);
        }

        public static void SetCenterX(DependencyObject obj, ElementCenter value)
        {
            obj.SetValue(CenterXProperty, value);
        }

        public static void SetCenterY(DependencyObject obj, ElementCenter value)
        {
            obj.SetValue(CenterYProperty, value);
        }

        public static void SetX(DependencyObject obj, double value)
        {
            obj.SetValue(XProperty, value);
        }

        public static void SetY(DependencyObject obj, double value)
        {
            obj.SetValue(YProperty, value);
        }

        #endregion AttachedProperties
    }
}