namespace Brjnk.Common.UI
{
    using System;
    using System.ComponentModel;
    using System.Linq;
    using System.Text.RegularExpressions;

    /// <summary>
    /// Definuje sted elementu. Tento bod elmentu bude umstn na souadnice Canvas.X, Canvas.Y
    /// </summary>
    [TypeConverter(typeof(ElementCenterConverter))]
    public struct ElementCenter
    {
        public ElementCenter(bool relative, bool fromOppositeSide, double value)
            : this()
        {
            Relative = relative;
            FromOppositeSide = fromOppositeSide;
            Value = value;
        }

        public bool Relative { get; private set; }

        public bool FromOppositeSide { get; private set; }

        public double Value { get; private set; }

        public static ElementCenter Parse(string input, IFormatProvider formatProvider)
        {
            bool relative = false, oppositeSide = false;
            double value;

            //odebrat bl znaky
            input = Regex.Replace(input, @"\s", "");
            if (string.IsNullOrEmpty(input)) return new ElementCenter();

            if (input.First() == '_')
            {
                oppositeSide = true;
                input = input.Substring(1, input.Length - 1);
            }
            if (input.Last() == '%')
            {
                relative = true;
                input = input.Substring(0, input.Length - 1);
            }

            bool succes = double.TryParse(input, System.Globalization.NumberStyles.Any, formatProvider, out value);
            if (succes == false) throw new FormatException();
            if (relative) value /= 100;
            return new ElementCenter(relative, oppositeSide, value);
        }

        public override string ToString()
        {
            return ToString(System.Globalization.CultureInfo.CurrentCulture);
        }

        public string ToString(IFormatProvider format)
        {
            return string.Format("{0}{1}{2}",
                  FromOppositeSide ? "_" : string.Empty,
                  Relative ? (Value * 100).ToString(format) : Value.ToString(format),
                  Relative ? "%" : string.Empty);
        }
    }
}