namespace Brjnk.Common.UI
{
    using System;
    using System.Runtime.InteropServices;

    /// <summary>
    /// Pevody mezi DIP a bnmi dlkovmi jednotkami.
    /// Tato tda vol Win Api pro zjitn aktulnho DPI.
    /// </summary>
    public static class DIPHelper
    {
        public static double GetDipFromInch(double inches)
        {
            return inches * 96;
        }

        public static double GetDipFromCm(double centimetres)
        {
            return (centimetres / 2.54) * 96;
        }

        public static double GetInchFromDip(double dip)
        {
            return dip / 96;
        }

        public static double GetCmFromDip(double dip)
        {
            return (dip / 96) * 2.54;
        }

        private static Dpi? dpi;

        public static double GetPixelCountX(double dip)
        {
            PrepareDpi();
            return dip / 96 * dpi.Value.X;
        }

        public static double GetPixelCountY(double dip)
        {
            PrepareDpi();
            return dip / 96 * dpi.Value.Y;
        }

        public static double GetDpiFromPixelX(double pixel)
        {
            PrepareDpi();
            return pixel / dpi.Value.X * 96;
        }

        public static double GetDpiFromPixelY(double pixel)
        {
            PrepareDpi();
            return pixel / dpi.Value.Y * 96;
        }

        private static void PrepareDpi()
        { if (dpi.HasValue == false) GetDpi(); }

        #region GetSystemDpi

        /// <summary>
        /// Logical pixels inch in X
        /// </summary>
        private const int LOGPIXELSX = 88;

        /// <summary>
        /// Logical pixels inch in Y
        /// </summary>
        private const int LOGPIXELSY = 90;

        [DllImport("user32.dll")]
        private static extern IntPtr GetDC(IntPtr hWnd);

        [DllImport("user32.dll")]
        private static extern int ReleaseDC(IntPtr hWnd, IntPtr hDc);

        [DllImport("gdi32.dll")]
        private static extern int GetDeviceCaps(IntPtr hdc, int index);

        public static Dpi GetDpi()
        {
            var dc = GetDC(IntPtr.Zero);
            var x = GetDeviceCaps(dc, LOGPIXELSX);
            var y = GetDeviceCaps(dc, LOGPIXELSY);
            int r = ReleaseDC(IntPtr.Zero, dc);
            System.Diagnostics.Debug.Assert(r == 1);
            dpi = new Dpi(x, y);
            return dpi.Value;
        }

        #endregion GetSystemDpi
    }

    public struct Dpi
    {
        public Dpi(double dpiX, double dpiY)
            : this()
        {
            this.X = dpiX;
            this.Y = dpiY;
        }

        public double X { get; private set; }

        public double Y { get; private set; }
    }
}