namespace Brjnk.Common.UI
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.ComponentModel;

    /// <summary>
    /// </summary>
    public class ObjectNode
    {
        private ObjectNode(string name, object obj, int depth)
        {
            Name = name;
            if ((object)obj == null) return;
            instance = obj;
            Type = obj.GetType();
        }

        public ObjectNode(string name, object obj) : this(name, obj, 0) { }

        private object instance;

        private List<ObjectNode> children = null;

        public List<ObjectNode> Children
        {
            get
            {
                if (children == null) ReadObject();
                return children;
            }
        }

        public string Name { get; private set; }

        public string Value { get; private set; }

        public Type Type { get; private set; }

        private void ReadObject()
        {
            children = new List<ObjectNode>();

            if ((object)instance == null)
            {
                Value = "<null>";
                return;
            }
            if (instance is string)
            {
                Value = instance.ToString();
                return;
            }

            var converter = TypeDescriptor.GetConverter(instance);

            //Textov hodnota aktulnho uzlu
            if (converter.CanConvertTo(typeof(string)))
                Value = converter.ConvertTo(instance, typeof(string)) as string;
            else
                Value = instance.ToString();

            //Vyhledn potomk aktulnho uzlu
            var type = instance.GetType();
            foreach (var property in type.GetProperties())
            {
                if (property.GetIndexParameters().Length == 0)
                    children.Add(new ObjectNode(property.Name, property.GetValue(instance, null)));
                else
                    children.Add(new ObjectNode(property.Name, "Evaluation is not supported"));
            }

            //Enumerace vech poloek, pokud se jedn o Enumerable
            var enumerable = instance as IEnumerable;
            if (enumerable != null)
            {
                int i = 0;
                foreach (var item in enumerable)
                {
                    children.Add(new ObjectNode("[" + i + "]", item));
                    i++;
                }
            }
        }
    }
}