namespace Brjnk.Common
{
    using System;
    using System.ComponentModel;
    using Brjnk.Common.Extensions;
    using Brjnk.Common.ValueTypes;

    /// <summary>
    /// </summary>
    [TypeConverter(typeof(StringValueConverter<Acceleration>))]
    public struct Acceleration : IPhysicalQuantity
    {
        private double value; // m/s^-2

        public Acceleration(double value)
        {
            this.value = value;
        }

        public double Value { get { return value; } }

        /// <summary>
        /// Implicitn konverze vrac zrychlen v m/s*s
        /// </summary>
        public static implicit operator double(Acceleration a)
        {
            return a.value;
        }

        public static implicit operator Acceleration(double a)
        {
            return new Acceleration(a);
        }

        public static object Parse(string p)
        {
            return Parser.Parse(p);
        }

        public override string ToString()
        {
            switch (UnitsSettings.Instance.Acceleration)
            {
                case AccelerationUnit.MetersPerSecond2:
                    return string.Format("{0:f02} m" + StringExtension.DotOperator + "s" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptTwo, value);
                case AccelerationUnit.KilemetersPerHour2:
                    return string.Format("{0:f02} km" + StringExtension.DotOperator + "h" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptTwo, value * 60 * 60 * 60 * 60 / 1000);
                default:
                    throw new ArgumentException("Nepodporovan jednotka zrychlen.");
            }
        }

        static Acceleration()
        {
            Parser = new ValueParser();
            Parser.AddUnit("m*s^-2", 1);
            Parser.AddUnit("km*h^-2", 1000 / (60 * 60 * 60 * 60));
            Parser.AddUnit("m" + StringExtension.DotOperator + "s^-2", 1);
            Parser.AddUnit("km" + StringExtension.DotOperator + "h^-2", 1000 / (60 * 60 * 60 * 60));
            Parser.AddUnit("m/s^2", 1);
            Parser.AddUnit("km/h^2", 1000 / (60 * 60 * 60 * 60));

            Parser.AddUnit("m*s" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptTwo, 1);
            Parser.AddUnit("km*h" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptTwo, 1000 / (60 * 60 * 60 * 60));
            Parser.AddUnit("m" + StringExtension.DotOperator + "s" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptTwo, 1);
            Parser.AddUnit("km" + StringExtension.DotOperator + "h" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptTwo, 1000 / (60 * 60 * 60 * 60));
            Parser.AddUnit("m/s" + StringExtension.SuperscriptTwo, 1);
            Parser.AddUnit("km/h" + StringExtension.SuperscriptTwo, 1000 / (60 * 60 * 60 * 60));
        }

        public static readonly ValueParser Parser;

        public double GetValue()
        {
            return value;
        }
    }

    public enum AccelerationUnit
    {
        MetersPerSecond2,
        KilemetersPerHour2
    }
}