using System;
using System.ComponentModel;
using System.Text.RegularExpressions;
using Brjnk.Common.ValueTypes;

namespace Brjnk.Common
{
    [TypeConverter(typeof(StringValueConverter<GpsCoordinates>))]
    public struct GpsCoordinates : IFormattable
    {
        private const char North = 'N';
        private const char East = 'E';
        private const char West = 'W';
        private const char South = 'S';

        private double latitude;
        private double longtitude;

        public GpsCoordinates(double latitude, double longtitude)
            : this()
        {
            this.latitude = latitude;
            this.longtitude = longtitude;
        }

        public GpsCoordinates(double latDeg, double latMin, double latSec, double lonDeg, double lonMin, double lonSec, bool south, bool west)
        {
            this.latitude = latDeg + latMin / 60 + latSec / 3600;
            this.longtitude = lonDeg + lonMin / 60 + lonSec / 3600;
            if (south) this.latitude *= -1;
            if (west ) this.longtitude *= -1;
        }

        public static Distance operator -(GpsCoordinates coordinate1, GpsCoordinates coordinate2)
        {
            return new Distance(coordinate1, coordinate2);
        }

        public int LatDegree
        {
            get { return (int)latitude; }
        }

        public double LatTotalDegree
        {
            get { return latitude; }
        }

        public int LatMinutes
        {
            get { return (int)LatTotalMinutes; }
        }

        public double LatTotalMinutes
        {
            get { return (latitude - LatDegree) * 60; }
        }

        public double LatTotalSeconds
        {
            get { return ((((latitude) - LatDegree) * 60) - LatMinutes) * 60; }
        }

        public int LonDegree
        {
            get { return (int)longtitude; }
        }

        public double LonTotalDegree
        {
            get { return longtitude; }
        }

        public int LonMinutes
        {
            get { return (int)LonTotalMinutes; }
        }

        public double LonTotalMinutes
        {
            get { return (longtitude - LonDegree) * 60; }
        }

        public double LonTotalSeconds
        {
            get { return ((((longtitude) - LonDegree) * 60) - LonMinutes) * 60; }
        }

        public CardinalDirection Area
        {
            get
            {
                CardinalDirection functionReturnValue = default(CardinalDirection);
                functionReturnValue = 0;
                if (latitude >= 0)
                    functionReturnValue = (CardinalDirection)((int)functionReturnValue | 1);
                if (longtitude >= 0)
                    functionReturnValue = (CardinalDirection)((int)functionReturnValue | 2); ;
                return functionReturnValue;
            }
            private set
            {
                if (((int)value & 1) == 1)
                {
                    latitude = Math.Abs(latitude);
                }
                else
                {
                    latitude = -Math.Abs(latitude);
                }

                if (((int)value & 2) == 2)
                {
                    longtitude = Math.Abs(longtitude);
                }
                else
                {
                    longtitude = -Math.Abs(longtitude);
                }
            }
        }

        #region Parsing

        //star - nepodporuje psmeno za souadnic
        //private const string regEx1 = //regEx pro zpis se znaky  " '
        //    @"^(?<H1>N|S|\+|-)?\s*(?<D1>\d{0,3}(\.|,)?\d*)\s*\s*((?<M1>\d{0,2}(\.|,)?\d*)\s*'\s*)?((?<S1>\d{0,2}(\.|,)?\d*)\s*(""|'')\s*)?(\s|;|,)*(?<H2>W|E|\+|-)?\s*(?<D2>\d{0,3}(\.|,)?\d*)\s*\s*((?<M2>\d{0,2}(\.|,)?\d*)'\s*\s*)?((?<S2>\d{0,2}(\.|,)?\d*)\s*(""|'')\s*)?$";

        private const string regEx1 =//regEx pro zpis se znaky  " '
            @"^(?<H1>N|S|\+|-)?\s*(?<D1>\d{0,3}(\.|,)?\d*)\s*\s*((?<M1>\d{0,2}(\.|,)?\d*)\s*'\s*)?((?<S1>\d{0,2}(\.|,)?\d*)\s*(""|'')\s*)?(?<H1b>N|S|\+|-)?\s*(\s|;|,)*(?<H2>W|E|\+|-)?\s*(?<D2>\d{0,3}(\.|,)?\d*)\s*\s*((?<M2>\d{0,2}(\.|,)?\d*)'\s*\s*)?((?<S2>\d{0,2}(\.|,)?\d*)\s*(""|'')\s*)?(?<H2b>W|E|\+|-)?$";

        private const string regEx2 = //regEx pro zpis BEZ znak  " '; pouv nap. MapSource
            @"^(?<H1>N|S|\+|-)?\s*(?<D1>\d{0,3}(\.|,)?\d*)\s*((?<M1>\d{0,2}(\.|,)?\d*)\s*)?((?<S1>\d{0,2}(\.|,)?\d*)\s*)?(?<H2>W|E|\+|-)?\s*(?<D2>\d{0,3}(\.|,)?\d*)\s*((?<M2>\d{0,2}(\.|,)?\d*)\s*)?((?<S2>\d{0,2}(\.|,)?\d*)\s*)?$";

        public static bool TryParse(string input, out GpsCoordinates result)
        {
            result = default(GpsCoordinates);
            if (string.IsNullOrEmpty(input)) { return false; }
            input = input.Trim();
            if (string.IsNullOrEmpty(input)) return false;
            Regex r = new Regex(regEx1, RegexOptions.IgnoreCase);
            GpsCoordinates value;
            if (ProcessRegEx(r, input, out value)) { result = value; return true; }
            r = new Regex(regEx2, RegexOptions.IgnoreCase);
            if (ProcessRegEx(r, input, out value)) { result = value; return true; }
            return false;
        }

        public static GpsCoordinates Parse(string input)
        {
            GpsCoordinates result;
            if (!TryParse(input, out result)) throw new FormatException("Nepodporovan formt souadnic");
            return result;
        }

        private static bool ProcessRegEx(Regex r, string input, out GpsCoordinates value)
        {
            var match = r.Match(input);
            if (match.Success)
            {
                value = CreateFromRegex(match);
                return true;
            }
            else
            {
                value = default(GpsCoordinates);
                return false;
            }
        }

        //pevede Regex Match na souadnice
        //H1 - hemisfra (-, S) pro jin polokouli
        //H1b- hemisfra uveden za selm
        //D1 - latitude stupn
        //M1 - latitude minuty
        //S1 - latitude vteiny
        //d1 - latitude stupn (st za desetinou tekou)
        //m1 - latitude minuty (st za desetinou tekou)
        //s1 - latitude vteiny (st za desetinou tekou)
        //longtitude m 2 (nap D2, M2...)
        private static GpsCoordinates CreateFromRegex(Match match)
        {
            string hemisphere1 = match.Groups["H1"].Value;
            string hemisphere2 = match.Groups["H2"].Value;
            if (string.IsNullOrEmpty(hemisphere1)) hemisphere1 = match.Groups["H1b"].Value;
            if (string.IsNullOrEmpty(hemisphere2)) hemisphere2 = match.Groups["H2b"].Value;
            double d1 = ToDbl(match, "D1");
            double m1 = ToDbl(match, "M1");
            double s1 = ToDbl(match, "S1");
            double d2 = ToDbl(match, "D2");
            double m2 = ToDbl(match, "M2");
            double s2 = ToDbl(match, "S2");
            bool south = (hemisphere1 == South.ToString() || hemisphere1 == "-");
            bool west = (hemisphere2 == West.ToString() || hemisphere2 == "-");
            return new GpsCoordinates(d1, m1, s1, d2, m2, s2, south, west);
        }

        private static double ToDbl(Match match, string groupName)
        {
            var group = match.Groups[groupName];
            if (!group.Success) return 0;
            if (string.IsNullOrEmpty(group.Value)) return 0;
            double retVal;
            if (!double.TryParse(group.Value, out retVal))
            {
                if (!double.TryParse(group.Value, System.Globalization.NumberStyles.Any, System.Globalization.CultureInfo.InvariantCulture, out retVal))
                {
                    return 0;
                }
            }
            return retVal;
        }

        #endregion Parsing

        #region ToString

        public override string ToString() { return ToString(CoordinationFormat.Seconds); }

        public string ToString(CoordinationFormat format)
        {
            switch (format)
            {
                case CoordinationFormat.Degrees:
                    return ToString(format, " ", 5);
                case CoordinationFormat.Minutes:
                    return ToString(format, " ", 4);
                case CoordinationFormat.Seconds:
                    return ToString(format, " ", 2);
                default:
                    throw new ArgumentOutOfRangeException("format");
            }
        }

        public string ToString(CoordinationFormat format, string separator, int decimalPlacesCount)
        {
            { return ToString(format, false, separator, decimalPlacesCount, System.Globalization.CultureInfo.CurrentCulture); }
        }

        public string ToString(CoordinationFormat format, bool useSign, string separator, int decimalPlacesCount, IFormatProvider formatProvider)
        {
            string doubleFormat = "F" + decimalPlacesCount;
            switch (format)
            {
                case CoordinationFormat.Degrees:
                    return string.Format("{0}{1}{2}{3}{4}",
                        GetLatCardinalDirectionChar(useSign),
                        Math.Abs(LatTotalDegree).ToString(doubleFormat, formatProvider),
                        separator,
                        GetLonCardinalDirectionChar(useSign),
                        Math.Abs(LonTotalDegree).ToString(doubleFormat, formatProvider));
                case CoordinationFormat.Minutes:
                    return string.Format("{0}{1} {2}'{3}{4}{5} {6}'",
                        GetLatCardinalDirectionChar(useSign),
                        Math.Abs(LatDegree),
                        Math.Abs(LatTotalMinutes).ToString(doubleFormat, formatProvider),
                        separator,
                        GetLonCardinalDirectionChar(useSign),
                        Math.Abs(LonDegree),
                        Math.Abs(LonTotalMinutes).ToString(doubleFormat, formatProvider));
                case CoordinationFormat.Seconds:

                    return string.Format("{0}{1} {2}' {3}\"{4}{5}{6} {7}' {8}\"",
                        GetLatCardinalDirectionChar(useSign),
                        Math.Abs(LatDegree),
                        Math.Abs(LatMinutes),
                        Math.Abs(LatTotalSeconds).ToString(doubleFormat, formatProvider),
                        separator,
                        GetLonCardinalDirectionChar(useSign),
                        Math.Abs(LonDegree),
                        Math.Abs(LonMinutes),
                        Math.Abs(LonTotalSeconds).ToString(doubleFormat, formatProvider));
                default:
                    throw new ArgumentException("Unsupported coordination format", "format");
            }
        }

        /// <summary>
        /// Return char for cardinal direction (N or S)
        /// </summary>
        private char GetLatCardinalDirectionChar(bool useSign)
        {
            if (useSign)
            {
                if (latitude >= 0) return '+';
                else return '-';
            }
            else
            {
                if (((int)Area & 1) == 1)
                    return North;
                return South;
            }
        }

        /// <summary>
        /// Return char for cardinal direction (E or W)
        /// </summary>
        private char GetLonCardinalDirectionChar(bool useSign)
        {
            if (useSign)
            {
                if (longtitude >= 0) return '+';
                else return '-';
            }
            else
            {
                if (((int)Area & 2) == 2)
                    return East;
                return West;
            }
        }

        /// <summary>
        /// </summary>
        /// <param name="format">
        /// Formtovac etzec
        /// Prvn znak - s=seconds; m=minutes; d=degrees
        /// Druh znak - poet desetinnch mst
        /// Tet znak - c=char (pouije N, S, W, E); s=znamnko (pouije +, -)
        /// Nsledujc znaky - pouij se pro oddlen zempisn ky a dlky
        /// </param>
        public string ToString(string format, IFormatProvider formatProvider)
        {
            format = format.ToLower();
            char coordFormat = 's'; //formt souadnice
            int decimalPlacesCount = 1; //poet desetinch mst
            string separator = " "; //znak mezi zempisnou kou a dlkou
            bool useSign = false; //true, pokud se m pout znamnko (+,-) msto psmena (N,W,S,E)
            if (format != "g")
            {
                if (format.Length >= 1)
                    coordFormat = format[0];
                if (format.Length >= 2)
                    decimalPlacesCount = int.Parse(format[1].ToString());
                if (format.Length >= 3)
                    if (format[2] == 's') useSign = true;
                if (format.Length >= 4)
                    separator = format.Substring(3);
            }

            switch (coordFormat)
            {
                case 'm':
                    return ToString(CoordinationFormat.Minutes, useSign, separator, decimalPlacesCount, formatProvider);
                case 's':
                    return ToString(CoordinationFormat.Seconds, useSign, separator, decimalPlacesCount, formatProvider);
                default:
                    return ToString(CoordinationFormat.Degrees, useSign, separator, decimalPlacesCount, formatProvider);
            }
        }

        #endregion ToString

        public static bool operator ==(GpsCoordinates coordinates1, GpsCoordinates coordinates2)
        {
            return coordinates1.latitude == coordinates2.latitude && coordinates1.longtitude == coordinates2.longtitude;
        }

        public static bool operator !=(GpsCoordinates coordinates1, GpsCoordinates coordinates2)
        {
            return !(coordinates1 == coordinates2);
        }

        public static explicit operator System.Windows.Point(GpsCoordinates gps)
        {
            return new System.Windows.Point(gps.LonTotalDegree, gps.LatTotalDegree);
        }

        public static explicit operator GpsCoordinates(System.Windows.Point point)
        {
            return new GpsCoordinates(point.Y, point.X);
        }

        public override bool Equals(object obj)
        {
            return this == (GpsCoordinates)obj;
        }

        public override int GetHashCode()
        {
            return this.latitude.GetHashCode() + this.longtitude.GetHashCode();
        }
    }

    public enum CoordinationFormat
    {
        Degrees,
        Minutes,
        Seconds
    }

    public enum CardinalDirection
    {
        SW = 0,
        NW = 1,
        SE = 2,
        NE = 3
        //FIRST BIT - north=1; south=0
        //SECOND BIT - west=0; east=1
    }
}