namespace Brjnk.Common
{
    using System;
    using System.ComponentModel;
    using Brjnk.Common.Extensions;
    using Brjnk.Common.ValueTypes;

    /// <summary>
    /// </summary>
    [TypeConverter(typeof(StringValueConverter<Speed>))]
    public struct Speed : IPhysicalQuantity
    {
        private double value; // m/s

        public Speed(double value)
        {
            this.value = value;
        }

        public double Value { get { return value; } }

        /// <summary>
        /// Implicitn konverze vrac rychlost v m/s
        /// </summary>
        public static implicit operator double(Speed s)
        {
            return s.value;
        }

        public static implicit operator Speed(double a)
        {
            return new Speed(a);
        }

        public override string ToString()
        {
            switch (UnitsSettings.Instance.Speed)
            {
                case SpeedUnit.MetersPerSecond:
                    return string.Format("{0:f02} m/s", value);
                case SpeedUnit.KilometersPerHour:
                    return string.Format("{0:f02} km/h", value * 3.6);
                default:
                    throw new ArgumentException("Nepodporovan jednotka rychlosti.");
            }
        }

        static Speed()
        {
            Parser = new ValueParser();
            Parser.AddUnit("m*s^-1", 1);
            Parser.AddUnit("km*h^-1", 1 / 3.6);
            Parser.AddUnit("m" + StringExtension.DotOperator + "s^-1", 1);
            Parser.AddUnit("km" + StringExtension.DotOperator + "h^-1", 1000 / (60 * 60 * 60 * 60));
            Parser.AddUnit("m/s", 1);
            Parser.AddUnit("km/h", 1 / 3.6);
            Parser.AddUnit("m/s^1", 1);
            Parser.AddUnit("km/h^1", 1 / 3.6);

            Parser.AddUnit("m*s" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptOne, 1);
            Parser.AddUnit("km*h" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptOne, 1 / 3.6);
            Parser.AddUnit("m" + StringExtension.DotOperator + "s" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptOne, 1);
            Parser.AddUnit("km" + StringExtension.DotOperator + "h" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptOne, 1000 / (60 * 60 * 60 * 60));
            Parser.AddUnit("m/s", 1);
            Parser.AddUnit("km/h", 1 / 3.6);
            Parser.AddUnit("m/s" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptOne, 1);
            Parser.AddUnit("km/h" + StringExtension.SuperscriptMinus + StringExtension.SuperscriptOne, 1 / 3.6);
        }

        public static readonly ValueParser Parser;

        public double GetValue()
        {
            return value;
        }
    }

    public enum SpeedUnit
    {
        MetersPerSecond,
        KilometersPerHour
    }
}