namespace Brjnk.Common
{
    using System;
    using System.Collections.Generic;
    using Brjnk.Common.Extensions;

    /// <summary>
    /// </summary>
    public class ValueParser
    {
        private bool isSealed = false;

        private List<Unit> units = new List<Unit>();

        public void AddUnit(string name, double coefficient)
        {
            if (isSealed) throw new InvalidOperationException("ValueParser je ji zapeetn");
            units.Add(new Unit(name, coefficient));
        }

        public void AddUnit(string name, Func<double, double> convertFunc)
        {
            if (isSealed) throw new InvalidOperationException("ValueParser je ji zapeetn");
            units.Add(new Unit(name, convertFunc));
        }

        public void Seal()
        {
            isSealed = true;
        }

        public bool TryParse(string input, out double result)
        {
            input = input.RemoveWhiteChars();
            input = input.Replace(System.Globalization.CultureInfo.CurrentCulture.NumberFormat.NumberGroupSeparator, string.Empty);
            if (double.TryParse(input, out result)) return true;
            foreach (var unit in units)
            {
                if (input.EndsWith(unit.Name, StringComparison.InvariantCultureIgnoreCase))
                {
                    var valueText = input.Substring(0, input.Length - unit.Name.Length);

                    if (double.TryParse(valueText, out result))
                    {
                        if (unit.ConvertFunc != null)
                        {
                            result = unit.ConvertFunc(result);
                        }
                        else
                        {
                            result = result * unit.Coefficient;
                        }
                        return true;
                    }
                    else continue;
                }
            }
            result = 0;
            return false;
        }

        public double Parse(string input)
        {
            double result;
            var success = TryParse(input, out result);
            if (success == false) throw new FormatException("Chybn formt.");
            return result;
        }

        private struct Unit
        {
            public Unit(string name, double coefficient)
                : this()
            {
                Name = name;
                Coefficient = coefficient;
            }

            public Unit(string name, Func<double, double> convertFunc)
                : this()
            {
                Name = name;
                ConvertFunc = convertFunc;
            }

            public string Name { get; private set; }

            public double Coefficient { get; private set; }

            public Func<double, double> ConvertFunc { get; private set; }
        }
    }
}