using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;

namespace Brjnk.Common
{
    [ContentProperty("Templates")]
    public class GenericTemplateSelector : DataTemplateSelector
    {
        private List<TemplateDefinition> templates = new List<TemplateDefinition>();

        public List<TemplateDefinition> Templates { get { return templates; } set { templates = value; } }

        public override DataTemplate SelectTemplate(object item, DependencyObject container)
        {
            if ((object)item == null) return null;
            var type = item.GetType();
            foreach (var def in templates)
            {
                if (def.ObjectType == type) return def.Template;
                else if (def.ObjectType.IsAssignableFrom(type) && def.IncludeInherited) return def.Template;
            }
            return null;
        }
    }

    [ContentProperty("Template")]
    public class TemplateDefinition
    {
        private bool includeInherited=true;

        public bool IncludeInherited
        {
            get { return includeInherited; }
            set { includeInherited = value; }
        }
        

        public Type ObjectType { get; set; }

        public DataTemplate Template { get; set; }
    }
}