namespace Brjnk.Common
{
    using System;
    using System.Linq;
    using System.Reflection;
    using System.Windows;

    /// <summary>
    /// Vchoz tda pro objekty, kter obsahuj read-only dependency property, kter maj bt nastaviteln zvenku (objektem kter tento objekt vytvoil).
    /// Pi voln konstruktoru je vrcen delegt na metodu umoujc zmnu libovoln read-only dependency property.
    /// </summary>
    public abstract class UpdateableObject : DependencyObject
    {
        private bool allProperties;

        /// <param name="allProperties">True, pokud maj bt nastaviteln vechny vlastnosti, nejenom vlastnosti ozanen "OutsideUpdateableAttribtue"</param>
        public UpdateableObject(out SetPropertyHandler updater, bool allProperties)
        {
            updater = new SetPropertyHandler(Update);
            this.allProperties = allProperties;
        }

        public void Update(DependencyProperty dependencyProperty, object newValue)
        {
            //nate vechny dependency property tohoto typu
            ReadAllProperties();
            var property = properties.Where(p => object.ReferenceEquals(p.DependencyProperty, dependencyProperty)).FirstOrDefault();
            if ((object)property == null)
                throw new Exception("K pedan DependencyProperty nebyl nalezen DependencyPropertyKey. Ovte zda je pslun DependencyPropertyKey lenem tto tdy jako static, readonly field. Ovte zda je pslun DependencyPropertyKey oznaen atributem OutsideUpdateable.");

            bool cancel = false;
            PropertyChanging(property, newValue, ref cancel);
            if (cancel) return;
            this.SetValue(property, newValue);
        }

        private bool IsEqualOrSubclass(Type type, Type type2)
        {
            return type == type2 || type.IsSubclassOf(type2);
        }

        [ThreadStatic()]
        private DependencyPropertyKey[] properties = null;

        private void ReadAllProperties()
        {
            if (properties != null) return;

            var query = this.GetType().GetFields(BindingFlags.NonPublic | BindingFlags.Static | BindingFlags.FlattenHierarchy)
                    .Where(t => t.FieldType == typeof(DependencyPropertyKey) && t.IsInitOnly);
            if (allProperties == false) query = query.Where(t => t.GetCustomAttributes(typeof(OutsideUpdateableAttribute), true).Count() > 0);
            properties = query.Select(f => (DependencyPropertyKey)f.GetValue(null)).ToArray();
        }

        protected virtual void PropertyChanging(DependencyPropertyKey dependencyPropertyKey, object newValue, ref bool cancel)
        {
        }
    }

    public delegate void SetPropertyHandler(DependencyProperty dependencyProperty, object newValue);
}