namespace Brjnk.CoordinateSystem2D
{
    using System;
    using System.Linq;
    using System.Windows;

    /// <summary>
    /// asov osa vyobrazujc dobu. Zkladn jednotka je sekunda.
    /// Je-li nasteveno IsDateTime na true, formtuje popisky jako datum a as.
    /// </summary>
    public class DateTimeAxis : AxisDefinition
    {
        public override GetAxisLabelsResult GetAxisLabels(double from, double to, double minSpacing)
        {
            var r = CalculateSteps(minSpacing);
            return new GetAxisLabelsResult(this.Caption, this.GenerateLabels(from, to, r).ToArray());
        }

        private CalculateStepsResult CalculateSteps(double minSpacing)
        {
            var result = new CalculateStepsResult();
            result.LargeZero = true;
            if (minSpacing <= 1)
            {
                result.stepLarge = 60;
                result.stepBig = Math.Pow(10, Math.Ceiling(Math.Log10(minSpacing)));
                result.stepMedium = result.stepBig / 2;
                result.stepSmall = result.stepBig / 10;
                result.formatterLarge = new Func<double, string>(FormatDateTime);
                result.formatterBig = new Func<double, string>(FormatSecondCount);
            }
            else if (minSpacing <= 10)
            {
                result.stepLarge = 3600;
                result.stepBig = 10;
                result.stepMedium = 5;
                result.stepSmall = 1;
                result.formatterLarge = new Func<double, string>(FormatDateTime);
                result.formatterBig = new Func<double, string>(FormatTimeFull);
            }
            else if (minSpacing <= 60)
            {
                result.stepLarge = 3600;
                result.stepBig = 60;
                result.stepMedium = 30;
                result.stepSmall = 10;
                result.formatterLarge = new Func<double, string>(FormatDateTime);
                result.formatterBig = new Func<double, string>(FormatTime);
            }
            else if (minSpacing <= 600)
            {
                result.stepLarge = 86400;
                result.stepBig = 600;
                result.stepMedium = 300;
                result.stepSmall = 60;
                result.formatterLarge = new Func<double, string>(FormatDateTime);
                result.formatterBig = new Func<double, string>(FormatTime);
            }
            else if (minSpacing <= 3600)
            {
                result.stepLarge = 86400;
                result.stepBig = 3600;
                result.stepMedium = 1800;
                result.stepSmall = 600;
                result.formatterLarge = new Func<double, string>(FormatDateTime);
                result.formatterBig = new Func<double, string>(FormatTime);
            }
            else if (minSpacing <= 14400)
            {
                result.stepLarge = 86400;
                result.stepBig = 14400;
                result.stepMedium = 3600;
                result.stepSmall = 1800;
                result.formatterLarge = new Func<double, string>(FormatDateTime);
                result.formatterBig = new Func<double, string>(FormatTime);
            }
            else if (minSpacing <= 86400)
            {
                result.stepLarge = 0;
                result.stepBig = 86400;
                result.stepMedium = 43200;
                result.stepSmall = 3600;
                result.formatterLarge = null;
                result.formatterBig = new Func<double, string>(FormatDate);
            }
            else
            {
                result.stepLarge = 0;
                result.stepBig = 86400 * Math.Pow(10, Math.Ceiling(Math.Log10(minSpacing / 86400)));
                result.stepMedium = result.stepBig / 2;
                result.stepSmall = result.stepBig / 10;
                result.formatterLarge = null;
                result.formatterBig = new Func<double, string>(FormatDate);
            }
            return result;
        }

        //as ve tvaru hh:mm.
        private string FormatTime(double value)
        {
            if (!IsValidSecondsCount(value)) return string.Empty;
            if (IsDateTime)
            {
                var d = CreateDateTime(value);

                return d.ToShortTimeString();
            }
            else
            {
                var t = CreateTimeSpan(value);

                return t.ToString(@"hh\:mm");
            }
        }

        //as ve tvaru hh:mm:ss.
        private string FormatTimeFull(double value)
        {
            if (!IsValidSecondsCount(value)) return string.Empty;
            if (IsDateTime)
            {
                var d = CreateDateTime(value);
                return d.ToLongTimeString();
            }
            else
            {
                var t = CreateTimeSpan(value);
                return t.ToString(@"hh\:mm\:ss");
            }
        }

        //Poet sekund na 3 desetin msta. (ss.fff)
        private string FormatSecondCount(double value)
        {
            if (!IsValidSecondsCount(value)) return string.Empty;
            if (IsDateTime)
            {
                var d = CreateDateTime(value);
                return d.ToString(@"ss\.fff");
            }
            else
            {
                var t = CreateTimeSpan(value);
                return t.ToString(@"ss\.fff");
            }
        }

        //Krtk datum a as, as ve tvaru hh:mm
        private string FormatDateTime(double value)
        {
            if (!IsValidSecondsCount(value)) return string.Empty;
            if (IsDateTime)
            {
                var d = CreateDateTime(value);
                return d.ToString(@"g");
            }
            else
            {
                var t = CreateTimeSpan(value);
                return t.ToString(@"g");
            }
        }

        //Krtk datum
        private string FormatDate(double value)
        {
            if (!IsValidSecondsCount(value)) return string.Empty;
            if (IsDateTime)
            {
                var d = CreateDateTime(value);
                return d.ToShortDateString();
            }
            else
            {
                var t = CreateTimeSpan(value);
                return t.ToString(@"%d");
            }
        }

        //private string FormatSecond(double value)
        //{
        //    if (!IsValidSecondsCount(value)) return string.Empty;
        //    if (IsDateTime)
        //    {
        //        var d = CreateDateTime(value);
        //        if (d.Second == 0) return d.ToString("G");
        //        return d.ToString("ss.fff");
        //    }
        //    else
        //    {
        //        var t = CreateTimeSpan(value);
        //        if (t.Seconds == 0) return t.ToString("g");
        //        return t.ToString(@"mm\:ss\.fff");
        //    }
        //}

        //private string FormatMinute(double value)
        //{
        //    if (!IsValidSecondsCount(value)) return string.Empty;
        //    if (IsDateTime)
        //    {
        //        var d = CreateDateTime(value);
        //        if (d.Minute == 0) return d.ToString("g");
        //        return d.ToString("t");
        //    }
        //    else
        //    {
        //        var t = CreateTimeSpan(value);
        //        if (t.Minutes == 0) return t.ToString("g");
        //        return t.ToString(@"hh\:mm");
        //    }
        //}

        //private string FormatHour(double value)
        //{
        //    if (!IsValidSecondsCount(value)) return string.Empty;
        //    if (IsDateTime)
        //    {
        //        var d = CreateDateTime(value);
        //        if (d.Hour == 0) return d.ToString("d");
        //        return d.ToString("t");
        //    }
        //    else
        //    {
        //        var t = CreateTimeSpan(value);
        //        if (t.Hours == 0) return t.ToString("g");
        //        return t.ToString(@"hh\:mm");
        //    }
        //}

        //private string FormatDay(double value)
        //{
        //    if (!IsValidSecondsCount(value)) return string.Empty;
        //    if (IsDateTime)
        //    {
        //        var d = CreateDateTime(value);
        //        return d.ToString("d");
        //    }
        //    else
        //    {
        //        var t = CreateTimeSpan(value);
        //        return t.Days.ToString();
        //    }
        //}

        //private string FormatDays(double value)
        //{
        //    if (!IsValidSecondsCount(value)) return string.Empty;

        //    if (IsDateTime)
        //    {
        //        var d = CreateDateTime(value);
        //        return d.ToString("d");
        //    }
        //    else
        //    {
        //        var t = CreateTimeSpan(value);
        //        return t.Days.ToString();
        //    }
        //}

        private bool IsValidSecondsCount(double value)
        {
            if (value < 0) return false;
            if (value > DateTime.MaxValue.Ticks / (1000 * 1000 * 10)) return false;
            return true;
        }

        public override string GetLabelTextFull(double value)
        {
            if (IsDateTime) return CreateDateTime(value).ToString();
            return CreateTimeSpan(value).ToString();
        }

        private DateTime CreateDateTime(double value)
        {
            return new DateTime((long)(value * 1000 * 1000 * 10));
        }

        private TimeSpan CreateTimeSpan(double value)
        {
            return TimeSpan.FromSeconds(value);
        }

        public bool IsDateTime
        {
            get { return (bool)GetValue(DateProperty); }
            set { SetValue(DateProperty, value); }
        }

        public static readonly DependencyProperty DateProperty =
            DependencyProperty.Register("IsDateTime", typeof(bool), typeof(DateTimeAxis), new UIPropertyMetadata(false, AxisChanged));

        protected override Freezable CreateInstanceCore()
        {
            return new DateTimeAxis();
        }
    }
}