namespace Brjnk.CoordinateSystem2D.Layers
{
    using System;
    using System.Collections.Generic;
    using System.Windows;
    using System.Windows.Media;
    using System.Windows.Media.Animation;
    using System.Windows.Media.Media3D;
    using Brjnk.Common;
    using Brjnk.Common.Extensions;

    /// <summary>
    /// </summary>
    public class Graph
    {
        private const double hitTestDistance = 4;

        private IList<GraphPointInternal> points;

        private ViewInfoBase view;

        public GraphStyle Style { get; set; }

        public GraphDataInfo DataInfo { get; set; }

        public Visual Visual { get { return drawing; } }

        private DrawingVisual drawing = new DrawingVisual();

        /// <summary>
        /// Pepot nov souadnice bod (nap. po zmn MinZ nebo po zmn transforman funkce osy)
        /// Po voln tto metody by mlo bt volno RefreshView pro aktualizaci vzhledu
        /// </summary>
        public void Recalculate(ViewInfoBase view)
        {
            this.view = view;

            if (view == null) return;

            foreach (var p in points)
                p.Transform(view, DataInfo);
        }

        /// <summary>
        /// Aktualizuje pohled (nap. po zoomovn)
        /// </summary>
        public void RefreshView(ViewInfoBase view)
        {
            this.view = view;

            if (view == null) return;

            using (var context = drawing.RenderOpen())
                //Vykreslovn pes StreamGeometry bylo zrueno, protoe StreamGeometry se chov podivn
                //  if (Style.LowColor != Style.HighColor)
                Draw(view, context, true);
            // else
            //Pokud je cel graf vykreslen jednou barvou, me bt pouit rychlej StreamGeometry
            //   Draw(view, context, false);
        }

        /// <summary>
        /// Nastav body grafu. Tato metoda neaktualizuje Visual. Je nutn volat  RefreshVisual pro aktualizaci vizuln podoby.
        /// </summary>
        public void SetPoints(IList<GraphPoint> points)
        {
            this.points = new List<GraphPointInternal>();

            foreach (var p in points)
                this.points.Add(new GraphPointInternal(p, view, DataInfo));
        }

        //ODSTRANIT
        //private void Draw(ViewInfoBase view, DrawingContext context)
        //{
        //    StreamGeometry geometry = new StreamGeometry();

        //    using (var gContext = geometry.Open())
        //    {
        //        Point3D? lastPoint = null;
        //        bool first = true;

        //        foreach (var p in GetGraphPoints(view))
        //        {
        //            //  Point3D  point = ProjectPoint(p, view);
        //            var projected = ProjectPoint(p, view);
        //            if (lastPoint.HasValue)
        //            {
        //                DrawJoinLine(lastPoint.Value, projected, Style, first,animation, gContext);
        //                first = false;
        //            }
        //            lastPoint = projected;

        //            if (p.Original.Interrupt) { first = true; lastPoint = null; };
        //        }
        //    }

        //    DoubleAnimation anim = null;
        //    if (Style.Highlight) anim = CreateAnimation(Style.Thickness);
        //    Pen pen = CreateSolidPen(1, Style, anim);
        //    context.DrawGeometry(null, pen, geometry);
        //}

        private void Draw(ViewInfoBase view, DrawingContext context, bool useGradient)
        {
            StreamGeometry streamGeometry = null;
            StreamGeometryContext geometryContext = null;
            if (!useGradient) { streamGeometry = new StreamGeometry(); geometryContext = streamGeometry.Open(); };

            Point3D? lastPoint = null;
            bool first = true;

            DoubleAnimation animation = null;
            if (Style.Highlight && Style.HighlightByAnimation) { animation = CreateAnimation(GetThickness(Style)); }

            int filtredCount = 0;

            foreach (var point in GetGraphPoints(view))
            {
                filtredCount++;
                Point3D projected = ProjectPoint(point, view);
                if (lastPoint.HasValue)
                {
                    if (useGradient) DrawJoinLineGradient(lastPoint.Value, projected, DataInfo, Style, first, animation, context);
                    else DrawJoinLine(lastPoint.Value, projected, DataInfo, first, geometryContext);
                    first = false;
                }

                lastPoint = projected;
                if (point.Original.Interrupt) { first = true; lastPoint = null; };
            }

            if (!useGradient)
            {
                geometryContext.Close();
                Pen pen = CreateSolidPen(1, Style, animation);
                context.DrawGeometry(null, pen, streamGeometry);
            }
            System.Diagnostics.Debug.WriteLine("Total: " + points.Count + "; Filtred: " + filtredCount);
        }

        private void DrawJoinLine(Point3D point1, Point3D point2, GraphDataInfo dataInfo, bool first, StreamGeometryContext context)
        {
            var joinLine = GetJoinLine(point1, point2, first, dataInfo.Constant);

            if (first)
            {
                context.BeginFigure(joinLine.Item1.As2DPoint(), false, false);
                context.LineTo(joinLine.Item2.As2DPoint(), true, false);
                if (joinLine.Item3.HasValue) context.LineTo(joinLine.Item3.Value.As2DPoint(), true, false);
            }
            else
            {
                context.LineTo(joinLine.Item1.As2DPoint(), true, false);
                context.LineTo(joinLine.Item2.As2DPoint(), true, false);
                if (joinLine.Item3.HasValue) context.LineTo(joinLine.Item3.Value.As2DPoint(), true, false);
            }

            //ODSTRANITPoint p1 = new Point(point1.X, point1.Y);
            //Point p2 = new Point(point2.X, point2.Y);
            //if (style.Constant.HasValue)
            //{
            //    if (style.Constant.Value == AxisOrientation.Horizontal)
            //    {
            //        var corner = new Point(p1.X, p2.Y);
            //        if (first)
            //        {
            //            context.BeginFigure(corner, false, false);
            //            context.LineTo(p2, true, false);
            //        }
            //        else
            //        {
            //            context.LineTo(corner, true, false);
            //            context.LineTo(p2, true, false);
            //        }
            //    }
            //    else
            //    {
            //        var corner = new Point(p2.X, p1.Y);
            //        if (first)
            //        {
            //            context.BeginFigure(corner, false, false);
            //            context.LineTo(p2, true, false);
            //        }
            //        else
            //        {
            //            context.LineTo(corner, true, false);
            //            context.LineTo(p2, true, false);
            //        }
            //    }
            //}
            //else
            //{
            //    if (first) context.BeginFigure(p1, false, false);
            //    context.LineTo(p2, true, false);
            //}
        }

        private void DrawJoinLineGradient(Point3D point1, Point3D point2, GraphDataInfo dataInfo, GraphStyle style, bool first, DoubleAnimation animation, DrawingContext context)
        {
            var joinLine = GetJoinLine(point1, point2, first, dataInfo.Constant);
            var pen = CreatePen(joinLine, style, animation);
            //var p1 = new Point(point1.X, point1.Y);
            //var p2 = new Point(point2.X, point2.Y);
            context.DrawLine(pen, joinLine.Item1.As2DPoint(), joinLine.Item2.As2DPoint());
            if (joinLine.Item3.HasValue) context.DrawLine(pen, joinLine.Item2.As2DPoint(), joinLine.Item3.Value.As2DPoint());
            //ODSTRANITif (style.Constant.HasValue)
            //{
            //    if (style.Constant.Value == AxisOrientation.Horizontal)
            //    {
            //        var corner = new Point(p1.X, p2.Y);
            //        if (first) context.DrawLine(pen, corner, p2);
            //        else
            //        {
            //            context.DrawLine(CreateSolidPen(point1.Z, style, animation), p1, corner);
            //            context.DrawLine(pen, corner, p2);
            //        }
            //    }
            //    else
            //    {
            //        var corner = new Point(p2.X, p1.Y,);
            //        if (first) context.DrawLine(pen, corner, p2);
            //        else
            //        {
            //            context.DrawLine(CreateSolidPen(point1.Z, style, animation), p1, corner);
            //            context.DrawLine(pen, corner, p2);
            //        }
            //    }
            //}
            //else
            //{
            //    context.DrawLine(pen, p1, p2);
            //}
        }

        private Tuple<Point3D, Point3D, Point3D?> GetJoinLine(Point3D point1, Point3D point2, bool first, AxisOrientation? constant)
        {
            if (constant.HasValue == false)
            {
                return new Tuple<Point3D, Point3D, Point3D?>(point1, point2, null);
            }
            else
            {
                Point3D corner;
                if (constant == AxisOrientation.Horizontal) corner = new Point3D(point1.X, point2.Y, point1.Z);
                else corner = new Point3D(point2.X, point1.Y, point1.Z);
                if (first) return new Tuple<Point3D, Point3D, Point3D?>(corner, point2, null);
                else return new Tuple<Point3D, Point3D, Point3D?>(point1, corner, point2);
            }
        }

        private Point3D ProjectPoint(GraphPointInternal p, ViewInfoBase view)
        {
            var relative = view.FromInternalToRelative(new Point(p.Transformed.X, p.Transformed.Y));
            return new Point3D(relative.X, relative.Y, p.Transformed.Z);
        }

        /// <summary>
        /// Vrt body grafu. Souadnice vech bod jsou ji pevedeny do obrazovkovch souadnic.
        /// Vynechv nedleit body pro snen jejich potu a rychlej vykreslovn.
        /// </summary>
        /// <param name="view"></param>
        /// <returns></returns>
        private IEnumerable<GraphPointInternal> GetGraphPoints(ViewInfoBase view)
        {
            return new GraphPointsFilter(points).GetPoints(view, Style.SimplifyDegree);
        }

        private Pen CreateSolidPen(double zValue, GraphStyle style, DoubleAnimation thicknessAnim)
        {
            var color = CalculateColor(style.LowColor, style.HighColor, zValue);
            var pen = new Pen(new SolidColorBrush(color), GetThickness(style));
            if (thicknessAnim != null) pen.BeginAnimation(Pen.ThicknessProperty, thicknessAnim);
            else { pen.Freeze(); } //Freeze() nelze volat na animovan objekty
            return pen;
        }

        private Pen CreatePen(Tuple<Point3D, Point3D, Point3D?> joinLine, GraphStyle style, DoubleAnimation thicknessAnim)
        {
            Brush brush;
            Point3D firstPoint; Point3D lastPoint;
            if (joinLine.Item3.HasValue) { firstPoint = joinLine.Item2; lastPoint = joinLine.Item3.Value; }
            else { firstPoint = joinLine.Item1; lastPoint = joinLine.Item2; }
            if (style.UseGradient)
            {
                var gradient = new LinearGradientBrush();
                gradient.GradientStops.Add(new GradientStop(CalculateColor(style.LowColor, style.HighColor, firstPoint.Z), 0));
                gradient.GradientStops.Add(new GradientStop(CalculateColor(style.LowColor, style.HighColor, lastPoint.Z), 1));
                gradient.StartPoint = firstPoint.As2DPoint();
                gradient.EndPoint = lastPoint.As2DPoint();
                gradient.MappingMode = BrushMappingMode.Absolute;
                brush = gradient;
            }
            else
            {
                brush = new SolidColorBrush(CalculateColor(style.LowColor, style.HighColor, lastPoint.Z));
            }

            var pen = new Pen(brush, GetThickness(style));
            if (thicknessAnim != null) pen.BeginAnimation(Pen.ThicknessProperty, thicknessAnim);
            else { pen.Freeze(); }
            return pen;
        }

        private DoubleAnimation CreateAnimation(double p)
        {
            var anim = new DoubleAnimation();
            anim.Duration = TimeSpan.FromSeconds(1);
            anim.AutoReverse = true;
            anim.RepeatBehavior = RepeatBehavior.Forever;
            anim.From = p;
            anim.To = 2 * p;
            Timeline.SetDesiredFrameRate(anim, 3);
            return anim;
        }

        private Color CalculateColor(Color low, Color high, double value)
        {
            if (value < 0 || value > 1)
            {
                System.Diagnostics.Debug.WriteLine("Neoekvan hodnota: " + value.ToString());
                value = 1;
            }
            double deltaR = high.R - low.R;
            double deltaG = high.G - low.G;
            double deltaB = high.B - low.B;
            double deltaA = high.A - low.A;
            return Color.FromArgb((byte)(low.A + value * deltaA),
                (byte)(low.R + value * deltaR),
                (byte)(low.G + value * deltaG),
                (byte)(low.B + value * deltaB));
        }

        /// <summary>
        /// Vrt hodnotu Z pepotenou do intervalu od 0 do 1 (v zvislosti na MinZ a MaxZ)
        /// </summary>
        internal static double GetRelativeZ(double value, GraphDataInfo dataInfo)
        {
            if (value < dataInfo.MinZ) return 0;
            if (value > dataInfo.MaxZ || dataInfo.MinZ == dataInfo.MaxZ) return 1;

            return (value - dataInfo.MinZ) / (dataInfo.MaxZ - dataInfo.MinZ);
        }

        internal static double GetAbsoluteZ(double value, GraphDataInfo dataInfo)
        {
            return dataInfo.MinZ + value * (dataInfo.MaxZ - dataInfo.MinZ);
        }

        private ZValue CalculateZ(GraphPointInternal startPoint, GraphPointInternal endPoint, double p)
        {
            if (Style.UseGradient)
            {
                double zAbs = startPoint.Original.Pos.Z + (endPoint.Original.Pos.Z - startPoint.Original.Pos.Z) * p;
                double zRelative = Graph.GetRelativeZ(zAbs, DataInfo);
                return new ZValue(zAbs, zRelative, CalculateColor(Style.LowColor, Style.HighColor, zRelative));
            }
            else
            {
                double zAbs = startPoint.Original.Pos.Z;
                double zRelative = Graph.GetRelativeZ(zAbs, DataInfo);
                return new ZValue(zAbs, zRelative, CalculateColor(Style.LowColor, Style.HighColor, zRelative));
            }
        }

        private double GetThickness(GraphStyle style)
        {
            if (style.Highlight && style.HighlightByAnimation == false)
            {
                return style.Thickness * 2;
            }
            return style.Thickness;
        }

        internal GraphNearestPoint GetNearestPoint(Point relativePosition)
        {
            if (points.Count < 2) return null;

            GetNearestPointResult result = default(GetNearestPointResult);
            int graphPointId = 0;
            ZValue z = default(ZValue);
            bool first = true;
            bool zeroParameter = false;

            for (int i = 0; i < points.Count - 1; i++)
            {
                if (points[i].Original.Interrupt) { first = true; continue; }
                var joinLine = GetJoinLine(ProjectPoint(points[i], view), ProjectPoint(points[i + 1], view), first, DataInfo.Constant);

                var b = ProcessNearestPoint(relativePosition, ref result, ref graphPointId, i, joinLine.Item1, joinLine.Item2);
                zeroParameter |= b && joinLine.Item3.HasValue;

                if (joinLine.Item3.HasValue)
                {
                    b = ProcessNearestPoint(relativePosition, ref result, ref graphPointId, i, joinLine.Item2, joinLine.Item3.Value);
                    if (b) zeroParameter = false;
                }
                first = false;
            }

            if (result == null || result.Distance > hitTestDistance) return null;
            z = CalculateZ(points[graphPointId], points[graphPointId + 1], zeroParameter ? 0 : result.Parameter);

            if (result.Parameter < 0.5)
                return new GraphNearestPoint(result, points[graphPointId].Original, points[graphPointId].Original, this.DataObject, z);
            else
                return new GraphNearestPoint(result, points[graphPointId].Original, points[graphPointId + 1].Original, this.DataObject, z);
        }

        private static bool ProcessNearestPoint(Point relativePosition, ref GetNearestPointResult result, ref int graphPointId, int i, Point3D p1, Point3D p2)
        {
            if (p1.X == p2.X && p1.Y == p2.Y) return false;

            var tmp = LineSegmentHelper.GetNearestPoint(p1.As2DPoint(), p2.As2DPoint(), relativePosition);
            if (result == null || tmp.Distance < result.Distance)
            {
                result = tmp;
                graphPointId = i;
                return true;
            }
            return false;
        }

        internal object DataObject { get; set; }
    }

    internal class GraphNearestPoint
    {
        public GraphNearestPoint(GetNearestPointResult nearestPoint, GraphPoint previousPoint, GraphPoint nearestsGraphPoint, object graphDataObject, ZValue zValue)
        {
            NearestPoint = nearestPoint;
            PreviousGraphPoint = previousPoint;
            this.NearestGraphPoint = nearestsGraphPoint;
            this.GraphDataObject = graphDataObject;
            Z = zValue;
        }

        public GetNearestPointResult NearestPoint { get; private set; }

        public GraphPoint PreviousGraphPoint { get; private set; }

        public ZValue Z { get; private set; }

        public GraphPoint NearestGraphPoint { get; private set; }

        /// <summary>
        /// Datov objekt reprezentujc celou adu grafu
        /// </summary>
        public object GraphDataObject { get; private set; }
    }
}