namespace Brjnk.CoordinateSystem2D.Layers
{
    using System.Windows;
    using System.Windows.Controls;

    /// <summary>
    /// </summary>
    public class GraphLayer : LayerBase
    {
        private GraphAdapterContext adapterContext;

        public GraphLayer()
        {
            CreateNewContext();
        }

        protected override void ViewUpdated()
        {
            adapterContext.UpdateView(view);
        }

        protected override void ViewChanged(ViewInfoBase oldValue, ViewInfoBase viewInfoBase)
        {
            adapterContext.Update(view);
        }

        protected override void TransformationChanged()
        {
            adapterContext.Update(view);
        }

        #region properties

        public GraphDataAdapter DataAdapter
        {
            get { return (GraphDataAdapter)GetValue(DataAdapterProperty); }
            set { SetValue(DataAdapterProperty, value); }
        }

        public static readonly DependencyProperty DataAdapterProperty =
             DependencyProperty.Register("DataAdapter", typeof(GraphDataAdapter), typeof(GraphLayer), new UIPropertyMetadata(null, AdapterChanged));

        public object DataSource
        {
            get { return (object)GetValue(DataSourceProperty); }
            set { SetValue(DataSourceProperty, value); }
        }

        public static readonly DependencyProperty DataSourceProperty =
            DependencyProperty.Register("DataSource", typeof(object), typeof(GraphLayer), new UIPropertyMetadata(null, DataChanged));

        public DataTemplate GraphInfoTemplate
        {
            get { return (DataTemplate)GetValue(GraphInfoTemplateProperty); }
            set { SetValue(GraphInfoTemplateProperty, value); }
        }

        public static readonly DependencyProperty GraphInfoTemplateProperty =
            DependencyProperty.Register("GraphInfoTemplate", typeof(DataTemplate), typeof(GraphLayer), new UIPropertyMetadata(null));

        public DataTemplate GraphMenuTemplate
        {
            get { return (DataTemplate)GetValue(GraphMenuTemplateProperty); }
            set { SetValue(GraphMenuTemplateProperty, value); }
        }

        public static readonly DependencyProperty GraphMenuTemplateProperty =
            DependencyProperty.Register("GraphMenuTemplate", typeof(DataTemplate), typeof(GraphLayer), new UIPropertyMetadata(null));

        public DataTemplateSelector GraphInfoTemplateSelector
        {
            get { return (DataTemplateSelector)GetValue(GraphInfoTemplateSelectorProperty); }
            set { SetValue(GraphInfoTemplateSelectorProperty, value); }
        }

        public static readonly DependencyProperty GraphInfoTemplateSelectorProperty =
            DependencyProperty.Register("GraphInfoTemplateSelector", typeof(DataTemplateSelector), typeof(GraphLayer), new UIPropertyMetadata(null));

        public DataTemplateSelector GraphMenuTemplateSelector
        {
            get { return (DataTemplateSelector)GetValue(GraphMenuTemplateSelectorProperty); }
            set { SetValue(GraphMenuTemplateSelectorProperty, value); }
        }

        public static readonly DependencyProperty GraphMenuTemplateSelectorProperty =
            DependencyProperty.Register("GraphMenuTemplateSelector", typeof(DataTemplateSelector), typeof(GraphLayer), new UIPropertyMetadata(null));

        #endregion properties

        private static void AdapterChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var oldAdapter = e.OldValue as GraphDataAdapter;
            if (oldAdapter != null) oldAdapter.Destroy();
            var target = (GraphLayer)sender;
            target.CreateNewContext();
            if (target.DataAdapter != null)
            {
                target.DataAdapter.Initialize(target.adapterContext);
                target.DataAdapter.SetData(target.DataSource);
            }
        }

        private void CreateNewContext()
        {
            adapterContext = new GraphAdapterContext();
            adapterContext.UpdateView(view);
            this.Content = adapterContext.Visual;
        }

        private static void DataChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var target = (GraphLayer)sender;
            target.DataAdapter.SetData(target.DataSource);
        }

        protected override void InfoPopupOpening(object sender, OpenPopupEventArgs e)
        {
            var data = adapterContext.GetNearestPoints(e.RelativePosition);
            base.InfoContent = LayerHelper.CreateContent(GraphInfoTemplate, GraphInfoTemplateSelector, data, p => p.GraphDataObject, ContentDecorator);
        }

        protected override void MenuOpening(object sender, OpenPopupEventArgs e)
        {
            var data = adapterContext.GetNearestPoints(e.RelativePosition);
            base.MenuContent = LayerHelper.CreateContent(GraphMenuTemplate, GraphMenuTemplateSelector, data, p => p.GraphDataObject, ContentDecorator);
        }

        private void ContentDecorator(UIElement element, GraphNearestPoint graphPoint)
        {
            SetZValue(element, graphPoint.Z);
            SetNearestPoint(element, graphPoint.NearestGraphPoint);
            SetPreviousPoint(element, graphPoint.PreviousGraphPoint);
            SetParameter(element, graphPoint.NearestPoint.Parameter);
        }

        #region attachedProperties

        public static GraphPoint GetNearestPoint(DependencyObject obj)
        {
            return (GraphPoint)obj.GetValue(NearestPointProperty);
        }

        public static void SetNearestPoint(DependencyObject obj, GraphPoint value)
        {
            obj.SetValue(NearestPointProperty, value);
        }

        /// <summary>
        /// Nejbli bod grafu vzhledem.
        /// </summary>
        public static readonly DependencyProperty NearestPointProperty =
            DependencyProperty.RegisterAttached("NearestPoint", typeof(GraphPoint), typeof(GraphLayer), new FrameworkPropertyMetadata() { Inherits = true });

        public static ZValue GetZValue(DependencyObject obj)
        {
            return (ZValue)obj.GetValue(ZValueProperty);
        }

        public static void SetZValue(DependencyObject obj, ZValue value)
        {
            obj.SetValue(ZValueProperty, value);
        }

        /// <summary>
        /// Z hodnota nejbliho bodu spojnice grafu.
        /// </summary>
        public static readonly DependencyProperty ZValueProperty =
            DependencyProperty.RegisterAttached("ZValue", typeof(ZValue), typeof(GraphLayer), new FrameworkPropertyMetadata() { Inherits = true });

        public static GraphPoint GetPreviousPoint(DependencyObject obj)
        {
            return (GraphPoint)obj.GetValue(PreviousPointProperty);
        }

        public static void SetPreviousPoint(DependencyObject obj, GraphPoint value)
        {
            obj.SetValue(PreviousPointProperty, value);
        }

        public static readonly DependencyProperty PreviousPointProperty =
    DependencyProperty.RegisterAttached("PreviousPoint", typeof(GraphPoint), typeof(GraphLayer), new UIPropertyMetadata());

        public static double GetParameter(DependencyObject obj)
        {
            return (double)obj.GetValue(ParameterProperty);
        }

        public static void SetParameter(DependencyObject obj, double value)
        {
            obj.SetValue(ParameterProperty, value);
        }

        public static readonly DependencyProperty ParameterProperty =
    DependencyProperty.RegisterAttached("Parameter", typeof(double), typeof(GraphLayer), new UIPropertyMetadata());

        #endregion attachedProperties
    }
}