namespace Brjnk.CoordinateSystem2D.Layers
{
    using System;
    using System.Collections.Generic;

    /// <summary>
    /// Vyfiltruje nadbyten grafov body, tj. body, kter neovlivn podobu graf a tm se zv rychlost vykreslovn
    /// </summary>
    internal class GraphPointsFilter
    {
        private IList<GraphPointInternal> points;

        public GraphPointsFilter(IList<GraphPointInternal> points)
        {
            this.points = points;
        }

        private double GuessDistance(GraphPoint point1, GraphPoint point2, ViewInfoBase view)
        {
            return Math.Abs(point1.Pos.X - point2.Pos.X) * view.Definition.ZoomX + Math.Abs(point1.Pos.Y - point2.Pos.Y) * view.Definition.ZoomY;
        }

        private double CalculateScreenDistance(GraphPointInternal point1, GraphPointInternal point2, ViewInfoBase view)
        {
            double a = (point1.Transformed.X - point2.Transformed.X) * view.Definition.ZoomX;
            double b = (point1.Transformed.Y - point2.Transformed.Y) * view.Definition.ZoomY;
            return Math.Sqrt(Math.Pow(a, 2) + Math.Pow(b, 2));
        }

        public IEnumerable<GraphPointInternal> GetPoints(ViewInfoBase view, double simplifyDegree)
        {
            GraphPointInternal last = default(GraphPointInternal); //pedchoz bod, intern 2D souadnice
            Sides area = 0; //neviditeln oblast, ve kter byl pedchoz bod
            bool lastOmittedInvisible = false; //true, pokud byl posledn bod vynechn kvli neviditelnosti (tj. byl mimo viditelnou oblast, nikoliv e byl vynechn pro nadbytenost)
            if (points.Count == 0) yield break;
            for (int i = 0; i < points.Count - 1; i++)
            {
                var point = points[i];
                //zjistit oblast, ve kter je tento bod
                Sides current = 0;
                if (point.Transformed.X < view.TopLeftInternal.X) current = current | Sides.Left;
                if (point.Transformed.Y < view.BottomLeftInternal.Y) current = current | Sides.Bottom;
                if (point.Transformed.X > view.TopRightInternal.X) current = current | Sides.Right;
                if (point.Transformed.Y > view.TopRightInternal.Y) current = current | Sides.Top;

                if (current > 0)
                {
                    //tento bod je mimo viditelnou oblast
                    if ((area == current))
                    {
                        //pedchoz i tento bod je ve stejn oblasti, me bt vynechn
                        last = point; area = current; lastOmittedInvisible = true;
                    }
                    else
                    {
                        //bod je v jin oblasti, mus bt vrcen
                        if (lastOmittedInvisible) yield return last;
                        yield return point;
                        last = point; area = current; lastOmittedInvisible = false;
                    }
                }
                else
                {
                    //tento bod je ve viditeln oblasti
                    if (lastOmittedInvisible)
                    {
                        //pokud byl pedchoz bod v neviditeln oblasti
                        yield return last;
                        yield return point;
                        last = point; area = 0; lastOmittedInvisible = false;
                    }
                    else
                    {
                        //pedchoz bod byl ve viditeln oblasti
                        if (point.Original.Interrupt)
                        {
                            //bod m nastaven atribut Interrupt, mus bt vrcen
                            yield return point;
                            last = null; area = 0; lastOmittedInvisible = false;
                        }
                        else if (last != null && CalculateScreenDistance(point, last, view) < simplifyDegree)
                        {
                            //bod bude vynechn, je pli blzko
                            area = 0; lastOmittedInvisible = false;
                        }
                        else
                        {
                            //pedchoz bod je pli vzdlen na to, aby tento mohl bt vynechn
                            yield return point;
                            last = point; area = 0; lastOmittedInvisible = false;
                        }
                    }
                }
            }

            //ped poslednm bodem vrt i pedchoz, pokud byl vynechn
            if (lastOmittedInvisible) yield return last;

            //vdy vrtit posledn bod, posledn bod neprochz pedchozm cyklem
            yield return points[points.Count - 1];
        }

        private enum Sides
        {
            Left = 1,
            Top = 2,
            Right = 4,
            Bottom = 8
        }
    }
}