namespace Brjnk.CoordinateSystem2D.Layers
{
    using System;
    using System.Windows;
    using System.Windows.Media;

    /// <summary>
    /// </summary>
    public class Grid : LayerBase
    {
        private GridVisual visual;

        public Grid()
        {
            visual = new GridVisual(CreateGridPens());
        }

        public override System.Windows.FrameworkElement Content
        {
            get { return visual; }
        }

        public override System.Windows.FrameworkElement MenuContent
        {
            get { return null; }
        }

        protected override void ViewChanged(ViewInfoBase oldValue, ViewInfoBase viewInfoBase)
        {
            visual.SetView(view);
        }

        // ViewUpdated nen teba zachytvat, nebo pekreslen je provdno pi zmn osy
        // Udlost zmny osa je registrovna pi zmn kontextu (ContextChanged)

        protected override void ContextChanged(LayerContext oldValue, LayerContext layerContext)
        {
            if (oldValue != null)
            {
                oldValue.ParentArea.RemoveHandler(Axis.LabelsGeneratedEvent, new LabelsGeneratedHandler(RefreshLabels));
            }

            if (layerContext != null)
            {
                layerContext.ParentArea.AddHandler(Axis.LabelsGeneratedEvent, new LabelsGeneratedHandler(RefreshLabels));
            }
        }

        private void RefreshLabels(object sender, LabelsGeneratedEventArgs args)
        {
            switch (args.Orientation)
            {
                case AxisOrientation.Horizontal:
                    visual.SetDataX(args.Labels);
                    break;

                case AxisOrientation.Vertical:
                    visual.SetDataY(args.Labels);
                    break;

                default:
                    throw new Exception("AxisOrientation m neznmou hodnotu");
            }
        }

        #region properties

        public Pen PenXLarge
        {
            get { return (Pen)GetValue(PenXLargeProperty); }
            set { SetValue(PenXLargeProperty, value); }
        }

        public static readonly DependencyProperty PenXLargeProperty =
            DependencyProperty.Register("PenXLarge", typeof(Pen), typeof(Grid),
            new UIPropertyMetadata(new Pen(Brushes.Black, 1.5), PensChanged));

        public Pen PenXBig
        {
            get { return (Pen)GetValue(PenXBigProperty); }
            set { SetValue(PenXBigProperty, value); }
        }

        public static readonly DependencyProperty PenXBigProperty =
            DependencyProperty.Register("PenXBig", typeof(Pen), typeof(Grid),
            new UIPropertyMetadata(new Pen(Brushes.Gray, 1), PensChanged));

        public Pen PenXMedium
        {
            get { return (Pen)GetValue(PenXMediumProperty); }
            set { SetValue(PenXMediumProperty, value); }
        }

        public static readonly DependencyProperty PenXMediumProperty =
            DependencyProperty.Register("PenXMedium", typeof(Pen), typeof(Grid),
            new UIPropertyMetadata(new Pen(Brushes.LightGray, 0.5), PensChanged));

        public Pen PenXSmall
        {
            get { return (Pen)GetValue(PenXSmallProperty); }
            set { SetValue(PenXSmallProperty, value); }
        }

        public static readonly DependencyProperty PenXSmallProperty =
            DependencyProperty.Register("PenXSmall", typeof(Pen), typeof(Grid),
            new UIPropertyMetadata(null, PensChanged));

        public Pen PenYLarge
        {
            get { return (Pen)GetValue(PenYLargeProperty); }
            set { SetValue(PenYLargeProperty, value); }
        }

        public static readonly DependencyProperty PenYLargeProperty =
            DependencyProperty.Register("PenYLarge", typeof(Pen), typeof(Grid),
            new UIPropertyMetadata(new Pen(Brushes.Black, 1.5), PensChanged));

        public Pen PenYBig
        {
            get { return (Pen)GetValue(PenYBigProperty); }
            set { SetValue(PenYBigProperty, value); }
        }

        public static readonly DependencyProperty PenYBigProperty =
            DependencyProperty.Register("PenYBig", typeof(Pen), typeof(Grid),
            new UIPropertyMetadata(new Pen(Brushes.Gray, 1), PensChanged));

        public Pen PenYMedium
        {
            get { return (Pen)GetValue(PenYMediumProperty); }
            set { SetValue(PenYMediumProperty, value); }
        }

        public static readonly DependencyProperty PenYMediumProperty =
            DependencyProperty.Register("PenYMedium", typeof(Pen), typeof(Grid),
            new UIPropertyMetadata(new Pen(Brushes.LightGray, 0.5), PensChanged));

        public Pen PenYSmall
        {
            get { return (Pen)GetValue(PenYSmallProperty); }
            set { SetValue(PenYSmallProperty, value); }
        }

        public static readonly DependencyProperty PenYSmallProperty =
            DependencyProperty.Register("PenYSmall", typeof(Pen), typeof(Grid),
            new UIPropertyMetadata(null, PensChanged));

        private static void PensChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var obj = sender as Grid;
            var pens = obj.CreateGridPens();
            obj.visual.SetPens(pens);
        }

        #endregion properties

        private GridPens CreateGridPens()
        {
            var pens = new GridPens();
            pens.XLarge = this.PenXLarge;
            pens.XBig = this.PenXBig;
            pens.XMedium = this.PenXMedium;
            pens.XSmall = this.PenXSmall;
            pens.YLarge = this.PenYLarge;
            pens.YBig = this.PenYBig;
            pens.YMedium = this.PenYMedium;
            pens.YSmall = this.PenYSmall;
            return pens;
        }
    }
}