namespace Brjnk.CoordinateSystem2D.Layers
{
    using System;
    using System.ComponentModel;
    using System.Windows;

    /// <summary>
    /// </summary>
    public abstract class LayerBase : DependencyObject, ILayer
    {
        private double opacity = 1;

        public double Opacity
        {
            get { return opacity; }
            set { opacity = value; OnPropertyChanged("Opacity"); }
        }

        private bool visible = true;

        public bool Visible
        {
            get { return visible; }
            set { visible = value; OnPropertyChanged("Visible"); }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        private FrameworkElement content;

        private FrameworkElement contextMenu;

        private FrameworkElement toolTip;

        public virtual System.Windows.FrameworkElement Content
        {
            get { return content; }
            protected set { content = value; OnPropertyChanged("Content"); }
        }

        public virtual System.Windows.FrameworkElement MenuContent
        {
            get { return contextMenu; }
            protected set { contextMenu = value; OnPropertyChanged("MenuContent"); }
        }

        public virtual System.Windows.FrameworkElement InfoContent
        {
            get { return toolTip; }
            protected set { toolTip = value; OnPropertyChanged("InfoContent"); }
        }

        protected ViewInfoBase view;
        protected LayerContext context;

        public void SetContext(LayerContext context, ViewInfoBase view)
        {
            var oldValue = this.context;
            if (oldValue != null)
            {
                oldValue.ParentArea.InfoPopupOpening -= new EventHandler<OpenPopupEventArgs>(InfoPopupOpening);
                oldValue.ParentArea.InfoPopupClosing -= new EventHandler(InfoPopupClosing);
                oldValue.ParentArea.MenuOpening -= new EventHandler<OpenPopupEventArgs>(MenuOpening);
                oldValue.ParentArea.MenuClosing -= new EventHandler(MenuClosing);
            }

            this.context = context;
            ContextChanged(oldValue, this.context);

            if (this.context != null)
            {
                this.context.ParentArea.InfoPopupOpening += new EventHandler<OpenPopupEventArgs>(InfoPopupOpening);
                this.context.ParentArea.InfoPopupClosing += new EventHandler(InfoPopupClosing);
                this.context.ParentArea.MenuOpening += new EventHandler<OpenPopupEventArgs>(MenuOpening);
                this.context.ParentArea.MenuClosing += new EventHandler(MenuClosing);
            }

            SetView(view);
        }

        public void SetView(ViewInfoBase view)
        {
            var oldValue = this.view;
            if (oldValue != null)
            {
                oldValue.ViewChanged -= new EventHandler<EventArgs>(view_ViewChanged);
                oldValue.TransformationChanged -= new EventHandler<EventArgs>(view_TransformationChanged);
                oldValue.ChangeCompleted -= new EventHandler<EventArgs>(view_ChangeCompleted);
            }

            this.view = view;

            if (view != null)
            {
                view.ViewChanged += new EventHandler<EventArgs>(view_ViewChanged);
                view.TransformationChanged += new EventHandler<EventArgs>(view_TransformationChanged);
                view.ChangeCompleted += new EventHandler<EventArgs>(view_ChangeCompleted);
            }

            ViewChanged(oldValue, this.view);
            ViewUpdated();
        }

        private void view_ChangeCompleted(object sender, EventArgs e)
        {
            ViewUpdateCompleted();
        }

        private void view_TransformationChanged(object sender, EventArgs e)
        {
            TransformationChanged();
        }

        private void view_ViewChanged(object sender, EventArgs e)
        {
            ViewUpdated();
        }

        protected virtual void MenuClosing(object sender, EventArgs e)
        {
        }

        protected virtual void MenuOpening(object sender, OpenPopupEventArgs e)
        {
        }

        protected virtual void InfoPopupClosing(object sender, EventArgs e)
        {
        }

        protected virtual void InfoPopupOpening(object sender, OpenPopupEventArgs e)
        {
        }

        /// <summary>
        /// Metoda je volna pi zmn pohledu (nap. pi posunut nebo zoomovn).
        /// </summary>
        protected virtual void ViewUpdated()
        {
        }

        /// <summary>
        /// Metoda je volna pi zmn pohledu. Na rozdl od ViewUpdated nen volna opakovan, nap. pi animovn pohledu.
        /// Z tohoto dvodu je tato metoda vhodn pro asov nronj vpoty
        /// </summary>
        protected virtual void ViewUpdateCompleted()
        {
        }

        /// <summary>
        /// Metoda je volna pi kliknut myi do soustavy souadnic.
        /// </summary>
        public virtual void MouseClick(MousePosition mousePosition)
        {
        }

        /// <summary>
        /// Metoda je volna pi zmn instance ViewInfoBase, tzn. pi zavoln ILayer.SetContext()
        /// Po voln tto metody je zavolna metoda ViewUpdated.
        /// </summary>
        protected virtual void ViewChanged(ViewInfoBase oldValue, ViewInfoBase viewInfoBase)
        {
        }

        /// <summary>
        /// Metoda je volna pi zmn transformace osy.
        /// </summary>
        protected virtual void TransformationChanged()
        {
        }

        protected virtual void ContextChanged(LayerContext oldValue, LayerContext layerContext)
        {
        }
    }
}