namespace Brjnk.CoordinateSystem2D.Layers
{
    using System;
    using System.Collections.Generic;
    using System.Windows;
    using System.Windows.Controls;

    /// <summary>
    /// </summary>
    public static class LayerHelper
    {
        /// <summary>Obal vechny pedan objekty do DataTemplate a vrt je jako vertikln StackPanel.</summary>
        public static FrameworkElement CreateContent<T>(DataTemplate template, DataTemplateSelector selector, IEnumerable<T> data)
        {
            return CreateContent(template, selector, data, o => o, null);
        }

        /// <summary>Obal vechny pedan objekty do DataTemplate a vrt je jako vertikln StackPanel.</summary>
        /// <param name="decorator">Delegt, kter bude zavoln na kad vytvoen element. Argumenty delegtu jsou UIElement (vytvoen vizuln prvek) a Object (datov objekt, ze kterho je prezentovn)</param>
        public static FrameworkElement CreateContent<T>(DataTemplate template, DataTemplateSelector selector, IEnumerable<T> data, Func<T, object> dataContextSelector, Action<UIElement, T> decorator)
        {
            if (template == null && selector == null) return null;

            var stackPanel = new StackPanel();
            foreach (var p in data)
            {
                UIElement element = CreateContent(template, selector, p, stackPanel, dataContextSelector, decorator);
                if (element == null) continue;
                stackPanel.Children.Add(element);
            }
            if (stackPanel.Children.Count == 0) return null;
            return stackPanel;
        }

        public static UIElement CreateContent<T>(DataTemplate template, DataTemplateSelector selector, T dataObject, DependencyObject container)
        {
            return CreateContent(template, selector, dataObject, container, null, null);
        }

        public static UIElement CreateContent<T>(DataTemplate template, DataTemplateSelector selector, T dataObject, DependencyObject container, Func<T, object> dataContextSelector, Action<UIElement, T> decorator)
        {
            UIElement element;
            object dataContext;
            if (dataContextSelector != null) dataContext = dataContextSelector(dataObject);
            else dataContext = dataObject;

            if (template != null)
            {
                element = template.LoadContent() as UIElement;
            }
            else if (selector != null)
            {
                element = selector.SelectTemplate(dataContext, container).LoadContent() as UIElement;
            }
            else
            {
                return null;
            }

            var fe = element as FrameworkElement;
            if (fe != null)
            {
                fe.DataContext = dataContext;
            }
            if (decorator != null) decorator(element, dataObject);
            return element;
        }
    }
}