namespace Brjnk.CoordinateSystem2D.Layers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;
    using System.Windows.Shapes;

    /// <summary>
    /// </summary>
    public class PointsAdapterContext
    {
        //Maximln vzdlenost pro vrcen metodou GetNearPoints();
        private const double hitTestRadius = 10;

        private Dictionary<object, PointInfo> points = new Dictionary<object, PointInfo>();
        private DataTemplate template;
        private DataTemplateSelector templateSelector;
        private Brjnk.Common.UI.Canvas canvas;
        private ViewInfoBase view;

        public PointsAdapterContext(Brjnk.Common.UI.Canvas canvas, DataTemplate template, DataTemplateSelector templateSelector)
        {
            this.canvas = canvas;
            this.template = template;
            this.templateSelector = templateSelector;
        }

        public void AddPoint(object key, object data, Point position, object customStyle)
        {
            PointInfo p;

            if (points.ContainsKey(key))
            {
                p = points[key];
            }
            else
            {
                p = new PointInfo();
                p.DataObject = data;
                p.Element = CreateElement(p);
                points.Add(key, p);
                canvas.Children.Add(p.Element);
            }

            p.DataObject = data;
            p.Position = position;
            if (view != null) p.Transformed = view.Transform(p.Position);
            p.CustomStyle = customStyle;
            PlacePoint(p);
            PointsLayer.SetCustomStyle(p.Element, customStyle);
            p.Element.DataContext = p.DataObject;
        }

        public void SetPosition(object key, Point position)
        {
            if (points.ContainsKey(key))
            {
                var p = points[key];
                p.Position = position;
                p.Transformed = view.Transform(p.Position);
                PlacePoint(p);
            }
        }

        public void DeletePoint(object key)
        {
            var element = points[key];
            points.Remove(key);
            canvas.Children.Remove(element.Element);
        }

        public void Clear()
        {
            points.Clear();
            canvas.Children.Clear();
        }

        /// <summary>
        /// Vrt relativn souadnice umstn objektu
        /// </summary>
        private Point GetDestinationPosition(PointInfo point)
        {
            if (view == null) return default(Point);
            return view.FromInternalToRelative(point.Transformed);
        }

        private void PlacePoint(PointInfo point)
        {
            var dest = GetDestinationPosition(point);
            Brjnk.Common.UI.Canvas.SetX(point.Element, dest.X);
            Brjnk.Common.UI.Canvas.SetY(point.Element, dest.Y);
        }

        private void PlaceAllPoints()
        {
            foreach (var p in points)
            {
                PlacePoint(p.Value);
            }
        }

        private FrameworkElement CreateElement(PointInfo point)
        {
            FrameworkElement retVal;
            if (template != null)
                retVal = template.LoadContent() as FrameworkElement;
            else if (templateSelector != null)
            {
                retVal = (templateSelector.SelectTemplate(point.DataObject, null) as DataTemplate).LoadContent() as FrameworkElement;
            }
            else
            {
                //Pokud nen nastavena ablona ani templateSelector, vytvo ern tvereek.
                retVal = new Rectangle() { Fill = Brushes.Black, Width = 1, Height = 1 };
            }
            retVal.DataContext = point.DataObject;
            return retVal;
        }

        /// <summary>
        /// Metoda mus bt volna vdy pi zmn pohledu, nap. posunut.
        /// Tato tda nezachytv ViewInfoBase.ViewChanged!
        /// </summary>
        internal void UpdateView(ViewInfoBase view)
        {
            this.view = view;
            PlaceAllPoints();
        }

        /// <summary>
        /// Pepot souadnice bodu, nap po zmn tranforman funkce osy.
        /// Pi zmn pouze pohledu m bt volna metoda UpdateView(), protoe je asov mn nron (nen toti nutn provdt transformaci souadnic)
        /// </summary>
        /// <param name="view"></param>
        internal void Recalculate(ViewInfoBase view)
        {
            foreach (var p in points)
                p.Value.Transformed = view.Transform(p.Value.Position);
        }

        /// <summary>
        /// Nastav nov ablony (po zmn)
        /// </summary>
        internal void SetTemplate(DataTemplate template, DataTemplateSelector selector)
        {
            canvas.Children.Clear();
            this.template = template;
            this.templateSelector = selector;

            foreach (var p in points)
            {
                p.Value.Element = CreateElement(p.Value);
                PointsLayer.SetCustomStyle(p.Value.Element, p.Value.CustomStyle);
                canvas.Children.Add(p.Value.Element);
            }
        }

        /// <summary>
        /// Vrt blzk body seazen od nejbliho k nejvzdlenjmu
        /// </summary>
        public IEnumerable<PointInfo> GetNearPoints(Point relative)
        {
            //kolekce bod a jejich vzdlenost od point
            List<Tuple<PointInfo, double>> outputPoints = new List<Tuple<PointInfo, double>>();

            foreach (var p in points)
            {
                var destination = GetDestinationPosition(p.Value);
                double distance = (relative - destination).Length;
                if (distance < hitTestRadius) outputPoints.Add(new Tuple<PointInfo, double>(p.Value, distance));
            }

            return outputPoints.OrderBy(p => p.Item2).Select(p => p.Item1);
        }
    }
}