namespace Brjnk.CoordinateSystem2D.Layers
{
    using System.Windows;
    using System.Windows.Controls;

    /// <summary>
    /// </summary>
    public class PointsLayer : LayerBase
    {
        private PointsAdapterContext adapterContext;

        public PointsLayer()
        {
            CreateNewContext();
        }

        private void CreateNewContext()
        {
            var canvas = new Brjnk.Common.UI.Canvas();
            this.Content = canvas;
            adapterContext = new PointsAdapterContext(canvas, this.PointTemplate, this.PointTemplateSelector);
            adapterContext.UpdateView(view);
        }

        protected override void ViewUpdated()
        {
            adapterContext.UpdateView(view);
        }

        protected override void ViewChanged(ViewInfoBase oldValue, ViewInfoBase viewInfoBase)
        {
            adapterContext.Recalculate(view);
        }

        protected override void TransformationChanged()
        {
            adapterContext.Recalculate(view);
        }

        protected override void InfoPopupOpening(object sender, OpenPopupEventArgs e)
        {
            this.InfoContent = LayerHelper.CreateContent(
                PointInfoTemplate,
                PointInfoTemplateSelector,
                adapterContext.GetNearPoints(e.RelativePosition),
                point => (point as PointInfo).DataObject,
                (element, data) => SetCustomStyle(element, (data as PointInfo).CustomStyle));
        }

        protected override void MenuOpening(object sender, OpenPopupEventArgs e)
        {
            this.MenuContent = LayerHelper.CreateContent(
                PointContextMenuTemplate,
                PointContextMenuTemplateSelector,
                adapterContext.GetNearPoints(e.RelativePosition),
                point => (point as PointInfo).DataObject,
                (element, data) => SetCustomStyle(element, (data as PointInfo).CustomStyle));
        }

        #region properties

        public PointsDataAdapter DataAdapter
        {
            get { return (PointsDataAdapter)GetValue(DataAdapterProperty); }
            set { SetValue(DataAdapterProperty, value); }
        }

        public static readonly DependencyProperty DataAdapterProperty =
             DependencyProperty.Register("DataAdapter", typeof(PointsDataAdapter), typeof(PointsLayer), new UIPropertyMetadata(null, AdapterChanged));

        public object DataSource
        {
            get { return (object)GetValue(DataSourceProperty); }
            set { SetValue(DataSourceProperty, value); }
        }

        public static readonly DependencyProperty DataSourceProperty =
            DependencyProperty.Register("DataSource", typeof(object), typeof(PointsLayer), new UIPropertyMetadata(null, DataChanged));

        public DataTemplate PointTemplate
        {
            get { return (DataTemplate)GetValue(PointTemplateProperty); }
            set { SetValue(PointTemplateProperty, value); }
        }

        public static readonly DependencyProperty PointTemplateProperty =
            DependencyProperty.Register("PointTemplate", typeof(DataTemplate), typeof(PointsLayer), new UIPropertyMetadata(null, TemplateChanged));

        public DataTemplateSelector PointTemplateSelector
        {
            get { return (DataTemplateSelector)GetValue(PointTemplateSelectorProperty); }
            set { SetValue(PointTemplateSelectorProperty, value); }
        }

        public static readonly DependencyProperty PointTemplateSelectorProperty =
            DependencyProperty.Register("PointTemplateSelector", typeof(DataTemplateSelector), typeof(PointsLayer), new UIPropertyMetadata(null, TemplateChanged));

        public DataTemplate PointInfoTemplate
        {
            get { return (DataTemplate)GetValue(PointInfoTemplateProperty); }
            set { SetValue(PointInfoTemplateProperty, value); }
        }

        public static readonly DependencyProperty PointInfoTemplateProperty =
            DependencyProperty.Register("PointInfoTemplate", typeof(DataTemplate), typeof(PointsLayer), new UIPropertyMetadata(null));

        public DataTemplateSelector PointInfoTemplateSelector
        {
            get { return (DataTemplateSelector)GetValue(PointInfoTemplateSelectorProperty); }
            set { SetValue(PointInfoTemplateSelectorProperty, value); }
        }

        public static readonly DependencyProperty PointInfoTemplateSelectorProperty =
            DependencyProperty.Register("PointInfoTemplateSelector", typeof(DataTemplateSelector), typeof(PointsLayer), new UIPropertyMetadata(null));

        public DataTemplate PointContextMenuTemplate
        {
            get { return (DataTemplate)GetValue(PointContextMenuTemplateProperty); }
            set { SetValue(PointContextMenuTemplateProperty, value); }
        }

        public static readonly DependencyProperty PointContextMenuTemplateProperty =
            DependencyProperty.Register("PointContextMenuTemplate", typeof(DataTemplate), typeof(PointsLayer), new UIPropertyMetadata(null));

        public DataTemplateSelector PointContextMenuTemplateSelector
        {
            get { return (DataTemplateSelector)GetValue(PointContextMenuTemplateSelectorProperty); }
            set { SetValue(PointContextMenuTemplateSelectorProperty, value); }
        }

        public static readonly DependencyProperty PointContextMenuTemplateSelectorProperty =
            DependencyProperty.Register("PointContextMenuTemplateSelector", typeof(DataTemplateSelector), typeof(PointsLayer), new UIPropertyMetadata(null));

        private static void AdapterChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var oldAdapter = e.OldValue as PointsDataAdapter;
            if (oldAdapter != null) oldAdapter.Destroy();
            var target = (PointsLayer)sender;
            target.CreateNewContext();
            if (target.DataAdapter != null)
            {
                target.DataAdapter.Initialize(target.adapterContext);
                target.DataAdapter.SetData(target.DataSource);
            }
        }

        private static void DataChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var target = (PointsLayer)sender;
            target.DataAdapter.SetData(target.DataSource);
        }

        private static void TemplateChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            var obj = ((PointsLayer)sender);
            obj.adapterContext.SetTemplate(obj.PointTemplate, obj.PointTemplateSelector);
        }

        public static object GetCustomStyle(DependencyObject obj)
        {
            return (object)obj.GetValue(CustomStyleProperty);
        }

        public static void SetCustomStyle(DependencyObject obj, object value)
        {
            obj.SetValue(CustomStyleProperty, value);
        }

        /// <summary>
        /// Attached Property kter umonuje pedat do DataTemplate dal data (k podrobnmu uren vzhledu)
        /// </summary>
        public static readonly DependencyProperty CustomStyleProperty =
            DependencyProperty.RegisterAttached("CustomStyle", typeof(object), typeof(PointsLayer), new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.Inherits));

        #endregion properties
    }
}