namespace Brjnk.CoordinateSystem2D
{
    using System;
    using System.Linq;
    using System.Windows;

    /// <summary>
    /// </summary>
    public class LinearAxis : AxisDefinition
    {
        public Func<double, string> LabelFormatter
        {
            get { return (Func<double, string>)GetValue(LabelFormatterProperty); }
            set { SetValue(LabelFormatterProperty, value); }
        }

        public static readonly DependencyProperty LabelFormatterProperty =
            DependencyProperty.Register("LabelFormatter", typeof(Func<double, string>), typeof(LinearAxis), new PropertyMetadata(null, AxisChanged));

        /// <summary>
        /// Zklad pro urovn dlk osy. (Nap. 10 pro dlky o velikosti 10^n)
        /// </summary>
        public int SpacingBase
        {
            get { return (int)GetValue(LabelsBaseProperty); }
            set { SetValue(LabelsBaseProperty, value); }
        }

        public static readonly DependencyProperty LabelsBaseProperty =
            DependencyProperty.Register("SpacingBase", typeof(int), typeof(LinearAxis), new PropertyMetadata(10, AxisChanged, CoerceLabelsBase));

        public bool Reverse
        {
            get { return (bool)GetValue(ReverseProperty); }
            set { SetValue(ReverseProperty, value); }
        }

        public static readonly DependencyProperty ReverseProperty =
            DependencyProperty.Register("Reverse", typeof(bool), typeof(LinearAxis), new PropertyMetadata(false, AxisChanged));

        public static object CoerceLabelsBase(DependencyObject d, object baseValue)
        {
            var value = (int)baseValue;
            if (value <= 1) return 10;
            return value;
        }

        public override GetAxisLabelsResult GetAxisLabels(double from, double to, double minSpacing)
        {
            double exp = Math.Ceiling(Math.Log(minSpacing) / Math.Log(SpacingBase));
            double stepBig = Math.Pow(SpacingBase, exp);
            double stepSmall = Math.Pow(SpacingBase, exp - 1);

            CalculateStepsResult r = new CalculateStepsResult();
            r.Reverse = this.Reverse;
            r.AutoMediumStep = true;
            r.LargeZero = true;
            r.formatterBig = this.LabelFormatter;
            r.formatterLarge = this.LabelFormatter;
            r.stepBig = stepBig;
            r.stepSmall = stepSmall;
            return new GetAxisLabelsResult(this.Caption, this.GenerateLabels(from, to, r).ToArray());
        }

        public override double Transform(double value)
        {
            return Reverse ? -value : value;
        }

        public override double TransformBack(double value)
        {
            return Reverse ? -value : value;
        }

        private string GetLabelText(double current)
        {
            if (object.ReferenceEquals(LabelFormatter, null)) return current.ToString();
            return LabelFormatter(current);
        }

        public Func<double, string> LabelFullFormatter
        {
            get { return (Func<double, string>)GetValue(LabelFullFormatterProperty); }
            set { SetValue(LabelFullFormatterProperty, value); }
        }

        public static readonly DependencyProperty LabelFullFormatterProperty =
            DependencyProperty.Register("LabelFullFormatter", typeof(Func<double, string>), typeof(AxisDefinition), new UIPropertyMetadata(null));

        /// <summary>
        /// Vrt reprezentaci hodnoty jako text, vetn jednotky.
        /// </summary>
        public override string GetLabelTextFull(double value)
        {
            if (LabelFullFormatter == null) return value.ToString();
            return LabelFullFormatter(value);
        }

        protected override Freezable CreateInstanceCore()
        {
            return new LinearAxis();
        }
    }
}