namespace Brjnk.CoordinateSystem2D
{
    using System;
    using System.Runtime.InteropServices;
    using System.Windows;
    using System.Windows.Input;
    using System.Windows.Threading;

    /// <summary>
    /// Tda zpracovv vstup z myi nad Area2D.
    /// </summary>
    internal class MouseObserver
    {
        private System.Windows.FrameworkElement element;

        private ButtonState[] buttons = new ButtonState[5];
        private MouseButton moveButton;
        private bool doubleClickEnabled;
        private Point lastPoint;
        private uint doubleClickInterval = 0;

        internal MouseObserver(System.Windows.FrameworkElement mouseObserverElement, bool waitForDoubleClick, MouseButton moveButton)
        {
            this.element = mouseObserverElement;
            element.MouseDown += new System.Windows.Input.MouseButtonEventHandler(element_MouseDown);
            element.MouseUp += new System.Windows.Input.MouseButtonEventHandler(element_MouseUp);
            element.MouseMove += new System.Windows.Input.MouseEventHandler(element_MouseMove);
            element.MouseEnter += new MouseEventHandler(element_MouseEnter);
            element.MouseLeave += new MouseEventHandler(element_MouseLeave);
            element.MouseWheel += new MouseWheelEventHandler(element_MouseWheel);

            this.moveButton = moveButton;
            doubleClickEnabled = waitForDoubleClick;

            PrepareToolTipTimer();

            if (doubleClickEnabled)
            {
                if (FindOutDoubleClickInterval() == false)
                {
                    System.Diagnostics.Debug.WriteLine("Pi voln GetDoubleClickTime() dolo k chyb.");
                    doubleClickInterval = 500;
                }

                tmrDoubleClick.Interval = TimeSpan.FromMilliseconds(doubleClickInterval);
                tmrDoubleClick.Tick += new EventHandler(tmr_Tick);
            }

            for (int i = 0; i < buttons.Length; i++)
                buttons[i] = new ButtonState();
        }

        #region doubleClick

        private System.Windows.Threading.DispatcherTimer tmrDoubleClick = new System.Windows.Threading.DispatcherTimer();

        private void tmr_Tick(object sender, EventArgs e)
        {
            tmrDoubleClick.Stop();
            StopDoubleClicks();
        }

        private void StartTimer()
        {
            tmrDoubleClick.Start();
        }

        private void StopDoubleClicks()
        {
            for (int i = 0; i < buttons.Length; i++)
            {
                if (buttons[i].waitingForSecond) OnClick((MouseButton)i, buttons[i].startPoint);
                buttons[i].waitingForSecond = false;
            }
        }

        /// <summary>
        /// Zjist interval pro dvojklik
        /// </summary>
        private bool FindOutDoubleClickInterval()
        {
            try
            {
                doubleClickInterval = GetDoubleClickTime();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine("GetDoubleClickTime() thrown exception \n:" + ex.ToString());
                return false;
            }
            return true;
        }

        [DllImport("user32.dll")]
        private static extern uint GetDoubleClickTime();

        /// <summary>
        /// Zapne rozliovn mezi jednoduchm klikem a dvojklikem.
        /// Nastaven na true zpsob zpodn vykonn jednoduchho kliku (z dvodu ekn na mon druh klik)
        /// </summary>
        internal bool WaitForDoubleClick
        {
            get { return doubleClickEnabled; }
        }

        #endregion doubleClick

        #region toolTip

        private DispatcherTimer tmrToolTip = new DispatcherTimer();
        private bool toolTipVisible = false;

        private void PrepareToolTipTimer()
        {
            tmrToolTip.Interval = TimeSpan.FromSeconds(2.5);
            tmrToolTip.Start();
            tmrToolTip.Tick += new EventHandler(tmrToolTip_Tick);
        }

        private void tmrToolTip_Tick(object sender, EventArgs e)
        {
            if (ToolTipOpen != null && mouseIn)
            {
                toolTipVisible = true;
                ToolTipOpen(Mouse.GetPosition(element));
            }
        }

        private void ResetToolTip()
        {
            if (toolTipVisible)
            {
                if (ToolTipClose != null) ToolTipClose();
                toolTipVisible = false;
            }

            tmrToolTip.Stop();
            tmrToolTip.Start();
        }

        #endregion toolTip

        private bool mouseIn = false;

        private void element_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            OnMouseWheel(e.GetPosition(element), e.Delta / 120.0);
        }

        private void element_MouseEnter(object sender, MouseEventArgs e)
        {
            mouseIn = true;

            for (int i = 0; i < buttons.Length; i++)
            {
                buttons[i].isPressed = MouseButtonState.Released;
            }
        }

        private Point l;

        private void element_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            ResetToolTip();

            var b = buttons[(int)moveButton];
            var current = e.GetPosition(element);

            if (b.isPressed == MouseButtonState.Pressed)
            {
                if (IsNear(current, b.startPoint) == false)
                { b.moving = true; b.waitingForSecond = false; }

                OnMove(current - lastPoint);
            }
            lastPoint = current;
            l = current;
        }

        private void element_MouseLeave(object sender, MouseEventArgs e)
        {
            mouseIn = false;
        }

        private void element_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            var b = buttons[(int)e.ChangedButton];
            b.isPressed = MouseButtonState.Released;
            if (!doubleClickEnabled && !b.moving) OnClick(e.ChangedButton, e.GetPosition(element));
        }

        private void element_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            var current = e.GetPosition(element);
            var b = buttons[(int)e.ChangedButton];
            b.isPressed = MouseButtonState.Pressed;
            b.moving = false;

            if (b.waitingForSecond && IsNear(b.startPoint, current))
            {
                b.waitingForSecond = false;
                OnDoubleClick(e.ChangedButton, current);
                return;
            }

            StopDoubleClicks();
            b.startPoint = e.GetPosition(element);

            if (doubleClickEnabled) { b.waitingForSecond = true; StartTimer(); }
        }

        private bool IsNear(Point lastPoint, Point position)
        {
            return ((position - lastPoint).Length < 4);
        }

        internal MouseButton MoveButton
        {
            get { return moveButton; }
        }

        #region Handlers

        internal delegate void MoveHandler(Vector move);

        /// <param name="stepsCount">Poet krok = delta / 120. Bhem jednoho kroku by se ml odscrollovat poet dk nasteven v SystemInformation.MouseWheelScrollLines.</param>
        internal delegate void MouseWheelHandler(Point position, double stepsCount);

        internal delegate void OpenToolTipHandler(Point position);

        internal MouseClickHandler Click { get; set; }

        internal MouseClickHandler DoubleClick { get; set; }

        internal MouseClickHandler ClickLeft { get; set; }

        internal MouseClickHandler DoubleClickLeft { get; set; }

        internal MouseClickHandler ClickRight { get; set; }

        internal MouseClickHandler DoubleClickRight { get; set; }

        internal MouseClickHandler ClickMiddle { get; set; }

        internal MouseClickHandler DoubleClickMiddle { get; set; }

        internal MoveHandler Move { get; set; }

        internal MouseWheelHandler MouseWheel { get; set; }

        internal OpenToolTipHandler ToolTipOpen { get; set; }

        internal Action ToolTipClose { get; set; }

        private void OnDoubleClick(MouseButton mouseButton, Point point)
        {
            if (DoubleClick != null) DoubleClick(mouseButton, point);
            if (mouseButton == MouseButton.Left && DoubleClickLeft != null) DoubleClickLeft(mouseButton, point);
            if (mouseButton == MouseButton.Right && DoubleClickRight != null) DoubleClickRight(mouseButton, point);
            if (mouseButton == MouseButton.Middle && DoubleClickMiddle != null) DoubleClickMiddle(mouseButton, point);
        }

        protected void OnMove(Vector move)
        {
            if (Move != null) Move(move);
        }

        private void OnClick(MouseButton mouseButton, Point point)
        {
            if (Click != null) Click(mouseButton, point);
            if (mouseButton == MouseButton.Left && ClickLeft != null) ClickLeft(mouseButton, point);
            if (mouseButton == MouseButton.Right && ClickRight != null) ClickRight(mouseButton, point);
            if (mouseButton == MouseButton.Middle && ClickMiddle != null) ClickMiddle(mouseButton, point);
        }

        private void OnMouseWheel(Point position, double stepsCount)
        {
            if (MouseWheel != null) MouseWheel(position, stepsCount);
        }

        #endregion Handlers

        private class ButtonState
        {
            public bool waitingForSecond;
            public MouseButtonState isPressed;
            public Point startPoint;
            public bool moving;
        }
    }

    public delegate void MouseClickHandler(MouseButton button, Point point);
}