namespace Brjnk.CoordinateSystem2D
{
    using System;
    using System.Windows;
    using Brjnk.Common;

    /// <summary>
    /// </summary>
    public abstract class ViewInfoBase : UpdateableObject
    {
        /// Relativn souadnice - souadnice s potekm v levm hornm rohu zobrazen, meno v DPI
        /// Intern souadnice - linern souadnicov s
        /// Skuten souadnice - vlastn souadnicov s, kter je skuten zobrazena, nemus bt linern

        public ViewInfoBase(out SetPropertyHandler updater)
            : base(out updater, false)
        {
            Refresh();
        }

        protected void Refresh()
        {
            if ((object)AxisX == null || (object)AxisY == null) return;
            Center = TransformBack(CenterInternal);
            Vector v = new Vector(AreaWidthDip / (2 * ZoomX), AreaHeightDip / (2 * ZoomY));
            Vector v2 = new Vector(AreaWidthDip / (2 * ZoomX), -AreaHeightDip / (2 * ZoomY));
            Point tl = CenterInternal - v2; Point tr = CenterInternal + v;
            Point bl = CenterInternal - v; Point br = CenterInternal + v2;
            TopLeftInternal = tl;
            TopRightInternal = tr;
            BottomLeftInternal = bl;
            BottomRightInternal = br;
            TopLeft = TransformBack(tl);
            TopRight = TransformBack(tr);
            BottomLeft = TransformBack(bl);
            BottomRight = TransformBack(br);
            FromX = bl.X; FromY = bl.Y;
            ToX = tr.X; ToY = tr.Y;
            LeftToRight = tr.X - tl.X;
            TopToBottom = tl.Y - bl.Y;

            leftTop = CenterInternal - new Vector(AreaWidthDip / (2 * ZoomX), -AreaHeightDip / (2 * ZoomY));
            OnViewChanged();
        }

        private Point leftTop; //intern souadnice levho hornho rohu.

        /// <summary>
        /// Pevede z relativnch souadnic (s potkem v levm hornm rohu zobrazen) na intern souadnice
        /// </summary>
        public Point FromRelativeToInternal(Point relativePoint)
        {
            return new Point(leftTop.X + relativePoint.X / ZoomX, leftTop.Y - relativePoint.Y / ZoomY);
        }

        public double FromRelativeToInternalX(double x)
        {
            return leftTop.X + x / ZoomX;
        }

        public double FromRelativeToInternalY(double y)
        {
            return leftTop.Y - y / ZoomY;
        }

        /// <summary>
        /// Pevede z internch souadnic na relativn "obrazovkov souadnice"
        /// </summary>
        public Point FromInternalToRelative(Point internalPoint)
        {
            return new Point((internalPoint.X - leftTop.X) * ZoomX, -(internalPoint.Y - leftTop.Y) * ZoomY);
        }

        public double FromInternalToRelativeX(double x)
        {
            return (x - leftTop.X) * ZoomX;
        }

        public double FromInternalToRelativeY(double y)
        {
            return (-y + leftTop.Y) * ZoomY;
        }

        /// <summary>
        /// Pevede z internch souadnic na skuten
        /// </summary>
        public Point TransformBack(Point point)
        {
            return new Point(AxisX.TransformBack(point.X), AxisY.TransformBack(point.Y));
        }

        /// <summary>
        /// Pevede ze skutench souadnic na intern souadnice
        /// </summary>
        public Point Transform(Point point)
        {
            return new Point(AxisX.Transform(point.X), AxisY.Transform(point.Y));
        }

        /// <summary>
        /// Pevede relativn obrazovkov souadnice na skuten souadnice v soustav souadnic
        /// </summary>
        /// <returns></returns>
        public Point FromRelative(Point point)
        {
            return TransformBack(FromRelativeToInternal(point));
        }

        /// <summary>
        /// Pevede skuten souadnice na relativn souadnice WPF
        /// </summary>
        /// <returns></returns>
        public Point ToRelative(Point point)
        {
            return FromInternalToRelative(Transform(point));
        }

        /// <summary>
        /// Pevede Size v DIP na velikost v internch souadnicch
        /// </summary>
        public Size GetInternalSize(Size size)
        {
            return new Size(size.Width / ZoomX, size.Height / ZoomY);
        }

        public Size GetDipSize(Size internalSize)
        {
            return new Size(internalSize.Width * ZoomX, internalSize.Height * ZoomY);
        }

        public Point CenterInternal
        {
            get { return Definition.Center; }
        }

        public double ZoomX
        {
            get { return Definition.ZoomX; }
        }

        public double ZoomY
        {
            get { return Definition.ZoomY; }
        }

        public void SetLeftBottomCorner(Point point, double zoomX, double zoomY)
        {
            SetView(point + new Vector(this.LeftToRight / (zoomX * 2), this.TopToBottom / (zoomY * 2)), zoomX, zoomY);
        }

        public void SetLeftTopCorner(Point point, double zoomX, double zoomY)
        {
            SetView(point + new Vector(this.LeftToRight / 2, -this.TopToBottom / 2), zoomX, zoomY);
        }

        public void SetRightTopCorner(Point point, double zoomX, double zoomY)
        {
            SetView(point + new Vector(-this.LeftToRight / 2, -this.TopToBottom / 2), zoomX, zoomY);
        }

        public void SetRightBottomCorner(Point point, double zoomX, double zoomY)
        {
            SetView(point + new Vector(-this.LeftToRight / 2, this.TopToBottom / 2), zoomX, zoomY);
        }

        public void SetZoom(double zoom)
        {
            SetZoom(zoom, zoom);
        }

        public void SetCenter(Point point)
        {
            SetView(Transform(point), Definition.ZoomX, Definition.ZoomY);
        }

        protected abstract void SetView(Point center, double zoomX, double zoomY);

        public abstract void SetZoom(double zoomX, double zoomY);

        #region properties

        internal ViewDefinition Definition
        {
            get { return (ViewDefinition)GetValue(DefinitionProperty); }
            set { SetValue(DefinitionPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey DefinitionPropertyKey =
            DependencyProperty.RegisterReadOnly("Definition", typeof(ViewDefinition), typeof(ViewInfoBase),
            new FrameworkPropertyMetadata(new ViewDefinition(new Point(0, 0), 1, 1)) { PropertyChangedCallback = DefinitionChanged });

        protected static readonly DependencyProperty DefinitionProperty = DefinitionPropertyKey.DependencyProperty;

        public Point Center
        {
            get { return (Point)GetValue(CenterProperty); }
            protected set { SetValue(CenterPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey CenterPropertyKey =
            DependencyProperty.RegisterReadOnly("Center", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty CenterProperty = CenterPropertyKey.DependencyProperty;

        public Point TopLeft
        {
            get { return (Point)GetValue(TopLeftProperty); }
            protected set { SetValue(TopLeftPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey TopLeftPropertyKey =
            DependencyProperty.RegisterReadOnly("TopLeft", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty TopLeftProperty = TopLeftPropertyKey.DependencyProperty;

        public Point TopRight
        {
            get { return (Point)GetValue(TopRightProperty); }
            protected set { SetValue(TopRightPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey TopRightPropertyKey =
            DependencyProperty.RegisterReadOnly("TopRight", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty TopRightProperty = TopRightPropertyKey.DependencyProperty;

        public Point BottomLeft
        {
            get { return (Point)GetValue(BottomLeftProperty); }
            protected set { SetValue(BottomLeftPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey BottomLeftPropertyKey =
            DependencyProperty.RegisterReadOnly("BottomLeft", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty BottomLeftProperty = BottomLeftPropertyKey.DependencyProperty;

        public Point BottomRight
        {
            get { return (Point)GetValue(BottomRightProperty); }
            protected set { SetValue(BottomRightPropertyKey, value); }
        }

        private static readonly DependencyPropertyKey BottomRightPropertyKey =
            DependencyProperty.RegisterReadOnly("BottomRight", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        protected static readonly DependencyProperty BottomRightProperty = BottomRightPropertyKey.DependencyProperty;

        public Point TopRightInternal
        {
            get { return (Point)GetValue(TopRightInternalProperty); }
            protected set { SetValue(TopRightInternalPropertyKey, value); }
        }

        private static readonly DependencyPropertyKey TopRightInternalPropertyKey =
            DependencyProperty.RegisterReadOnly("TopRightInternal", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty TopRightInternalProperty = TopRightInternalPropertyKey.DependencyProperty;

        public Point BottomLeftInternal
        {
            get { return (Point)GetValue(BottomLeftInternalProperty); }
            protected set { SetValue(BottomLeftInternalPropertyKey, value); }
        }

        private static readonly DependencyPropertyKey BottomLeftInternalPropertyKey =
            DependencyProperty.RegisterReadOnly("BottomLeftInternal", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty BottomLeftInternalProperty = BottomLeftInternalPropertyKey.DependencyProperty;

        public Point TopLeftInternal
        {
            get { return (Point)GetValue(TopLeftInternalProperty); }
            protected set { SetValue(TopLeftInternalPropertyKey, value); }
        }

        private static readonly DependencyPropertyKey TopLeftInternalPropertyKey =
            DependencyProperty.RegisterReadOnly("TopLeftInternal", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty TopLeftInternalProperty = TopLeftInternalPropertyKey.DependencyProperty;

        public Point BottomRightInternal
        {
            get { return (Point)GetValue(BottomRightInternalProperty); }
            protected set { SetValue(BottomRightInternalPropertyKey, value); }
        }

        private static readonly DependencyPropertyKey BottomRightInternalPropertyKey =
            DependencyProperty.RegisterReadOnly("BottomRightInternal", typeof(Point), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty BottomRightInternalProperty = BottomRightInternalPropertyKey.DependencyProperty;

        /// <summary>
        /// Potek osy X v netransformovanch souadnicch
        /// </summary>
        public double FromX
        {
            get { return (double)GetValue(FromXProperty); }
            protected set { SetValue(FromXPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey FromXPropertyKey =
            DependencyProperty.RegisterReadOnly("FromX", typeof(double), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty FromXProperty = FromXPropertyKey.DependencyProperty;

        /// <summary>
        /// Konec osy X v netransformovanch souadnicch
        /// </summary>
        public double ToX
        {
            get { return (double)GetValue(ToXProperty); }
            protected set { SetValue(ToXPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey ToXPropertyKey =
            DependencyProperty.RegisterReadOnly("ToX", typeof(double), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty ToXProperty = ToXPropertyKey.DependencyProperty;

        /// <summary>
        /// Potek osy Y v netransformovanch souadnicch
        /// </summary>
        public double FromY
        {
            get { return (double)GetValue(FromYProperty); }
            protected set { SetValue(FromYPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey FromYPropertyKey =
            DependencyProperty.RegisterReadOnly("FromY", typeof(double), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty FromYProperty = FromYPropertyKey.DependencyProperty;

        /// <summary>
        /// Konec osy Y v netransformovanch souadnicch
        /// </summary>
        public double ToY
        {
            get { return (double)GetValue(ToYProperty); }
            protected set { SetValue(ToYPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey ToYPropertyKey =
            DependencyProperty.RegisterReadOnly("ToY", typeof(double), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty ToYProperty = ToYPropertyKey.DependencyProperty;

        public AxisDefinition AxisX
        {
            get { return (AxisDefinition)GetValue(AxisXProperty); }
            protected set { SetValue(AxisXPropertyKey, value); }
        }

        [OutsideUpdateable]
        protected static readonly DependencyPropertyKey AxisXPropertyKey =
            DependencyProperty.RegisterReadOnly("AxisX", typeof(AxisDefinition), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty AxisXProperty = AxisXPropertyKey.DependencyProperty;

        public AxisDefinition AxisY
        {
            get { return (AxisDefinition)GetValue(AxisYProperty); }
            protected set { SetValue(AxisYPropertyKey, value); }
        }

        [OutsideUpdateable]
        protected static readonly DependencyPropertyKey AxisYPropertyKey =
            DependencyProperty.RegisterReadOnly("AxisY", typeof(AxisDefinition), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty AxisYProperty = AxisYPropertyKey.DependencyProperty;

        /// <summary>
        /// ka mezi levm a pravm okrajem (tj. rozdl mezi x souadnic levho rohu a pravho rohu)
        /// Intern souadnice
        /// </summary>
        public double LeftToRight
        {
            get { return (double)GetValue(LeftToRightProperty); }
            protected set { SetValue(LeftToRightPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey LeftToRightPropertyKey =
            DependencyProperty.RegisterReadOnly("LeftToRight", typeof(double), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty LeftToRightProperty = LeftToRightPropertyKey.DependencyProperty;

        /// <summary>
        /// Dlka mezi hornm a dolnm okrajem (tj. rozdl mezi y souadnic hornho rohu a dolnho rohu)
        /// Intern souadnice
        /// </summary>
        public double TopToBottom
        {
            get { return (double)GetValue(TopToBottomProperty); }
            protected set { SetValue(TopToBottomPropertyKey, value); }
        }

        protected static readonly DependencyPropertyKey TopToBottomPropertyKey =
            DependencyProperty.RegisterReadOnly("TopToBottom", typeof(double), typeof(ViewInfoBase), new PropertyMetadata());

        public static readonly DependencyProperty TopToBottomProperty = TopToBottomPropertyKey.DependencyProperty;

        /// <summary>
        /// ka zobrazen sti soustavy souadnic v Dip
        /// </summary>
        public double AreaWidthDip
        {
            get { return (double)GetValue(AreaWidthDipProperty); }
            protected set { SetValue(AreaWidthDipPropertyKey, value); }
        }

        [OutsideUpdateable]
        protected static readonly DependencyPropertyKey AreaWidthDipPropertyKey =
            DependencyProperty.RegisterReadOnly("AreaWidthDip", typeof(double), typeof(ViewInfoBase), new PropertyMetadata(0.0, Resized));

        public static readonly DependencyProperty AreaWidthDipProperty = AreaWidthDipPropertyKey.DependencyProperty;

        /// <summary>
        /// Vka zobrazen sti soustavy souadnic v Dip
        /// </summary>
        public double AreaHeightDip
        {
            get { return (double)GetValue(AreaHeightDipProperty); }
            protected set { SetValue(AreaHeightDipPropertyKey, value); }
        }

        [OutsideUpdateable]
        protected static readonly DependencyPropertyKey AreaHeightDipPropertyKey =
            DependencyProperty.RegisterReadOnly("AreaHeightDip", typeof(double), typeof(ViewInfoBase), new PropertyMetadata(0.0, Resized));

        public static readonly DependencyProperty AreaHeightDipProperty = AreaHeightDipPropertyKey.DependencyProperty;

        private static void Resized(object sender, DependencyPropertyChangedEventArgs args)
        {
            var target = (sender as ViewInfoBase);
            target.Refresh();
            target.OnChangeCompleted();
        }

        private static void DefinitionChanged(object sender, DependencyPropertyChangedEventArgs args)
        {
            (sender as ViewInfoBase).Refresh();
        }

        #endregion properties

        /// <summary>
        /// Udlost nastane pi zmn pohledu, nap. posunut, zoomovn nebo zmna velikosti pohledu.
        /// </summary>
        public event EventHandler<EventArgs> ViewChanged;

        /// <summary>
        /// Udlost je volna pi zmn transformace nebo jejch parametru.
        /// </summary>
        public event EventHandler<EventArgs> TransformationChanged;

        /// <summary>
        /// Udlost je vyvolna pi dokonen zmny. Nap. pi dokonen animovn zoomu.
        /// Tato udlost je (na rozdl od ViewChanged) vhodn k provdn asov nronjch aktualizac)
        /// </summary>
        public event EventHandler<EventArgs> ChangeCompleted;

        protected void OnViewChanged()
        {
            if (ViewChanged != null) ViewChanged(this, EventArgs.Empty);
        }

        protected void OnTransformationChanged()
        {
            if (TransformationChanged != null) TransformationChanged(this, EventArgs.Empty);
        }

        protected void OnChangeCompleted()
        {
            if (ChangeCompleted != null) ChangeCompleted(this, EventArgs.Empty);
        }

        public struct ViewDefinition
        {
            public ViewDefinition(Point center, double zoomX, double zoomY)
                : this()
            {
                this.Center = center;
                this.ZoomX = zoomX;
                this.ZoomY = zoomY;
            }

            public Point Center { get; private set; }

            private double _zoomX;

            public double ZoomX
            {
                get
                { return _zoomX; }
                private set
                { _zoomX = value; }
            }

            private double _zoomY;

            public double ZoomY
            {
                get
                { return _zoomY; }
                private set
                { _zoomY = value; }
            }
        }
    }
}