namespace Brjnk.GpxExplorer.Data.GPX
{
    using System;
    using System.Collections.Generic;
    using System.Windows.Media;

    /// <summary>
    /// Zajiuje konverzi WaypointSymbol a Color ze souboru gpx
    /// </summary>
    public static class Converter
    {
        private static List<Tuple<string, WaypointSymbol>> symbols;

        static Converter()
        {
            PrepareSymbols();

            PrepareColors();
        }

        private static void PrepareSymbols()
        {
            var stream = System.Windows.Application.GetResourceStream(new Uri(@"pack://application:,,,/GpxExplorer;Component/Data/GPX/SymbolsConversion.txt"));
            var reader = new System.IO.StreamReader(stream.Stream);
            symbols = new List<Tuple<string, WaypointSymbol>>();
            while (reader.EndOfStream == false)
            {
                var line = reader.ReadLine();
                if (string.IsNullOrEmpty(line) || line.Contains(";") == false) continue;
                var parts = line.Split(';');
                symbols.Add(new Tuple<string, WaypointSymbol>(parts[0], (WaypointSymbol)Enum.Parse(typeof(WaypointSymbol), parts[1])));
            }
            reader.Dispose();
        }

        private static void PrepareColors()
        {
            colors.Add(new Tuple<string, Color>("Black", Colors.Black));
            colors.Add(new Tuple<string, Color>("DarkRed", Colors.DarkRed));
            colors.Add(new Tuple<string, Color>("DarkGreen", Colors.DarkGreen));
            colors.Add(new Tuple<string, Color>("DarkYellow", Colors.Gold));
            colors.Add(new Tuple<string, Color>("DarkBlue", Colors.DarkBlue));
            colors.Add(new Tuple<string, Color>("DarkMagenta", Colors.DarkMagenta));
            colors.Add(new Tuple<string, Color>("DarkCyan", Colors.DarkCyan));
            colors.Add(new Tuple<string, Color>("LightGray", Colors.LightGray));
            colors.Add(new Tuple<string, Color>("DarkGray", Colors.DarkGray));
            colors.Add(new Tuple<string, Color>("Red", Colors.Red));
            colors.Add(new Tuple<string, Color>("Green", Colors.Green));
            colors.Add(new Tuple<string, Color>("Yellow", Colors.Yellow));
            colors.Add(new Tuple<string, Color>("Blue", Colors.Blue));
            colors.Add(new Tuple<string, Color>("Magenta", Colors.Magenta));
            colors.Add(new Tuple<string, Color>("Cyan", Colors.Cyan));
            colors.Add(new Tuple<string, Color>("White", Colors.White));
            colors.Add(new Tuple<string, Color>("Transparent", Colors.Transparent));
        }

        public static WaypointSymbol StringToSymbol(string str)
        {
            foreach (var i in symbols)
            {
                if (i.Item1 == str) return i.Item2;
            }
            return WaypointSymbol.Unknown;
        }

        public static string SymbolToString(WaypointSymbol symbol)
        {
            foreach (var i in symbols)
            {
                if (i.Item2 == symbol) return i.Item1;
            }
            return string.Empty;
        }

        private static List<Tuple<string, Color>> colors = new List<Tuple<string, Color>>();

        public static Color StringToColor(string str)
        {
            foreach (var c in colors)
            {
                if (c.Item1 == str) return c.Item2;
            }
            return Colors.Black;
        }

        public static string ColorToString(Color color)
        {
            var nearest = colors[0];
            double d = GetDistance(colors[0].Item2, color);
            for (int i = 1; i < colors.Count; i++)
            {
                double a = GetDistance(colors[i].Item2, color);
                if (a < d) { d = a; nearest = colors[i]; }
            }
            return nearest.Item1;
        }

        private static double GetDistance(Color color, Color color2)
        {
            return Math.Sqrt(Math.Pow(color.R - color2.R, 2) + Math.Pow(color.G - color2.G, 2) + Math.Pow(color.B - color2.B, 2));
        }

        internal static Color StringToColor(SerializationClasses.DisplayColor_t color)
        {
            return StringToColor(color.ToString());
        }
    }
}