namespace Brjnk.GpxExplorer.Data.GPX
{
    using System;
    using System.Linq;
    using System.Windows.Media;
    using Brjnk.GpxExplorer.Data.GPX.SerializationClasses;

    /// <summary>
    /// </summary>
    public class GpxRoute : Route, IGpxObject
    {
        private rteType data;

        public GpxRoute(rteType data)
        {
            this.data = data;
            Points = new MVVM.ObservableCollectionEx<RoutePoint>();

            if (data.rtept == null) data.rtept = new System.Collections.Generic.List<wptType>();

            foreach (var p in data.rtept)
            {
                Points.Add(new GpxRoutePoint(p));
            }
        }

        public GpxRoute() : this(new rteType()) { }

        public object GetUnderlayingObject()
        {
            data.rtept.Clear();
            foreach (var p in Points)
            {
                wptType o;
                if (p is GpxRoutePoint)
                {
                    o = (wptType)(p as GpxRoutePoint).GetUnderlayingObject();
                }
                else
                {
                    o = (wptType)p.ConvertTo<GpxRoutePoint>().GetUnderlayingObject();
                }
                data.rtept.Add(o);
            }
            return data;
        }

        public override string Name
        {
            get
            {
                return data.name;
            }
            set
            {
                data.name = value;
                OnPropertyChanged("Name");
            }
        }

        public override string Comment
        {
            get
            {
                return data.cmt;
            }
            set
            {
                data.cmt = value;
                OnPropertyChanged("Comment");
            }
        }

        public override Link Link
        {
            get
            {
                if (data.link == null || data.link.Count == 0) return null;
                return new Link(data.link.First().text, data.link.First().href);
            }
            set
            {
                if (value == null) data.link = null;
                else
                {
                    linkType l = new linkType();
                    l.href = value.Url;
                    l.text = value.Text;
                    if (data.link.Count == 0) data.link.Add(l);
                    else data.link[0] = l;
                }
                OnPropertyChanged("Link");
            }
        }

        public override string Description
        {
            get
            {
                return data.desc;
            }
            set
            {
                data.desc = value;
                OnPropertyChanged("Description");
            }
        }

        //Barva je uloena i v tto promn, protoe ve formtu Gpx nelze reprezentovat vechny barvy
        //Po uloen dat do gpx je vybrna nejbli mon barva (setter vlastnosti Color)
        private Color? color;

        public override Color? Color
        {
            get
            {
                if (color != null) return color;
                if (data.extensions == null || data.extensions.routeExtension == null) return null;
                else if (data.extensions.routeExtension.DisplayColorSpecified == false) return null;
                else return Converter.StringToColor(data.extensions.routeExtension.DisplayColor);
            }
            set
            {
                color = value;

                if (color.HasValue)
                {
                    if (data.extensions == null) data.extensions = new routeExtensionsType();
                    if (data.extensions.routeExtension == null) data.extensions.routeExtension = new RouteExtension_t();
                    data.extensions.routeExtension.DisplayColorSpecified = true;
                    data.extensions.routeExtension.DisplayColor = (DisplayColor_t)Enum.Parse(typeof(DisplayColor_t), Converter.ColorToString(value.Value));
                }
                else
                {
                    if (data.extensions != null && data.extensions.routeExtension != null)
                        data.extensions.routeExtension.DisplayColorSpecified = false;
                }
                OnPropertyChanged("Color");
            }
        }
    }
}