namespace Brjnk.GpxExplorer.Data.GPX.SerializationClasses
{
    using System;
    using System.Collections.Generic;

    /// <summary>
    ///
    /// </summary>
    /// <typeparam name="Cover">Obal</typeparam>
    /// <typeparam name="Wrapped">Obalovan typ</typeparam>
    public class CollectionWrapper<Cover, Wrapped> : IList<Cover> where Cover : XmlClassWrapper<Wrapped>
    {
        private List<Wrapped> data;

        public CollectionWrapper(List<Wrapped> data)
        {
            if ((object)data == null) throw new ArgumentNullException();
            this.data = data;
        }

        public List<Wrapped> Data
        {
            get { return data; }
            set { data = value; }
        }

        public int IndexOf(Cover item)
        {
            return data.IndexOf(item.WrappedData);
        }

        public void Insert(int index, Cover item)
        {
            data.Insert(index, item.WrappedData);
        }

        public void RemoveAt(int index)
        {
            data.RemoveAt(index);
        }

        public Cover this[int index]
        {
            get
            {
                return Activator.CreateInstance(typeof(Cover), new object[] { data[index] }) as Cover;
            }
            set
            {
                data[index] = value.WrappedData;
            }
        }

        public void Add(Cover item)
        {
            data.Add(item.WrappedData);
        }

        public void Clear()
        {
            data.Clear();
        }

        public bool Contains(Cover item)
        {
            return data.Contains(item.WrappedData);
        }

        public void CopyTo(Cover[] array, int arrayIndex)
        {
            throw new NotImplementedException("Tato metoda nen implementovna na CollectionWrapper");
            //data.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get { return data.Count; }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public bool Remove(Cover item)
        {
            return data.Remove(item.WrappedData);
        }

        public IEnumerator<Cover> GetEnumerator()
        {
            return new CollectionWrapperEnumerator<Cover>(this);
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this.GetEnumerator();
        }

        protected class CollectionWrapperEnumerator<T> : IEnumerator<T>
        {
            private int index = -1;
            private IList<T> collection;

            public CollectionWrapperEnumerator(IList<T> collection)
            {
                this.collection = collection;
            }

            public T Current
            {
                get { return collection[index]; }
            }

            public void Dispose()
            {
            }

            object System.Collections.IEnumerator.Current
            {
                get { return this.Current; }
            }

            public bool MoveNext()
            {
                index++;
                return index < collection.Count;
            }

            public void Reset()
            {
                index = -1;
            }
        }
    }
}