namespace Brjnk.GpxExplorer.Data
{
    using System;
    using System.ComponentModel;
    using System.Windows.Media;
    using Brjnk.MVVM;

    /// <summary>
    /// </summary>
    public abstract class Route : INotifyPropertyChanged
    {
        // Pi upravovn tto tdy mus bt nov vlstnosti pidny do metody ConvertTo
        // Metoda ConvertTo zajiuje konverzi mezi rznmi potomky tto tdy (nap. pokud jsou data z rznch zdroj / typ souboru)
        // Metoda ConvertTo zkopruje vlastnosti definovan touto tdou

        public abstract string Name { get; set; }

        public abstract string Comment { get; set; }

        public abstract Link Link { get; set; }

        public abstract string Description { get; set; }

        public abstract Color? Color { get; set; }

        private ObservableCollectionEx<RoutePoint> points = new ObservableCollectionEx<RoutePoint>();

        public ObservableCollectionEx<RoutePoint> Points
        {
            get { return points; }
            set { points = value; OnPropertyChanged("Points"); }
        }

        public T ConvertTo<T>() where T : Route
        {
            var instance = Activator.CreateInstance(typeof(T)) as Route;
            instance.Name = Name;
            instance.Comment = Comment;
            instance.Link = Link;
            instance.Description = Description;
            CloneItems(instance);
            return instance as T;
        }

        private void CloneItems(Route instance)
        {
            instance.Points = new ObservableCollectionEx<RoutePoint>();
            foreach (var item in Points)
            {
                instance.Points.Add((RoutePoint)item.Clone());
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}