namespace Brjnk.GpxExplorer.Data
{
    using System;
    using System.ComponentModel;
    using System.Windows.Media;
    using Brjnk.MVVM;

    /// <summary>
    /// Pedstavuje stopu.
    /// Zddn tdy mus mt bezparametrick konstruktor.
    /// </summary>
    public abstract class Track : INotifyPropertyChanged
    {
        // Pi upravovn tto tdy mus bt nov vlstnosti pidny do metody ConvertTo
        // Metoda ConvertTo zajiuje konverzi mezi rznmi potomky tto tdy (nap. pokud jsou data z rznch zdroj / typ souboru)
        // Metoda ConvertTo zkopruje vlastnosti definovan touto tdou

        public abstract string Name { get; set; }

        public abstract string Comment { get; set; }

        public abstract Link Link { get; set; }

        public abstract string Description { get; set; }

        public abstract Color? Color { get; set; }

        private ObservableCollectionEx<TrackSegment> segments;

        public ObservableCollectionEx<TrackSegment> Segments
        {
            get { return segments; }
            set { segments = value; OnPropertyChanged("Segments"); }
        }

        public T ConvertTo<T>() where T : Track
        {
            var instance = Activator.CreateInstance(typeof(T)) as Track;
            instance.Name = Name;
            instance.Comment = Comment;
            instance.Link = Link;
            instance.Description = Description;
            instance.Color = Color;
            CloneItems(instance);
            return instance as T;
        }

        private void CloneItems(Track instance)
        {
            instance.Segments = new ObservableCollectionEx<TrackSegment>();
            foreach (var item in Segments)
            {
                instance.Segments.Add((TrackSegment)item.Clone());
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}