namespace Brjnk.GpxExplorer.Data
{
    using System;
    using System.ComponentModel;
    using Brjnk.Common;

    /// <summary>
    /// </summary>
    public abstract class Waypoint : INotifyPropertyChanged
    {
        // Pi upravovn tto tdy mus bt nov vlstnosti pidny do metody ConvertTo
        // Metoda ConvertTo zajiuje konverzi mezi rznmi potomky tto tdy (nap. pokud jsou data z rznch zdroj / typ souboru)
        // Metoda ConvertTo zkopruje vlastnosti definovan touto tdou

        public abstract GpsCoordinates Position { get; set; }

        public abstract string Name { get; set; }

        public abstract string Comment { get; set; }

        public abstract Distance? Elevation { get; set; }

        public abstract Link Link { get; set; }

        public abstract string Description { get; set; }

        public abstract DateTime? Time { get; set; }

        public abstract GeocacheInfo Geocache { get; set; }

        public abstract WaypointSymbol? Symbol { get; set; }

        public T ConvertTo<T>() where T : Waypoint
        {
            var instance = Activator.CreateInstance(typeof(T)) as Waypoint;
            instance.Position = Position;
            instance.Name = Name;
            instance.Comment = Comment;
            instance.Elevation = Elevation;
            instance.Link = Link;
            instance.Description = Description;
            instance.Geocache = Geocache;
            instance.Symbol = Symbol;
            instance.Time = Time;
            return instance as T;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}