﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using Brjnk.Common;

namespace Brjnk.GpxExplorer.UI
{
    /// <summary>
    /// Interaction logic for SearchPositionArgsEditor.xaml
    /// </summary>
    public partial class SearchPositionArgsEditor : UserControl
    {
        public SearchPositionArgsEditor()
        {
            InitializeComponent();
            this.LostFocus += new RoutedEventHandler(SearchPositionArgsEditor_LostFocus);

            mainGrd.DataContext = this;
            mainGrd.BindingGroup = new System.Windows.Data.BindingGroup();
            mainGrd.BindingGroup.ValidationRules.Add(new Vr());
        }

        private void SearchPositionArgsEditor_LostFocus(object sender, RoutedEventArgs e)
        {
            mainGrd.BindingGroup.CommitEdit();
            LimitValidationFailed = (Validation.GetHasError(mainGrd));
        }

        public Distance Limit
        {
            get { return (Distance)GetValue(LimitProperty); }
            set { SetValue(LimitProperty, value); }
        }

        public static readonly DependencyProperty LimitProperty =
            DependencyProperty.Register("Limit", typeof(Distance), typeof(SearchPositionArgsEditor), new UIPropertyMetadata(new Distance(500), Changed));

        public Distance SearchAgainLimit
        {
            get { return (Distance)GetValue(SearchAgainLimitProperty); }
            set { SetValue(SearchAgainLimitProperty, value); }
        }

        public static readonly DependencyProperty SearchAgainLimitProperty =
            DependencyProperty.Register("SearchAgainLimit", typeof(Distance), typeof(SearchPositionArgsEditor), new UIPropertyMetadata(new Distance(1000), Changed));

        public bool SearchInGaps
        {
            get { return (bool)GetValue(SearchInGapsProperty); }
            set { SetValue(SearchInGapsProperty, value); }
        }

        public static readonly DependencyProperty SearchInGapsProperty =
            DependencyProperty.Register("SearchInGaps", typeof(bool), typeof(SearchPositionArgsEditor), new UIPropertyMetadata(true));

        public bool LimitValidationFailed
        {
            get { return (bool)GetValue(LimitValidationFailedProperty); }
            protected set { SetValue(LimitValidationFailedPropertyKey, value); }
        }

        private static readonly DependencyPropertyKey LimitValidationFailedPropertyKey =
            DependencyProperty.RegisterReadOnly("LimitValidationFailed", typeof(bool), typeof(SearchPositionArgsEditor), new PropertyMetadata(false));

        public static readonly DependencyProperty LimitValidationFailedProperty = LimitValidationFailedPropertyKey.DependencyProperty;

        private static void Changed(object sender, DependencyPropertyChangedEventArgs e)
        {
        }

        private class Vr : ValidationRule
        {
            public override ValidationResult Validate(object value, System.Globalization.CultureInfo cultureInfo)
            {
                var bg = value as BindingGroup;
                object newLimit, newSearchAgainLimit;

                if (!bg.TryGetValue(bg.Items[0], "Limit", out newLimit)) System.Diagnostics.Debug.WriteLine("Unknown fail");
                if (!bg.TryGetValue(bg.Items[0], "SearchAgainLimit", out newSearchAgainLimit)) System.Diagnostics.Debug.WriteLine("Unknown fail");

                Distance newLimitD = Distance.Parser.Parse(newLimit as string);
                Distance newSearchAgainLimitD = Distance.Parser.Parse(newSearchAgainLimit as string);

                if (newLimitD > newSearchAgainLimitD) return new ValidationResult(false, null);

                return ValidationResult.ValidResult;
            }
        }
    }
}