namespace Brjnk.GpxExplorer.ViewModel
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Linq;
    using System.Windows.Media;
    using System.Windows.Media.Imaging;
    using Brjnk.CoordinateSystem2D.Layers;
    using Brjnk.MVVM;

    /// <summary>
    /// </summary>
    public class Photo : DataItem, INotifyPropertyChanged, ISelectable
    {
        private const int fullPhotoCacheCount = 5;

        /// <summary>
        /// Vyrovnvac pam pro obrzky v pln velikosti. Je sdlen mezi instancemi
        /// </summary>
        private static List<Tuple<string, ImageSource>> fullPhotoCache = new List<Tuple<string, ImageSource>>();

        public ObservableCollectionEx<LocatedPhoto> Locations
        { get { return GetWatchedProperty<ObservableCollectionEx<LocatedPhoto>>("Locations"); } }

        private Uri source;

        public int PixelWidth { get; protected set; }

        public int PixelHeight { get; protected set; }

        public double Widht { get; protected set; }

        public double Height { get; protected set; }

        public DateTime CreationTime { get; protected set; }

        public DateTime CorrectedCreationTime { get { return CreationTime + TimeCorrection; } }

        public string FileName { get; protected set; }

        public string FullName { get; protected set; }

        public ImageSource Thumbnail { get; protected set; }

        public bool Fail { get; private set; }

        /// <summary>
        /// True, pokud datum vytvoen fotografie byl uren z vlastnosti souboru, nikoliv z EXIF dat.
        /// </summary>
        public bool FileCreationTimeUsed { get; protected set; }

        private TimeSpan timeCorrection;

        public TimeSpan TimeCorrection
        {
            get { return timeCorrection; }
            set { timeCorrection = value; OnPropertyChanged("TimeCorrection"); OnPropertyChanged("CorrectedCreationTime"); }
        }

        private bool isVisible = true;

        public bool IsVisible
        {
            get { return isVisible; }
            set { isVisible = value; OnPropertyChanged("IsVisible"); }
        }

        private PhotoOrientation orientation;

        public PhotoOrientation Orientation
        {
            get { return orientation; }
            set { orientation = value; OnPropertyChanged("Orientation"); }
        }

        public ImageSource PhotoFull
        {
            //Tato vlastnost me bt volna v jinm vlkn (pi asynchronm Bindingu)
            get
            {
                var fromCache = fullPhotoCache.FirstOrDefault(t => string.Equals(FullName, t.Item1, StringComparison.InvariantCultureIgnoreCase));
                if (fromCache != null) return fromCache.Item2;

                var bitmap = new BitmapImage(new Uri(FullName));
                bitmap.Freeze();
                fullPhotoCache.Add(new Tuple<string, ImageSource>(FullName, bitmap));
                if (fullPhotoCache.Count > fullPhotoCacheCount) fullPhotoCache.RemoveAt(0);
                return bitmap;
            }
        }

        public Photo(Uri source) : this(source, TimeSpan.Zero) { }

        public Photo(Uri source, TimeSpan timeCorrection)
        {
            DefineWatchedProperty("Locations", new ObservableCollectionEx<LocatedPhoto>());

            this.source = source;
            this.TimeCorrection = timeCorrection;
            BitmapFrame frame;
            try
            {
                frame = BitmapFrame.Create(source);
            }
            catch
            {
                Fail = true;
                return;
            }
            PixelHeight = frame.PixelHeight;
            PixelWidth = frame.PixelWidth;
            Widht = frame.Width;
            Height = frame.Height;
            FileName = System.IO.Path.GetFileName(source.LocalPath);
            FullName = source.LocalPath;

            try
            {
                var metadata = frame.Metadata as BitmapMetadata;
                CreationTime = DateTime.Parse(metadata.DateTaken);
                if (metadata.ContainsQuery(@"/app1/ifd/exif:{uint=274}"))
                {
                    Orientation = (PhotoOrientation)(System.UInt16)metadata.GetQuery(@"/app1/ifd/exif:{uint=274}");
                }
            }
            catch
            {
                if (CreationTime == default(DateTime))
                {
                    CreationTime = new System.IO.FileInfo(source.LocalPath).CreationTime;
                    FileCreationTimeUsed = true;
                }
            }
            LoadThumbnail(FullName);
        }

        private void LoadThumbnail(string path)
        {
            double scale = Math.Min(256 / (double)PixelWidth, 256 / (double)PixelHeight);
            var b = new BitmapImage();
            b.BeginInit();

            var thumb = new BitmapImage();
            thumb.BeginInit();
            thumb.UriSource = new Uri(path);
            thumb.DecodePixelHeight = (int)(PixelHeight * scale);
            thumb.DecodePixelWidth = (int)(PixelWidth * scale);

            thumb.EndInit();
            Thumbnail = thumb;
            // Thumbnail = new WriteableBitmap(new TransformedBitmap(frame, new ScaleTransform(scale, scale)));
        }

        public BitmapImage LoadPhoto()
        {
            BitmapImage b = new BitmapImage(source);
            return b;
        }

        private bool isSelected;

        public bool IsSelected
        {
            get { return isSelected; }
            set { isSelected = value; OnPropertyChanged("IsSelected"); }
        }
    }
}