namespace Brjnk.GpxExplorer.ViewModel
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Windows.Media;
    using Brjnk.Common;
    using Brjnk.GpxExplorer.Computing;
    using Brjnk.GpxExplorer.Data;
    using Brjnk.MVVM;

    public class RouteVM : Wrapper<Route>, ISearchablePath, IGpsData, ISelectable, IHideable
    {
        static RouteVM()
        {
            WrapProperty(new PropertyWrapperInfo<Route>("Points",
                r => r.Points, null, o => new CollectionWrapper<RoutePoint, RoutePointVM>((ObservableCollectionEx<RoutePoint>)o), false));
        }

        public RouteVM(Route source)
            : base(source)
        {
            this.Changed += new System.EventHandler<ChangedEventArgs>(RouteVM_Changed);
            if (source.Color.HasValue == false) color = AutoColorSelector.GetNextColor();
            Recalculate();
        }

        private void RouteVM_Changed(object sender, ChangedEventArgs e)
        {
            if (e.Type == ChangeType.ChildAdded || e.Type == ChangeType.ChildRemoved || e.Type == ChangeType.Reset)
                Recalculate();
            else if (e.BeginsPathWith("Points", "Source"))
                Recalculate();
        }

        public CollectionWrapper<RoutePoint, RoutePointVM> Points
        {
            get { return (CollectionWrapper<RoutePoint, RoutePointVM>)GetWrapper("Points"); }
        }

        public Distance Distance { get; private set; }

        private void Recalculate()
        {
            var context = new RouteCalculationContext();

            foreach (var point in Points)
            {
                context.AddPoint(point);

                point.Azimuth = context.Azimuth;
                point.Distance = context.Distance;
                point.DistanceDelta = context.DistanceDelta;
                point.Number = context.Number;
            }
            Distance = context.Distance;
        }

        public IEnumerable<FindPositionResult> FindPosition(IPoint position, FindPositionArgs args)
        {
            FindPositionHelper helper = new FindPositionHelper(args, Enumerable.Repeat(Points.Cast<IPoint>(), 1));
            return helper.GetResult(position.GetPos());
        }

        private bool isSelected;

        public bool IsSelected
        {
            get { return isSelected; }
            set { isSelected = value; OnPropertyChanged("IsSelected"); }
        }

        private bool isVisible = true;

        public bool IsVisible
        {
            get { return isVisible; }
            set { isVisible = value; OnPropertyChanged("IsVisible"); }
        }

        private Color color;

        public Color Color
        {
            get
            {
                if (Source.Color.HasValue == false) return color;
                return Source.Color.Value;
            }
            set { Source.Color = value; OnPropertyChanged("Color"); }
        }

        public void GetBounds(BoundsBuilder builder)
        {
            foreach (var p in Points) builder.AddPoint(p.GetPos());
        }

        public FindTimeResult FindTime(System.DateTime dateTime, bool searchInGaps)
        {
            throw new Exception("Metoda FindTime nen podporovna na objektu typu RouteVM");
        }
    }
}