namespace Brjnk.GpxExplorer.ViewModel
{
    using System;

    /// <summary>
    /// </summary>
    public class TrackPointAttributeInfo
    {
        private TrackPointAttributeInfo(TrackPointAttribute attribute, TrackPointAttributeType type, bool onlyGrowing)
        {
            Type = type;
            Attribute = attribute;
            OnlyGrowing = onlyGrowing;
        }

        public TrackPointAttribute Attribute { get; private set; }

        public TrackPointAttributeType Type { get; private set; }

        public bool OnlyGrowing { get; private set; }

        public static readonly TrackPointAttributeInfo Distance =
            new TrackPointAttributeInfo(TrackPointAttribute.Distance, TrackPointAttributeType.Type1, true);

        public static readonly TrackPointAttributeInfo DistanceDelta =
    new TrackPointAttributeInfo(TrackPointAttribute.DistanceDelta, TrackPointAttributeType.Type1, false);

        public static readonly TrackPointAttributeInfo Duration =
    new TrackPointAttributeInfo(TrackPointAttribute.Duration, TrackPointAttributeType.Type1, true);

        public static readonly TrackPointAttributeInfo DurationDelta =
    new TrackPointAttributeInfo(TrackPointAttribute.DurationDelta, TrackPointAttributeType.Type1, false);

        public static readonly TrackPointAttributeInfo DateTime =
    new TrackPointAttributeInfo(TrackPointAttribute.DateTime, TrackPointAttributeType.Type1, true);

        public static readonly TrackPointAttributeInfo Elevation =
    new TrackPointAttributeInfo(TrackPointAttribute.Elevation, TrackPointAttributeType.Type1, false);

        public static readonly TrackPointAttributeInfo Rised =
    new TrackPointAttributeInfo(TrackPointAttribute.Rised, TrackPointAttributeType.Type1, true);

        public static readonly TrackPointAttributeInfo Fallen =
    new TrackPointAttributeInfo(TrackPointAttribute.Fallen, TrackPointAttributeType.Type1, false);

        public static readonly TrackPointAttributeInfo Speed =
    new TrackPointAttributeInfo(TrackPointAttribute.Speed, TrackPointAttributeType.Type2, false);

        public static readonly TrackPointAttributeInfo VerticalSpeed =
    new TrackPointAttributeInfo(TrackPointAttribute.VerticalSpeed, TrackPointAttributeType.Type2, false);

        public static readonly TrackPointAttributeInfo Acceleration =
    new TrackPointAttributeInfo(TrackPointAttribute.Acceleration, TrackPointAttributeType.Type3, false);

        public static readonly TrackPointAttributeInfo Slope =
    new TrackPointAttributeInfo(TrackPointAttribute.Slope, TrackPointAttributeType.Type2, false);

        public static readonly TrackPointAttributeInfo Azimuth =
    new TrackPointAttributeInfo(TrackPointAttribute.Azimuth, TrackPointAttributeType.Type2, false);

        public override string ToString()
        {
            return Attribute.ToString();
        }

        public Func<ITrackPoint, double> GetSelector(bool joined)
        {
            if (Attribute == TrackPointAttribute.Distance)
            {
                if (joined)
                    return new Func<ITrackPoint, double>(t => t.DistanceJoined.GetDoubleValue());
                else
                    return new Func<ITrackPoint, double>(t => t.Distance.GetDoubleValue());
            }
            if (Attribute == TrackPointAttribute.Duration)
            {
                if (joined)
                    return new Func<ITrackPoint, double>(t => t.DurationJoined.GetDoubleValue());
                else
                    return new Func<ITrackPoint, double>(t => t.Duration.GetDoubleValue());
            }
            if (Attribute == TrackPointAttribute.DateTime)
            {
                return new Func<ITrackPoint, double>(t => t.Time.GetDoubleValue());
            }
            throw new Exception();
        }
    }

    public enum TrackPointAttribute
    {
        Distance,
        DistanceDelta,
        Duration,
        DurationDelta,
        DateTime,
        Elevation,
        Rised,
        Fallen,
        Speed,
        VerticalSpeed,
        Acceleration,
        Slope,
        Azimuth
    }

    public enum TrackPointAttributeType
    {
        /// <summary>
        /// Kad bod m hodnotu atributu, kter pslu pouze jemu (nap. elevace, datum a as)
        /// </summary>
        Type1,

        /// <summary>
        /// Hodnota pslu pedchozmu segmentu (sece) stopy, prvn bod tedy hodnotu nem (nap rychlost)
        /// </summary>
        Type2,

        /// <summary>
        /// Hodnota je vypotna z pedchozho a nsledujcho segmentu. Prvn a posledn bod nem hodnotu (nap. zrychlen)
        /// </summary>
        Type3
    }
}