using System;
using Brjnk.Common;
using Brjnk.GpxExplorer.Data;
using Brjnk.GpxExplorer.Workspace;
using Brjnk.MVVM;

namespace Brjnk.GpxExplorer.ViewModel
{
    public class TrackPointVM : Wrapper<TrackPoint>, ITrackPoint, IPoint, ISearchResults, IGpsData, ISelectable
    {
        public TrackPointVM(TrackPoint source) : base(source) { }

        public ObservableCollectionEx<SearchResult> SearchResults { get; set; }

        private int _Number;

        public int Number
        {
            get
            {
                return _Number;
            }
            set
            {
                _Number = value;
                OnPropertyChanged("Number");
            }
        }

        private int _NumberInSegment;

        public int NumberInSegment
        {
            get
            {
                return _NumberInSegment;
            }
            set
            {
                _NumberInSegment = value;
                OnPropertyChanged("NumberInSegment");
            }
        }

        private int _SegmentNumber;

        public int SegmentNumber
        {
            get
            {
                return _SegmentNumber;
            }
            set
            {
                _SegmentNumber = value;
                OnPropertyChanged("SegmentNumber");
            }
        }

        //Vzdlenost
        private TrackPointAttributeValue<Distance> _distance;

        public TrackPointAttributeValue<Distance> Distance
        {
            get
            {
                return _distance;
            }
            set
            {
                _distance = value;
                OnPropertyChanged("Distance");
            }
        }

        private TrackPointAttributeValue<Distance> _distanceJoined;

        public TrackPointAttributeValue<Distance> DistanceJoined
        {
            get
            {
                return _distanceJoined;
            }
            set
            {
                _distanceJoined = value;
                OnPropertyChanged("DistanceJoined");
            }
        }

        private TrackPointAttributeValue<Distance> _DistanceDeltaJoined;

        public TrackPointAttributeValue<Distance> DistanceDeltaJoined
        {
            get
            {
                return _DistanceDeltaJoined;
            }
            set
            {
                _DistanceDeltaJoined = value;
                OnPropertyChanged("DistanceDeltaJoined");
                OnPropertyChanged("DistanceDelta");
            }
        }

        public TrackPointAttributeValue<Distance> DistanceDelta
        {
            get
            {
                if (Flag == TrackPointFlag.FirstInSegment) return DistanceDeltaJoined;
                return TrackPointAttributeValue<Distance>.CreateUnset();
            }
        }

        //Trvn (as)

        private TrackPointAttributeValue<DateTime> time;

        public TrackPointAttributeValue<DateTime> Time
        {
            get { return time; }
            set { time = value; OnPropertyChanged("Time"); }
        }

        private TrackPointAttributeValue<TimeSpan> duration;

        public TrackPointAttributeValue<TimeSpan> Duration
        {
            get
            {
                return duration;
            }
            set
            {
                duration = value;
                OnPropertyChanged("Duration");
            }
        }

        private TrackPointAttributeValue<TimeSpan> durationJoined;

        public TrackPointAttributeValue<TimeSpan> DurationJoined
        {
            get
            {
                return durationJoined;
            }
            set
            {
                durationJoined = value;
                OnPropertyChanged("DurationJoined");
            }
        }

        private bool hasDurationDelta;

        public bool HasDurationDelta
        {
            private get { return hasDurationDelta; }
            set { hasDurationDelta = value; OnPropertyChanged("DurationDelta"); }
        }

        public TrackPointAttributeValue<TimeSpan> DurationDelta
        {
            get
            {
                if (hasDurationDelta || DurationDeltaJoined.IsUnavailable) return durationDeltaJoined;
                return TrackPointAttributeValue<TimeSpan>.CreateUnset();
            }
        }

        private TrackPointAttributeValue<TimeSpan> durationDeltaJoined;

        public TrackPointAttributeValue<TimeSpan> DurationDeltaJoined
        {
            get { return durationDeltaJoined; }
            set
            {
                durationDeltaJoined = value;
                OnPropertyChanged("DurationDelta");
                OnPropertyChanged("DurationDeltaJoined");
            }
        }

        //Nastoupno, naklesno, delta elevation
        private TrackPointAttributeValue<Distance> _Rised;

        public TrackPointAttributeValue<Distance> Rised
        {
            get
            {
                return _Rised;
            }
            set
            {
                _Rised = value;
                OnPropertyChanged("Rised");
            }
        }

        private TrackPointAttributeValue<Distance> _fallen;

        public TrackPointAttributeValue<Distance> Fallen
        {
            get
            {
                return _fallen;
            }
            set
            {
                _fallen = value;
                OnPropertyChanged("Fallen");
            }
        }

        private TrackPointAttributeValue<Distance> _RisedJoined;

        public TrackPointAttributeValue<Distance> RisedJoined
        {
            get
            {
                return _RisedJoined;
            }
            set
            {
                _RisedJoined = value;
                OnPropertyChanged("RisedJoined");
            }
        }

        private TrackPointAttributeValue<Distance> _fallenJoined;

        public TrackPointAttributeValue<Distance> FallenJoined
        {
            get
            {
                return _fallenJoined;
            }
            set
            {
                _fallenJoined = value;
                OnPropertyChanged("FallenJoined");
            }
        }

        private TrackPointAttributeValue<Distance> elevationDeltaJoined;

        public TrackPointAttributeValue<Distance> ElevationDeltaJoined
        {
            get { return elevationDeltaJoined; }
            set { elevationDeltaJoined = value; OnPropertyChanged("ElevationDelta"); OnPropertyChanged("ElevationDeltaJoined"); }
        }

        private bool hasElevationDelta;

        public bool HasElevationDelta
        {
            private get { return hasElevationDelta; }
            set { hasElevationDelta = value; OnPropertyChanged("ElevationDelta"); }
        }

        public TrackPointAttributeValue<Distance> ElevationDelta
        {
            get
            {
                if (!hasElevationDelta) return TrackPointAttributeValue<Distance>.CreateUnset();
                return elevationDeltaJoined;
            }
        }

        //Nadmosk vka je uloena i zde.
        //Na rozdl od zdrojov hodnoty je tato hodnota dopotna, pokud chyb.
        private TrackPointAttributeValue<Distance> elevation;

        public TrackPointAttributeValue<Distance> Elevation
        {
            get { return elevation; }
            set { elevation = value; OnPropertyChanged("Elevation"); }
        }

        //Rychlost, zrychlen
        private bool hasAcceleration;

        public bool HasAcceleration
        {
            private get { return hasAcceleration; }
            set { hasAcceleration = value; OnPropertyChanged("Acceleration"); }
        }

        public TrackPointAttributeValue<Acceleration> Acceleration
        {
            get
            {
                if (AccelerationJoined.IsUnavailable || hasAcceleration) return AccelerationJoined;
                return TrackPointAttributeValue<Acceleration>.CreateUnset();
            }
        }

        private TrackPointAttributeValue<Acceleration> accelerationJoined;

        public TrackPointAttributeValue<Acceleration> AccelerationJoined
        {
            get { return accelerationJoined; }
            set { accelerationJoined = value; OnPropertyChanged("AccelerationJoined"); OnPropertyChanged("Acceleration"); }
        }

        private bool hasSpeed;

        public bool HasSpeed
        {
            private get { return hasSpeed; }
            set { hasSpeed = value; OnPropertyChanged("Speed"); }
        }

        public TrackPointAttributeValue<Speed> Speed
        {
            get
            {
                if (SpeedJoined.IsUnavailable || hasSpeed) return SpeedJoined;
                return TrackPointAttributeValue<Speed>.CreateUnset();
            }
        }

        private TrackPointAttributeValue<Speed> speedJoined;

        public TrackPointAttributeValue<Speed> SpeedJoined
        {
            get { return speedJoined; }
            set { speedJoined = value; OnPropertyChanged("SpeedJoined"); OnPropertyChanged("Speed"); }
        }

        private bool hasVerticalSpeed;

        public bool HasVerticalSpeed
        {
            private get { return hasVerticalSpeed; }
            set { hasVerticalSpeed = value; OnPropertyChanged("VerticalSpeed"); }
        }

        public TrackPointAttributeValue<Speed> VerticalSpeed
        {
            get
            {
                if (VerticalSpeedJoined.IsUnavailable || hasVerticalSpeed) return VerticalSpeedJoined;
                return TrackPointAttributeValue<Speed>.CreateUnset();
            }
        }

        //Vertikln rychlost, stoupn
        private TrackPointAttributeValue<Speed> verticalSpeedJoined;

        public TrackPointAttributeValue<Speed> VerticalSpeedJoined
        {
            get { return verticalSpeedJoined; }
            set { verticalSpeedJoined = value; OnPropertyChanged("VerticalSpeedJoined"); OnPropertyChanged("VerticalSpeed"); }
        }

        private TrackPointAttributeValue<Slope> slopeJoined;

        public TrackPointAttributeValue<Slope> SlopeJoined
        {
            get { return slopeJoined; }
            set { slopeJoined = value; OnPropertyChanged("SlopeJoined"); OnPropertyChanged("Slope"); }
        }

        private bool hasSlope;

        public bool HasSlope
        {
            private get { return hasSlope; }
            set { hasSlope = value; OnPropertyChanged("Slope"); }
        }

        public TrackPointAttributeValue<Slope> Slope
        {
            get
            {
                if (SlopeJoined.IsUnavailable || hasSlope) return SlopeJoined;
                return TrackPointAttributeValue<Slope>.CreateUnset();
            }
        }

        //Azimut
        public TrackPointAttributeValue<Azimuth> Azimuth
        {
            get
            {
                if (flag == TrackPointFlag.FirstInSegment) return TrackPointAttributeValue<Azimuth>.CreateUnset();
                return AzimuthJoined;
            }
        }

        private TrackPointAttributeValue<Azimuth> azimuthJoined;

        public TrackPointAttributeValue<Azimuth> AzimuthJoined
        {
            get { return azimuthJoined; }
            set { azimuthJoined = value; OnPropertyChanged("AzimuthJoined"); OnPropertyChanged("Azimuth"); }
        }

        private TrackPointFlag flag;

        public TrackPointFlag Flag
        {
            get { return flag; }
            set { flag = value; OnPropertyChanged("Flag"); }
        }

        private ElevationExtreme elevationLocalExtreme;

        public ElevationExtreme ElevationLocalExtreme
        {
            get { return elevationLocalExtreme; }
            set { elevationLocalExtreme = value; OnPropertyChanged("ElevationLocalExtreme"); }
        }

        public GpsCoordinates GetPos()
        {
            return Source.Position;
        }

        public void GetBounds(BoundsBuilder builder)
        {
            builder.AddPoint(GetPos());
        }

        private bool isSelected;

        public bool IsSelected
        {
            get { return isSelected; }
            set { isSelected = value; OnPropertyChanged("IsSelected"); }
        }

        public TrackPointAttributeValue<double> GetAttributeValue(TrackPointAttributeInfo attribute, bool includeGaps)
        {
            if (includeGaps)
            {
                switch (attribute.Attribute)
                {
                    case TrackPointAttribute.Distance:
                        return this.DistanceJoined.ToDouble();
                    case TrackPointAttribute.DistanceDelta:
                        return this.DistanceDeltaJoined.ToDouble();
                    case TrackPointAttribute.Duration:
                        return this.DurationJoined.ToDouble();
                    case TrackPointAttribute.DurationDelta:
                        return this.DurationDeltaJoined.ToDouble();
                    case TrackPointAttribute.DateTime:
                        if (this.Source.Time.HasValue == false) return TrackPointAttributeValue<double>.CreateUnavailable();
                        return new TrackPointAttributeValue<double>(this.Source.Time.Value.Ticks / 10000000);
                    case TrackPointAttribute.Elevation:
                        return this.Elevation.ToDouble();
                    case TrackPointAttribute.Rised:
                        return this.RisedJoined.ToDouble();
                    case TrackPointAttribute.Fallen:
                        return this.FallenJoined.ToDouble();
                    case TrackPointAttribute.Speed:
                        return this.SpeedJoined.ToDouble();
                    case TrackPointAttribute.VerticalSpeed:
                        return this.VerticalSpeedJoined.ToDouble();
                    case TrackPointAttribute.Acceleration:
                        return this.AccelerationJoined.ToDouble();
                    case TrackPointAttribute.Slope:
                        return this.SlopeJoined.ToDouble();
                    case TrackPointAttribute.Azimuth:
                        return this.AzimuthJoined.ToDouble();
                    default:
                        throw new Exception("Unknown TrackPointAttribute");
                }
            }
            else
            {
                switch (attribute.Attribute)
                {
                    case TrackPointAttribute.Distance:
                        return this.Distance.ToDouble();
                    case TrackPointAttribute.DistanceDelta:
                        return this.DistanceDelta.ToDouble();
                    case TrackPointAttribute.Duration:
                        return this.Duration.ToDouble();
                    case TrackPointAttribute.DurationDelta:
                        return this.DurationDelta.ToDouble();
                    case TrackPointAttribute.DateTime:
                        if (this.Source.Time.HasValue == false) return TrackPointAttributeValue<double>.CreateUnavailable();
                        return new TrackPointAttributeValue<double>(this.Source.Time.Value.Ticks / 10000000);
                    case TrackPointAttribute.Elevation:
                        return this.Elevation.ToDouble();
                    case TrackPointAttribute.Rised:
                        return this.Rised.ToDouble();
                    case TrackPointAttribute.Fallen:
                        return this.Fallen.ToDouble();
                    case TrackPointAttribute.Speed:
                        return this.Speed.ToDouble();
                    case TrackPointAttribute.VerticalSpeed:
                        return this.VerticalSpeed.ToDouble();
                    case TrackPointAttribute.Acceleration:
                        return this.Acceleration.ToDouble();
                    case TrackPointAttribute.Slope:
                        return this.Slope.ToDouble();
                    case TrackPointAttribute.Azimuth:
                        return this.Azimuth.ToDouble();
                    default:
                        throw new Exception("Unknown TrackPointAttribute");
                }
            }
        }
    }

    public enum TrackPointFlag : byte
    {
        None,
        FirstInTrack,
        FirstInSegment,
        LastInTrack,
        LastInSegment
    }

    public enum ElevationExtreme : byte
    {
        None,
        Maximum,
        Minimum
    }
}