namespace Brjnk.GpxExplorer.ViewModel
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Windows.Data;
    using System.Windows.Media;
    using Brjnk.Common;
    using Brjnk.GpxExplorer.Computing;
    using Brjnk.GpxExplorer.Data;
    using Brjnk.MVVM;

    public class TrackVM : Wrapper<Track>, ISearchablePath, IGpsData, ISelectable, IHideable
    {
        static TrackVM()
        {
            WrapProperty(new PropertyWrapperInfo<Track>("Segments",
                     r => r.Segments, null, o => new CollectionWrapper<TrackSegment, TrackSegmentVM>((ObservableCollectionEx<TrackSegment>)o), false));
        }

        public TrackVM(Track source)
            : base(source)
        {
            Recalculate();
            RebuildFlatPoints();
            if (source.Color.HasValue == false) color = AutoColorSelector.GetNextColor();
            this.Changed += new EventHandler<ChangedEventArgs>(TrackVM_Changed);
        }

        private void TrackVM_Changed(object sender, ChangedEventArgs e)
        {
            if (e.Type == ChangeType.ChildAdded || e.Type == ChangeType.ChildRemoved || e.Type == ChangeType.Reset)
            {
                RebuildFlatPoints();
                Recalculate();
            }
            if (e.Type == ChangeType.PropertyChanged && e.PathEquals("Segments", "Points"))
            {
                Recalculate();
                RebuildFlatPoints();
            }
            else if (e.BeginsPathWith("Segments", "Points", "Source"))
                Recalculate();
        }

        private void RebuildFlatPoints()
        {
            flatPoints.Clear();
            foreach (var s in Segments)
            {
                flatPoints.Add(new CollectionContainer() { Collection = s.Points });
            }
        }

        public IEnumerable<TrackPointVM> GetAllPoints()
        {
            return Segments.SelectMany(s => s.Points);
        }

        public CollectionWrapper<TrackSegment, TrackSegmentVM> Segments
        {
            get { return (CollectionWrapper<TrackSegment, TrackSegmentVM>)GetWrapper("Segments"); }
        }

        private CompositeCollection flatPoints = new CompositeCollection();

        public CompositeCollection FlatPoints
        {
            get { return flatPoints; }
        }

        public TrackPointAttributeValue<double> GetAttributeValue(TrackPointAttributeInfo attribute, bool includeGaps, TrackPointVM previous, TrackPointVM next, double parameter)
        {
            if (attribute.Type == TrackPointAttributeType.Type2)
            {
                return next.GetAttributeValue(attribute, includeGaps);
            }
            else
            {
                var previousVal = previous.GetAttributeValue(attribute, includeGaps);
                var nextVal = next.GetAttributeValue(attribute, includeGaps);
                if (previousVal.State != AttributeValueState.ValueAvailable || nextVal.State != AttributeValueState.ValueAvailable) return TrackPointAttributeValue<double>.CreateUnavailable();
                return previousVal.Value + (nextVal.Value - previousVal.Value) * parameter;
            }
        }

        //public double? GetAttributeMaxValue(TrackPointAttributeInfo attribute, bool includeGaps)
        //{
        //    return Segments.SelectMany(s => s.Points).Max(p => GetAttributeValue(attribute, p, includeGaps).Nullable);
        //}

        //public double? GetAttributeMinValue(TrackPointAttributeInfo attribute, bool includeGaps)
        //{
        //    return Segments.SelectMany(s => s.Points).Min(p => GetAttributeValue(attribute, p, includeGaps));
        //}

        private Distance distance;

        public Distance Distance
        {
            get { return distance; }
            private set { distance = value; OnPropertyChanged("Distance"); }
        }

        private Distance distanceJoined;

        public Distance DistanceJoined
        {
            get { return distanceJoined; }
            private set { distanceJoined = value; OnPropertyChanged("DistanceJoined"); }
        }

        private Distance? rised;

        public Distance? Rised
        {
            get { return rised; }
            private set { rised = value; OnPropertyChanged("Rised"); }
        }

        private Distance? fallen;

        public Distance? Fallen
        {
            get { return fallen; }
            private set { fallen = value; OnPropertyChanged("Fallen"); }
        }

        private Speed? avgSpeed;

        public Speed? AvgSpeed
        {
            get { return avgSpeed; }
            private set { avgSpeed = value; OnPropertyChanged("AvgSpeed"); }
        }

        private Speed? avgSpeedJoined;

        public Speed? AvgSpeedJoined
        {
            get { return avgSpeedJoined; }
            private set { avgSpeedJoined = value; OnPropertyChanged("AvgSpeedJoined"); }
        }

        private TimeSpan? duration;

        public TimeSpan? Duration
        {
            get { return duration; }
            private set { duration = value; OnPropertyChanged("Duration"); }
        }

        private TimeSpan? durationJoined;

        public TimeSpan? DurationJoined
        {
            get { return durationJoined; }
            private set { durationJoined = value; OnPropertyChanged("DurationJoined"); }
        }

        private Distance? risedJined;

        public Distance? RisedJoined
        {
            get { return risedJined; }
            set { risedJined = value; OnPropertyChanged("RisedJoined"); }
        }

        private Distance? fallenJoined;

        public Distance? FallenJoined
        {
            get { return fallenJoined; }
            set { fallenJoined = value; OnPropertyChanged("FallenJoined"); }
        }

        public IEnumerable<TrackPointVM> GetPoints(int startSegmentIndex, int startPointIndex, int endSegmentIndex, int endPointIndex, bool includeFirst, bool includeLast)
        {
            if (startSegmentIndex > endSegmentIndex) throw new ArgumentException("StartSegment mus bt men ne EndSegment");
            if (startSegmentIndex == endSegmentIndex && startPointIndex > endPointIndex) throw new ArgumentException("StartPoint mus bt men ne EndPoint");
            if (startSegmentIndex == endSegmentIndex && startPointIndex == endPointIndex)
            {
                if (includeFirst && includeLast) yield return Segments[startPointIndex].Points[startPointIndex];
                yield break;
            }

            for (int s = startSegmentIndex; s <= endSegmentIndex; s++)
            {
                var segment = Segments[s];
                for (int i = startPointIndex; i < segment.Points.Count; i++)
                {
                    if (s == startSegmentIndex && i == startPointIndex && includeFirst == false) continue;
                    bool isLast = s == endSegmentIndex && i == endPointIndex;
                    if (isLast && includeLast == false) break;
                    yield return segment.Points[i];
                    if (isLast) break;
                }
            }
        }

        public VirtualTrackPoint GetVirtualPoint(int segmentNumber, int pointNumber, double parameter)
        {
            var segment = Segments[segmentNumber];
            TrackPointVM point1 = segment.Points[pointNumber];
            TrackPointVM point2;
            bool isInGap = false;
            //pokud je posledn v segmentu
            if (segment.Points.Count - 1 == pointNumber)
            {
                isInGap = true;
                point2 = Segments[segmentNumber + 1].Points[0];
            }
            else { point2 = segment.Points[pointNumber + 1]; }

            var retVal = new VirtualTrackPoint();
            retVal.IsInGap = isInGap;

            if (point1.Distance.HasValue && point2.Distance.HasValue)
                retVal.Distance = point1.Distance.Value + (point2.Distance.Value - point1.Distance.Value) * parameter;
            else retVal.Distance = TrackPointAttributeValue<Distance>.CreateUnavailable();

            if (point1.DistanceJoined.HasValue && point2.DistanceJoined.HasValue)
                retVal.DistanceJoined = point1.DistanceJoined.Value + (point2.DistanceJoined.Value - point1.DistanceJoined.Value) * parameter;
            else retVal.DistanceJoined = TrackPointAttributeValue<Distance>.CreateUnavailable();

            if (point1.Duration.HasValue && point2.Duration.HasValue)
                retVal.Duration = TimeSpan.FromSeconds(point1.Duration.Value.TotalSeconds + (point2.Duration.Value - point1.Duration.Value).TotalSeconds * parameter);
            else retVal.Duration = TrackPointAttributeValue<TimeSpan>.CreateUnavailable();

            if (point1.Duration.HasValue && point2.Duration.HasValue)
                retVal.DurationJoined = TimeSpan.FromSeconds(point1.DurationJoined.Value.TotalSeconds + (point2.DurationJoined.Value - point1.DurationJoined.Value).TotalSeconds * parameter);
            else retVal.DistanceJoined = TrackPointAttributeValue<Distance>.CreateUnavailable();

            if (point1.Elevation.HasValue && point2.Elevation.HasValue)
                retVal.Elevation = point1.Elevation.Value + (point2.Elevation.Value - point1.Elevation.Value) * parameter;
            else retVal.Elevation = TrackPointAttributeValue<Distance>.CreateUnavailable();

            if (point1.Rised.HasValue && point2.Rised.HasValue)
                retVal.Rised = point1.Rised.Value + (point2.Rised.Value - point1.Rised.Value) * parameter;
            else retVal.Rised = TrackPointAttributeValue<Distance>.CreateUnavailable();

            if (point1.RisedJoined.HasValue && point2.RisedJoined.HasValue)
                retVal.RisedJoined = point1.RisedJoined.Value + (point2.RisedJoined.Value - point1.RisedJoined.Value) * parameter;
            else retVal.RisedJoined = TrackPointAttributeValue<Distance>.CreateUnavailable();

            if (point1.Fallen.HasValue && point2.Fallen.HasValue)
                retVal.Fallen = point1.Fallen.Value + (point2.Fallen.Value - point1.Fallen.Value) * parameter;
            else retVal.Fallen = TrackPointAttributeValue<Distance>.CreateUnavailable();

            if (point1.FallenJoined.HasValue && point2.FallenJoined.HasValue)
                retVal.FallenJoined = point1.FallenJoined.Value + (point2.FallenJoined.Value - point1.FallenJoined.Value) * parameter;
            else retVal.FallenJoined = TrackPointAttributeValue<Distance>.CreateUnavailable();

            return retVal;
        }

        public IList<ITrackPoint> GetPointsInInterval(double from, double to, TrackPointAttributeInfo attribute, bool joined)
        {
            var pointFirst = FindTrackPoint(from, attribute, joined);
            var pointLast = FindTrackPoint(to, attribute, joined);

            if (pointFirst == null) return null;
            if (pointLast == null) return null;

            var middlePoints = GetPoints(pointFirst.PreviousPoint.SegmentNumber, pointFirst.PreviousPoint.NumberInSegment,
                                        pointLast.PreviousPoint.SegmentNumber, pointLast.PreviousPoint.NumberInSegment, false, false);
            var retVal = new List<ITrackPoint>();
            retVal.Add(GetVirtualPoint(pointFirst.PreviousPoint.SegmentNumber, pointFirst.PreviousPoint.NumberInSegment, pointFirst.Parameter));
            retVal.AddRange(middlePoints);
            retVal.Add(GetVirtualPoint(pointLast.PreviousPoint.SegmentNumber, pointLast.PreviousPoint.NumberInSegment, pointLast.Parameter));
            return retVal;
        }

        public IList<VirtualTrackPoint> GetAvgPoints(double from, TrackPointAttributeInfo attribute, bool joined, double step, double count)
        {
            var retVal = new List<VirtualTrackPoint>();
            FindTrackPointResult previous = null;

            for (int i = 0; i < count; i++)
            {
                var points = GetPointsInInterval(from + step * i, from + step * (i + 1), attribute, joined);
                if (points == null) continue;
                else { previous = FindTrackPoint(from + step * i, attribute, joined); }
                var point = VirtualTrackPoint.GetAvgPoint(points, attribute.GetSelector(joined));

                retVal.Add(point);
            }

            ITrackPoint previousTrkPt;
            if (previous != null) previousTrkPt = previous.PreviousPoint;
            else previousTrkPt = VirtualTrackPoint.CreateFirst();

            for (int i = 0; i < retVal.Count; i++)
            {
                var current = retVal[i];
                if (previousTrkPt.DurationJoined.HasValue)
                    DurationDeltaCalculator.Calculate(previousTrkPt, true, current);
                DistanceDeltaCalculator.Calculate(previousTrkPt, true, current);
                if (previousTrkPt.DurationJoined.HasValue)
                    SpeedCalculator.Calculate(previousTrkPt, true, current);
                if (previousTrkPt.DurationJoined.HasValue && previousTrkPt.Elevation.HasValue)
                    VerticalSpeedCalculator.Calculate(previousTrkPt, true, current);
                if (previousTrkPt.Elevation.HasValue)
                    SlopeCalculator.Calculate(previousTrkPt, true, current);
                if (previousTrkPt.DurationJoined.HasValue)
                    AccelerationCalculator.Calculate(previousTrkPt, true, current);
                previousTrkPt = current;
            }

            return retVal;
        }

        #region Search

        public FindTrackPointResult FindTrackPoint(double value, TrackPointAttributeInfo attribute, bool joined)
        {
            if (attribute.OnlyGrowing == false) throw new InvalidOperationException("Vyhledvn me probhat pouze podle parametru, kter pod roste.");

            var allPoints = this.GetAllPoints().ToList();

            var firstPoint = allPoints.First().GetAttributeValue(attribute, joined);
            var lastPoint = allPoints.Last().GetAttributeValue(attribute, joined);
            if (firstPoint.HasValue == false || lastPoint.HasValue == false)
            {
                if (firstPoint.GetDoubleValue() > value || lastPoint.GetDoubleValue() < value) return null;
                return DoSlowSearch(allPoints, value, attribute, joined);
            }
            else
            {
                if (firstPoint.GetDoubleValue() > value || lastPoint.GetDoubleValue() < value) return null;
                //todo: nahradit FastSearch
                return DoSlowSearch(allPoints, value, attribute, joined);
            }
        }

        //private FindTrackPointResult DoFastSearch(List<TrackPointVM> allPoints, int from, int to, double value, TrackPointAttributeInfo attribute, bool joined)
        //{
        //    if (from == to)
        //    {
        //        var previousPoint = from > 0 ? allPoints[from - 1] : null;
        //        var point = allPoints[from];
        //        var nextPoint = from < allPoints.Count - 1 ? allPoints[from + 1] : null;

        //        if (from == 0)
        //        {
        //            double pointValue = point.GetAttributeValue(attribute, joined).GetDoubleValue();
        //            double nextPointValue = point.GetAttributeValue(attribute, joined).GetDoubleValue();
        //            return new FindTrackPointResult(point, )
        //        }
        //        if (from == allPoints.Count - 1)
        //        {
        //        }
        //        else
        //        {
        //        }

        //    }
        //    else
        //    {
        //    }
        //}

        private FindTrackPointResult DoSlowSearch(List<TrackPointVM> allPoints, double value, TrackPointAttributeInfo attribute, bool joined)
        {
            if (allPoints.Count < 2) return null;

            TrackPointVM last = allPoints[0];
            double lastValue = last.GetAttributeValue(attribute, joined).GetDoubleValue();

            for (int i = 0; i < allPoints.Count; i++)
            {
                var current = allPoints[i];
                var currentVal = current.GetAttributeValue(attribute, joined).GetDoubleValue();
                if (currentVal > value)
                {
                    if (lastValue <= value)
                    {
                        bool isInGap = current.SegmentNumber != last.SegmentNumber;
                        return new FindTrackPointResult(last, (value - lastValue) / (currentVal - lastValue), isInGap);
                    }
                    else return null;
                }

                last = current;
                lastValue = last.GetAttributeValue(attribute, joined).GetDoubleValue();
            }
            return null;
        }

        public FindTimeResult FindTime(DateTime dateTime, bool searchInInterruptSegments)
        {
            List<DateTime> arg = new List<DateTime>() { dateTime };
            var result = FindTime(arg, searchInInterruptSegments);
            if (result == null) return null;
            return result.FirstOrDefault().Value;
        }

        public IDictionary<DateTime, FindTimeResult> FindTime(List<DateTime> dateTimes, bool searchInInterruptSegments)
        {
            dateTimes.Sort();
            var retVal = new Dictionary<DateTime, FindTimeResult>();
            TrackPointVM previous = null;
            int dateTimesPos = 0;
            var points = Segments.SelectMany(c => c.Points);
            if (!points.Any()) return retVal; //przdn stopa

            while (dateTimesPos < dateTimes.Count) //najt prvn as, kter me bt soust stopy
            {
                if (dateTimes[dateTimesPos] >= (points.First().Source.Time ?? DateTime.MinValue)) break;
                dateTimesPos++;
            }

            if (dateTimesPos == dateTimes.Count) return retVal; // dn vsledek, vechny vyhledvan datumy jsou ped zatkem stopy

            foreach (var segment in Segments)
            {
                foreach (var point in segment.Points)
                {
                    if (!point.Source.Time.HasValue)
                    {
                        System.Diagnostics.Debug.WriteLine("Nelze vyhledvat as ve stop, pokud vechny body stopy neobsahuj asovou informaci.", this.Source.Name);
                        return null;
                    }
                    if (previous != null)
                    {
                        while (dateTimesPos < dateTimes.Count)
                        {
                            var currentTime = dateTimes[dateTimesPos];
                            if (currentTime >= previous.Source.Time.Value && currentTime <= point.Source.Time.Value)
                            {
                                double p = (currentTime - previous.Source.Time.Value).TotalSeconds / (point.Source.Time.Value - previous.Source.Time.Value).TotalSeconds;
                                retVal.Add(currentTime, new FindTimeResult(previous, point, p));
                                dateTimesPos++;
                            }
                            else break;
                        }
                    }

                    previous = point;
                }
                if (!searchInInterruptSegments) previous = null;
            }
            return retVal;
        }

        public IEnumerable<FindPositionResult> FindPosition(IPoint position, FindPositionArgs args)
        {
            FindPositionHelper helper = new FindPositionHelper(args, this.Segments.Select(seg => seg.Points.Cast<IPoint>()));
            return helper.GetResult(position.GetPos());
        }

        #endregion Search

        protected void Recalculate()
        {
            var helper = new TrackCalculationHelper(this);
            var stopWatch = System.Diagnostics.Stopwatch.StartNew();
            helper.Calculate();
            stopWatch.Stop();
            System.Diagnostics.Debug.WriteLine(stopWatch.ElapsedMilliseconds.ToString(), "TrackCalculationTime " + this.Source.Name);

            this.AvgSpeed = helper.AvgSpeed;
            this.AvgSpeedJoined = helper.AvgSpeedJoined;
            this.Distance = helper.Distance;
            this.DistanceJoined = helper.DistanceJoined;
            this.Fallen = helper.Fallen;
            this.FallenJoined = helper.FallenJoined;
            this.Rised = helper.Rised;
            this.RisedJoined = helper.RisedJoined;
            this.Duration = helper.Duration;
            this.DurationJoined = helper.DurationJoined;
        }

        private bool isSelected;

        public bool IsSelected
        {
            get { return isSelected; }
            set { isSelected = value; OnPropertyChanged("IsSelected"); }
        }

        private bool isVisible = true;

        public bool IsVisible
        {
            get { return isVisible; }
            set { isVisible = value; OnPropertyChanged("IsVisible"); }
        }

        private Color color;

        public Color Color
        {
            get
            {
                if (Source.Color.HasValue == false) return color;
                return Source.Color.Value;
            }
            set { Source.Color = value; OnPropertyChanged("Color"); }
        }

        public void GetBounds(BoundsBuilder builder)
        {
            foreach (var segment in Segments) segment.GetBounds(builder);
        }
    }
}