namespace Brjnk.GpxExplorer.Workspace
{
    using System.Windows.Data;
    using Brjnk.Common.Extensions;
    using Brjnk.CoordinateSystem2D;
    using Brjnk.CoordinateSystem2D.Layers;
    using Brjnk.GpxExplorer.WPF;

    /// <summary>
    /// </summary>
    public class AreaManager
    {
        public PointsLayer PointsLayer { get; private set; }

        public PointsLayer RoutePointsLayer { get; private set; }

        public PointsLayer TrackPointsLayer { get; private set; }

        public PointsLayer SearchResultsLayer { get; private set; }

        public GraphLayer RoutesLayer { get; private set; }

        public GraphLayer TracksLayer { get; private set; }

        public PhotosLayer PhotosLayer { get; private set; }

        public MapLayer MapLayer { get; private set; }

        public Grid GridLayer { get; private set; }

        private Area2D area;

        public Area2D Area
        {
            get { return area; }
            set { area = value; }
        }

        public AreaManager(Area2D area, PointsLayer pointsLayer,
            GraphLayer routesLayer, GraphLayer tracksLayer,
            PhotosLayer photosLayer, MapLayer mapLayer,
            Grid gridLayer, PointsLayer routePointsLayer,
            PointsLayer trackPointsLayer, PointsLayer searchResultsLayer)
        {
            Area = area;
            area.MenuOpening += new System.EventHandler<OpenPopupEventArgs>(area_MenuOpening);

            PointsLayer = pointsLayer;
            RoutePointsLayer = routePointsLayer;
            TrackPointsLayer = trackPointsLayer;
            SearchResultsLayer = searchResultsLayer;
            RoutesLayer = routesLayer;
            TracksLayer = tracksLayer;
            PhotosLayer = photosLayer;
            MapLayer = mapLayer;
            GridLayer = gridLayer;

            Area.Layers.Add(GridLayer);
            Area.Layers.Add(MapLayer);
            Area.Layers.Add(RoutesLayer);
            Area.Layers.Add(RoutePointsLayer);
            Area.Layers.Add(TracksLayer);
            Area.Layers.Add(TrackPointsLayer);
            Area.Layers.Add(PointsLayer);
            Area.Layers.Add(SearchResultsLayer);
            Area.Layers.Add(PhotosLayer);

            //Nastav binding vlastnosti NetworkMode, aby se hodnota aktualizovala pi zmn nastaven
            BindingOperations.SetBinding(MapLayer, MapLayer.NetworkModeProperty, new Binding("NetworkMode") { Source = App.Instance.Settings });
        }

        private DataEditor editor;

        public void SetEditor(DataEditor editor)
        {
            this.editor = editor;
        }

        private void area_MenuOpening(object sender, OpenPopupEventArgs e)
        {
            if (area.Layers.Contains(MapLayer))
            {
                var coord = MapLayer.GetGpsCoordinates(e.Position);
                string[] coords = { coord.ToString(Common.CoordinationFormat.Degrees), coord.ToString(Common.CoordinationFormat.Minutes), coord.ToString(Common.CoordinationFormat.Seconds) };
                for (int i = 0; i < coords.Length; i++)
                {
                    var str = coords[i];
                    var mi = new System.Windows.Controls.MenuItem();
                    mi.FontSize = 8;
                    mi.Header = str;
                    mi.CommandParameter = str;
                    mi.Command = new StringToClipboardCommand();
                    e.AddCustomContent(mi);
                }

                var newWaypointMi = new System.Windows.Controls.MenuItem();
                newWaypointMi.Header = Localization.LocalizationManager.Instance.GetString("createWaypoint");
                newWaypointMi.CommandParameter = coord;
                newWaypointMi.Command = this.editor.CreateWaypointCommand;
                e.AddCustomContent(newWaypointMi);
            }
        }

        public bool ShowAxisX
        {
            get { return Area.ShowAxisX; }
            set { Area.ShowAxisX = value; }
        }

        public bool ShowAxisY
        {
            get { return Area.ShowAxisY; }
            set { Area.ShowAxisY = value; }
        }

        public AxisDefinition AxisX
        {
            get { return Area.AxisX; }
            set { Area.AxisX = value; }
        }

        public AxisDefinition AxisY
        {
            get { return Area.AxisY; }
            set { Area.AxisY = value; }
        }

        public void SetAxisesVisibility(bool visibility) { ShowAxisX = visibility; ShowAxisY = visibility; }

        public void Hide(ILayer layer)
        {
            Area.Layers.RemoveWhere(o => object.ReferenceEquals(o, layer));
        }

        public void Show(ILayer layer)
        {
            if (!Area.Layers.Contains(MapLayer)) Area.Layers.Insert(0, layer);
        }
    }
}