namespace Brjnk.GpxExplorer.Workspace
{
    using System;
    using System.Runtime.InteropServices;
    using System.Windows;
    using System.Windows.Interop;

    //Zavol API SetClipboardViewer s ukazatelem na okno, do kterho pijde zprva (WM_DRAWCLIPBOARD) o zmn schrnky
    //SetClipboardViewer vrac ukazatel na okno, ktermu m bt zprva dle pedna (dal okno, kter sleduje schrnku)
    //WM_CHANGECBCHAIN informuje o zmn nsledujcho okna

    /// <summary>
    /// Sleduje obsach schrnky, pi zmn obsahu vyvol udlost
    /// <remarks>Metoda Unregister mus bt zavolna ped ukonenm aplikace!</remarks>
    /// </summary>
    public class ClipboardWatcher
    {
        private const int WM_DRAWCLIPBOARD = 0x0308;
        private const int WM_CHANGECBCHAIN = 0x030D;
        private const int WM_DESTROY = 0x0002;

        public ClipboardWatcher(Window window)
        {
            handle = new WindowInteropHelper(window).Handle;
            Register();
        }

        private IntPtr handle;
        private IntPtr next;
        private bool registered = false;

        public void Register()
        {
            Unregister(); //odregistrovat, pokud ji SetClipboardViewer bylo volno
            var hwndSource = HwndSource.FromHwnd(handle);
            hwndSource.AddHook(Process);
            next = SetClipboardViewer(handle);
            registered = true;
        }

        public void Unregister()
        {
            if (registered)
                ChangeClipboardChain(handle, next);
            registered = false;
        }

        private IntPtr Process(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            if (msg == WM_DRAWCLIPBOARD) { handled = true; SendMessage(next, msg, wParam, lParam); OnClipboardChanged(); }
            if (msg == WM_CHANGECBCHAIN)
            {
                if (wParam == next) { next = lParam; }
                else { SendMessage(next, msg, wParam, lParam); }
                handled = true;
            }
            if (msg == WM_DESTROY) Unregister();
            return IntPtr.Zero;
        }

        #region Events

        public delegate void ClipboardChangedEventHandler();

        public event ClipboardChangedEventHandler ClipboardChanged;

        private void OnClipboardChanged()
        {
            if (ClipboardChanged != null) ClipboardChanged();
        }

        #endregion Events

        #region DllImport

        [System.Runtime.InteropServices.DllImportAttribute("User32.dll")]
        public static extern IntPtr SetClipboardViewer(IntPtr hWndNewViewer);

        [DllImport("User32.dll", CharSet = CharSet.Auto)]
        public static extern bool ChangeClipboardChain(IntPtr hWndRemove, IntPtr hWndNewNext);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern int SendMessage(IntPtr hwnd, int wMsg, IntPtr wParam, IntPtr lParam);

        #endregion DllImport
    }
}