namespace Brjnk.GpxExplorer.Workspace
{
    using System;
    using System.Linq;
    using Brjnk.GpxExplorer.ViewModel;
    using Brjnk.MVVM;

    /// <summary>
    /// </summary>
    public class DataChangeWatcher
    {
        private DataModel data;

        public DataChangeWatcher(DataModel data)
        {
            this.data = data;
            this.data.Changed += new EventHandler<MVVM.ChangedEventArgs>(data_Changed);
        }

        private void data_Changed(object sender, MVVM.ChangedEventArgs e)
        {
            if (e.Type == ChangeType.ChildAdded || e.Type == MVVM.ChangeType.ChildRemoved)
                ChildAddedOrRemovedEvent(e);
            else if (e.Type == MVVM.ChangeType.PropertyChanged)
                PropertyChangedEvent(e);
            else if (e.Type == MVVM.ChangeType.Reset)
                ResetEvent(e);
            else throw new Exception("Unknow ChangeType");
        }

        private void ResetEvent(MVVM.ChangedEventArgs e)
        {
            if (e.PropertyNodes.Count() == 0)
            {
                //zmna vech dat
                OnWaypointsReseted();
                OnRoutesReseted();
                OnTracksReseted();
                OnPhotosReseted();
            }
            else if (e.PropertyNodes.First().Name == "Waypoints")
                OnWaypointsReseted();
            else if (e.PropertyNodes.First().Name == "Routes")
                OnRoutesReseted();
            else if (e.PropertyNodes.First().Name == "Tracks")
                OnTracksReseted();
            else if (e.PropertyNodes.First().Name == "Photos")
                OnPhotosReseted();
        }

        private void PropertyChangedEvent(MVVM.ChangedEventArgs e)
        {
            if (e.PropertyNodes.Count() <= 1) return;
            if (e.PropertyNodes.Last().Name == "IsSelected")
            {
                var selectable = e.Nodes.Last() as ISelectable;
                if (selectable != null) OnIsSelectedChanged(new WatcherEventArgs<ISelectable>(selectable));
            }
            else if (e.PropertyNodes.Last().Name == "IsVisible")
            {
                var hideable = e.Nodes.Last() as IHideable;
                if (hideable != null) OnIsVisibleChanged(new WatcherEventArgs<IHideable>(hideable));
            }
            else if (e.BeginsPathWith("Waypoints", "Source"))
            {
                OnWaypointDataChanged(new WatcherEventArgs<WaypointVM>((WaypointVM)e.Nodes.ElementAt(1)));
            }
            else if (e.BeginsPathWith(new string[] { "Routes", "Points", "Source" }))
            {
                OnRouteDataChanged(new WatcherEventArgs<RouteVM>((RouteVM)e.Nodes.ElementAt(1)));
            }
            else if (e.BeginsPathWith(new string[] { "Routes", "Color" }))
            {
                OnRouteDataChanged(new WatcherEventArgs<RouteVM>((RouteVM)e.Nodes.ElementAt(1)));
            }
            else if (e.BeginsPathWith(new string[] { "Tracks", "Segments", "Points", "Source" }))
            {
                OnTrackDataChanged(new WatcherEventArgs<TrackVM>((TrackVM)e.Nodes.ElementAt(1)));
            }
            else if (e.BeginsPathWith(new string[] { "Tracks", "Color" }))
            {
                OnTrackDataChanged(new WatcherEventArgs<TrackVM>((TrackVM)e.Nodes.ElementAt(1)));
            }
            else if (e.BeginsPathWith(new string[] { "Photos", "Collection", "CorrectedCreationTime" }))
            {
                OnPhotoTimeChanged(new WatcherEventArgs<Photo>((Photo)e.Nodes.ElementAt(2)));
            }
        }

        private void ChildAddedOrRemovedEvent(MVVM.ChangedEventArgs e)
        {
            var args = new WatcherEventArgs<object>(e.Child);
            if (e.Type == MVVM.ChangeType.ChildAdded)
                OnObjectAdded(args);
            else
                OnObjectRemoved(args);

            if (args.ObjectType == typeof(TrackPointVM))
            {
                var args2 = new WatcherEventArgs<TrackVM>((args.Object as TrackPointVM).Parent.Parent as TrackVM);
                OnTrackDataChanged(args2);
            }
            else if (args.ObjectType == typeof(TrackSegmentVM))
            {
                var args2 = new WatcherEventArgs<TrackVM>((args.Object as TrackSegmentVM).Parent as TrackVM);
                OnTrackDataChanged(args2);
            }
            else if (args.ObjectType == typeof(RoutePointVM))
            {
                var args2 = new WatcherEventArgs<RouteVM>((args.Object as RoutePointVM).Parent as RouteVM);
                OnRouteDataChanged(args2);
            }
        }

        #region Events

        protected void OnObjectAdded(WatcherEventArgs<object> e)
        {
            if (ObjectAdded != null) ObjectAdded(this, e);
        }

        protected void OnObjectRemoved(WatcherEventArgs<object> e)
        {
            if (ObjectRemoved != null) ObjectRemoved(this, e);
        }

        protected void OnIsSelectedChanged(WatcherEventArgs<ISelectable> e)
        {
            if (IsSelectedChanged != null) IsSelectedChanged(this, e);
        }

        protected void OnIsVisibleChanged(WatcherEventArgs<IHideable> e)
        {
            if (IsVisibleChanged != null) IsVisibleChanged(this, e);
        }

        protected void OnWaypointsReseted()
        {
            if (WaypointsReseted != null) WaypointsReseted(this, EventArgs.Empty);
        }

        protected void OnRoutesReseted()
        {
            if (RoutesReseted != null) RoutesReseted(this, EventArgs.Empty);
        }

        protected void OnTracksReseted()
        {
            if (TracksReseted != null) TracksReseted(this, EventArgs.Empty);
        }

        protected void OnPhotosReseted()
        {
            if (PhotosReseted != null) PhotosReseted(this, EventArgs.Empty);
        }

        protected void OnRouteDataChanged(WatcherEventArgs<RouteVM> e)
        {
            if (RouteDataChanged != null) RouteDataChanged(this, e);
        }

        protected void OnTrackDataChanged(WatcherEventArgs<TrackVM> e)
        {
            if (TrackDataChanged != null) TrackDataChanged(this, e);
        }

        protected void OnWaypointDataChanged(WatcherEventArgs<WaypointVM> e)
        {
            if (WaypointDataChanged != null) WaypointDataChanged(this, e);
        }

        protected void OnPhotoTimeChanged(WatcherEventArgs<Photo> e)
        {
            if (PhotoTimeChanged != null) PhotoTimeChanged(this, e);
        }

        public event EventHandler<WatcherEventArgs<object>> ObjectAdded;

        public event EventHandler<WatcherEventArgs<object>> ObjectRemoved;

        public event EventHandler<WatcherEventArgs<ISelectable>> IsSelectedChanged;

        public event EventHandler<WatcherEventArgs<IHideable>> IsVisibleChanged;

        public event EventHandler WaypointsReseted;

        public event EventHandler RoutesReseted;

        public event EventHandler TracksReseted;

        public event EventHandler PhotosReseted;

        public event EventHandler<WatcherEventArgs<TrackVM>> TrackDataChanged;

        public event EventHandler<WatcherEventArgs<RouteVM>> RouteDataChanged;

        public event EventHandler<WatcherEventArgs<WaypointVM>> WaypointDataChanged;

        public event EventHandler<WatcherEventArgs<Photo>> PhotoTimeChanged;

        #endregion Events
    }

    public class WatcherEventArgs<T> : EventArgs
    {
        public WatcherEventArgs(T obj)
        {
            Object = obj;
            if ((object)obj != null) ObjectType = obj.GetType();
        }

        public T Object { get; private set; }

        public Type ObjectType { get; private set; }
    }
}