namespace Brjnk.GpxExplorer.Workspace
{
    using System;
    using System.ComponentModel;
    using Brjnk.Common;
    using Brjnk.CoordinateSystem2D;
    using Brjnk.GpxExplorer.ViewModel;

    /// <summary>
    /// </summary>
    public class MapDataAdapter : DataAdapter
    {
        private MapManager map;

        private WeakEventListener listener;

        public MapDataAdapter(DataModel model, SearchService searchService, DataView view, MapManager map)
            : base(model, searchService, view)
        {
            this.map = map;
            listener = new WeakEventListener(MapViewChanged);

            PropertyChangedEventManager.AddListener(map, listener, "Angle");
            PropertyChangedEventManager.AddListener(map, listener, "Provider");

            base.Thickness = 3;
        }

        private bool MapViewChanged(Type managerType, object sender, EventArgs e)
        {
            base.Invalidate();
            return true;
        }

        protected override System.Windows.Point? GetSearchRequestPosition(SearchRequest request)
        {
            if (request is SearchPositionRequest)
            {
                var posRequest = request as SearchPositionRequest;
                return map.Project(posRequest.Position);
            }
            return null;
        }

        protected override System.Windows.Point? GetSearchResultPosition(SearchResult result)
        {
            if (result is SearchDateTimeResult)
            {
                var dateResult = result as SearchDateTimeResult;
                return map.Project(dateResult.OriginalResultTyped.Position);
            }
            return null;
        }

        protected override System.Windows.Point? GetPhotoPosition(LocatedPhoto photo)
        {
            return map.Project(photo.Position);
        }

        protected override System.Windows.Point? GetWaypointPosition(WaypointVM waypoint)
        {
            return map.Project(waypoint.Source.Position);
        }

        protected override System.Windows.Point? GetTrackPointPosition(TrackVM track, TrackPointVM previous, TrackPointVM current, TrackPointVM next)
        {
            return map.Project(current.Source.Position);
        }

        protected override System.Windows.Point? GetRoutePointPosition(RouteVM route, RoutePointVM previous, RoutePointVM current, RoutePointVM next)
        {
            return map.Project(current.Source.Position);
        }

        /// <summary>
        /// Vrt osu, podle kter maj bt spojnice mezi body konstantn. Vrt null pro spojen bod sekou
        /// </summary>
        protected override AxisOrientation? GetGraphConstantInfo(RouteVM route)
        {
            return null;
        }

        protected override AxisOrientation? GetGraphConstantInfo(TrackVM route)
        {
            return null;
        }
    }
}