namespace Brjnk.GpxExplorer.Workspace
{
    using System;
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using System.Configuration;
    using Brjnk.GpxExplorer.Localization;
    using Brjnk.Maps;

    /// <summary>
    /// </summary>
    public class Settings : ApplicationSettingsBase, INotifyPropertyChanged
    {
        [UserScopedSetting]
        [SettingsSerializeAs(SettingsSerializeAs.Xml)]
        public ObservableCollection<CustomWebMapSource> CustomMapSources
        {
            get
            {
                if (this["CustomMapSources"] == null) CustomMapSources = new ObservableCollection<CustomWebMapSource>();
                return (ObservableCollection<CustomWebMapSource>)this["CustomMapSources"];
            }
            set { this["CustomMapSources"] = value; }
        }

        [UserScopedSetting]
        [SettingsSerializeAs(SettingsSerializeAs.Xml)]
        public ObservableCollection<CustomExternalApplication> CustomExternalApplications
        {
            get
            {
                if (this["CustomExternalApplications"] == null) CustomExternalApplications = new ObservableCollection<CustomExternalApplication>();
                return (ObservableCollection<CustomExternalApplication>)this["CustomExternalApplications"];
            }
            set { this["CustomExternalApplications"] = value; }
        }

        [UserScopedSetting()]
        public string SelectedMapSource
        {
            get { return (string)this["SelectedMapSource"]; }
            set { this["SelectedMapSource"] = value; }
        }

        [UserScopedSetting()]
        public SearchConfiguration SearchConfiguration
        {
            get
            {
                if (this["SearchConfiguration"] == null) SearchConfiguration = new SearchConfiguration();
                return (SearchConfiguration)this["SearchConfiguration"];
            }
            set { this["SearchConfiguration"] = value; }
        }

        [UserScopedSetting]
        public WaypointStyle WaypointStyle
        {
            get
            {
                if (this["WaypointStyle"] == null) WaypointStyle = new WaypointStyle();
                return (WaypointStyle)this["WaypointStyle"];
            }
            set { this["WaypointStyle"] = value; }
        }

        [UserScopedSetting]
        public RoutePointStyle RoutePointStyle
        {
            get
            {
                if (this["RoutePointStyle"] == null) RoutePointStyle = new RoutePointStyle();
                return (RoutePointStyle)this["RoutePointStyle"];
            }
            set { this["RoutePointStyle"] = value; }
        }

        [UserScopedSetting]
        public TrackPointStyle TrackPointStyle
        {
            get
            {
                if (this["TrackPointStyle"] == null) TrackPointStyle = new TrackPointStyle();
                return (TrackPointStyle)this["TrackPointStyle"];
            }
            set { this["TrackPointStyle"] = value; }
        }

        [UserScopedSetting]
        public SearchPointStyle SearchPointStyle
        {
            get
            {
                if (this["SearchPointStyle"] == null) SearchPointStyle = new SearchPointStyle();
                return (SearchPointStyle)this["SearchPointStyle"];
            }
            set { this["SearchPointStyle"] = value; }
        }

        [UserScopedSetting]
        public ClipboardSettings ClipboardSettings
        {
            get
            {
                if (this["ClipboardSettings"] == null) ClipboardSettings = new ClipboardSettings();
                return (ClipboardSettings)this["ClipboardSettings"];
            }
            set { this["ClipboardSettings"] = value; }
        }

        [UserScopedSetting]
        public IsoLanguageName Language
        {
            get
            {
                if (this["Language"] == null) this["Language"] = new IsoLanguageName();
                return (IsoLanguageName)this["Language"];
            }
            set
            {
                Localization.LocalizationManager.Instance.Language = (value);
                this["Language"] = value;
            }
        }

        [UserScopedSetting]
        public NetworkMode NetworkMode
        {
            get
            {
                if ((object)this["NetworkMode"] == null) NetworkMode = Maps.NetworkMode.Online;
                return (NetworkMode)this["NetworkMode"];
            }
            set
            {
                this["NetworkMode"] = value;
            }
        }

        [UserScopedSetting]
        public bool RoutePointsMultipleSelectionMode
        {
            get
            {
                if ((object)this["RoutePointsMultipleSelectionMode"] == null) RoutePointsMultipleSelectionMode = true;
                return (bool)this["RoutePointsMultipleSelectionMode"];
            }
            set
            {
                this["RoutePointsMultipleSelectionMode"] = value;
            }
        }

        [UserScopedSetting]
        public bool TrackPointsMultipleSelectionMode
        {
            get
            {
                if ((object)this["TrackPointsMultipleSelectionMode"] == null) TrackPointsMultipleSelectionMode = true;
                return (bool)this["TrackPointsMultipleSelectionMode"];
            }
            set
            {
                this["TrackPointsMultipleSelectionMode"] = value;
            }
        }

        [UserScopedSetting]
        public bool AutoCheckForNewVersion
        {
            get
            {
                if ((object)this["AutoCheckForNewVersion"] == null) AutoCheckForNewVersion = true;
                return (bool)this["AutoCheckForNewVersion"];
            }
            set
            {
                this["AutoCheckForNewVersion"] = value;
            }
        }

        [UserScopedSetting]
        public bool AskBeforeCheckForNewVersion
        {
            get
            {
                if ((object)this["AskBeforeCheckForNewVersion"] == null) AskBeforeCheckForNewVersion = true;
                return (bool)this["AskBeforeCheckForNewVersion"];
            }
            set
            {
                this["AskBeforeCheckForNewVersion"] = value;
            }
        }

        [UserScopedSetting]
        public DateTime LastCheckForNewVersion
        {
            get
            {
                if ((object)this["LastCheckForNewVersion"] == null) LastCheckForNewVersion = System.DateTime.Now;
                return (DateTime)this["LastCheckForNewVersion"];
            }
            set
            {
                this["LastCheckForNewVersion"] = value;
            }
        }

        [UserScopedSetting]
        public bool UpdateRequired
        {
            get
            {
                if ((object)this["UpdateRequired"] == null) return true;
                return (bool)this["UpdateRequired"];
            }
            set
            {
                this["UpdateRequired"] = value;
            }
        }
    }
}