namespace Brjnk.GpxExplorer.Workspace
{
    using System;
    using System.ComponentModel;
    using Brjnk.Common;
    using Brjnk.GpxExplorer.UI;

    /// <summary>
    /// </summary>
    public class VersionInfo : INotifyPropertyChanged
    {
        public VersionInfo()
        {
            ReadVersionInfo();
            CreateCommands();
            if (App.Instance.Settings.AutoCheckForNewVersion)
            {
                if ((DateTime.Now - App.Instance.Settings.LastCheckForNewVersion).TotalDays > NewVersionCheckPeriod)
                    DoAutoCheck();
            }
        }

        private void DoAutoCheck()
        {
            if (App.Instance.Settings.AskBeforeCheckForNewVersion)
            {
                var answer = UserMessages.Question(Brjnk.GpxExplorer.Localization.LocalizationManager.Instance.GetString("questionDoCheckForNewVersion"),
                     Brjnk.GpxExplorer.Localization.LocalizationManager.Instance.GetString("yes"),
                     Brjnk.GpxExplorer.Localization.LocalizationManager.Instance.GetString("no"));

                if ((answer ?? 1) == 0)
                    CheckNewVersion();
            }
            else
            {
                CheckNewVersion();
            }
            App.Instance.Settings.LastCheckForNewVersion = DateTime.Now;
        }

        private void ReadVersionInfo()
        {
            var assemblyName = System.Reflection.Assembly.GetExecutingAssembly().GetName();
            var version = assemblyName.Version;
            Version = version;
            Date = new DateTime(2000, 1, 1).AddDays(version.Build);
        }

        public bool IsBeta
        {
            get { return true; }
        }

        private Version version;

        public Version Version
        {
            get { return version; }
            private set { version = value; OnPropertyChanged("Version"); }
        }

        private DateTime date;

        public DateTime Date
        {
            get { return date; }
            private set { date = value; OnPropertyChanged("Date"); }
        }

        public int NewVersionCheckPeriod
        {
            get { return ReadNewVersionCheckPeriod(); }
        }

        private int ReadNewVersionCheckPeriod()
        {
            string value = System.Configuration.ConfigurationManager.AppSettings["NewVersionCheckPeriod"];
            if (string.IsNullOrEmpty(value)) return 50;
            int days;
            if (int.TryParse(value, out days))
            {
                if (days > 0) return days;
            }
            return 50;
        }

        private void CreateCommands()
        {
            CheckNewVersionCommand = new GenericCommand(o => CheckNewVersion());
        }

        public GenericCommand CheckNewVersionCommand { get; private set; }

        private void CheckNewVersion()
        {
            var window = new CheckNewVersionWindow();
            window.Show();
        }

        /// <summary>
        /// Vrt hodnotu AppDomain.GetData() pro kl DataDirectory
        /// </summary>
        /// <remarks>Tato vlastnost nepodporuje INotifyPropertyChanged</remarks>
        public string DataDirectory
        {
            get { return AppDomain.CurrentDomain.GetData("DataDirectory").ToString(); }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null) PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}