namespace Brjnk.Maps.Downloader
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    ///Fronta, kter upednostuje poloky s vy prioritou. Poloky se stejnou prioritou jsou seazeny LILO
    ///NEN thread-safe
    /// </summary>
    public class PriorityQueue<T> : IEnumerable<T>
    {
        private SortedList<int, LinkedList<T>> queues = new SortedList<int, LinkedList<T>>(new DescendingComparer<int>());
        private int count = 0;

        public void Enqueue(int priority, T item)
        {
            if (item == null) throw new ArgumentNullException();
            if (queues.ContainsKey(priority))
            {
                queues[priority].AddLast(item);
            }
            else
            {
                var q = new LinkedList<T>();
                q.AddLast(item);
                queues.Add(priority, q);
            }
            count++;
        }

        public T Dequeue()
        {
            int priority;
            return Dequeue(out priority);
        }

        public T Dequeue(out int priority)
        {
            return DequeueOrPeek(out priority, true);
        }

        public int Count
        {
            get
            {
                return count;
            }
        }

        public T Peek()
        {
            int priority;
            return Peek(out priority);
        }

        public T Peek(out int priority)
        {
            return DequeueOrPeek(out priority, false);
        }

        /// <summary>
        /// Odstran vechny objekty splujc podmnku
        /// </summary>
        /// <param name="onlyFirst">True, pokud se m odstranit pouze prvn vskyt (s nejvt prioritou)</param>
        public void RemoveWhere(Predicate<T> predict, bool onlyFirst = false)
        {
            //     var toRemove = new List<T>();
            for (int x = 0; x < queues.Count; x++)
            {
                var queue = queues.Values[x];
                var current = queue.First;

                while (true)
                {
                    if (current == null) break;
                    if (predict(current.Value))
                    {
                        queue.Remove(current);
                        count--;
                    }
                    current = current.Next;
                }
            }
        }

        private T DequeueOrPeek(out int priority, bool dequeue)
        {
            foreach (var queueKvp in queues)
            {
                if (queueKvp.Value.Count != 0)
                {
                    priority = queueKvp.Key;
                    if (dequeue)
                    {
                        var queue = queueKvp.Value;
                        var retVal = queue.First;
                        queue.RemoveFirst();
                        //odebrat frontu dan priority, pokud neobsahuje dal prvky
                        if (queue.Count == 0) queues.Remove(queueKvp.Key);
                        count--;
                        return retVal.Value;
                    }
                    else return PeekPrivate(queueKvp.Key);
                }
            }
            throw new System.InvalidOperationException("Fronta je przdn. PriorityQueue is empty.");
        }

        private T PeekPrivate(int priority)
        {
            return queues[priority].First.Value;
        }

        //-----------Enumerator
        private IEnumerator<T> GetEnumerator()
        {
            return queues.SelectMany(pair => pair.Value).GetEnumerator();
        }

        IEnumerator<T> IEnumerable<T>.GetEnumerator()
        {
            return GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
    }

    internal class DescendingComparer<T> : IComparer<T>
    {
        public int Compare(T x, T y)
        {
            return -Comparer<T>.Default.Compare(x, y);
        }
    }
}