namespace Brjnk.Maps
{
    using System;
    using System.ComponentModel;
    using System.Threading;
    using System.Windows.Media.Imaging;

    /// <summary>
    /// </summary>
    public class GetTileTask : System.ComponentModel.INotifyPropertyChanged
    {
        private Action<Downloader.DownloadTask> cancelDownload;
        private Downloader.DownloadTask downloadTask;
        private Exception exception;
        private TileId id;
        private BitmapImage image;
        private bool isCompleted;
        private bool isCancelled = false;
        private GetTileSource source;
        private SynchronizationContext syncContext;

        public GetTileTask(TileId tile, Action<Downloader.DownloadTask> cancelDownload, out GetTileTaskUpdater updater)
        {
            id = tile;
            this.cancelDownload = cancelDownload;
            syncContext = SynchronizationContext.Current;
            updater = new GetTileTaskUpdater(
                new Action<Downloader.DownloadTask>(t => DownloadTask = t),
                new Action<GetTileSource>(s => Source = s),
                Complete
            );
        }

        public event EventHandler Completed;

        public event PropertyChangedEventHandler PropertyChanged;

        public Downloader.DownloadTask DownloadTask
        {
            get { return downloadTask; }
            protected set
            {
                downloadTask = value;
                downloadTask.Completed += new EventHandler(downloadTask_Completed);
                if (isCancelled) CancelDownload();
                OnPropertyChanged("DownloadTask");
            }
        }

        public Exception Exception
        {
            get { return exception; }
            set { exception = value; OnPropertyChanged("Exception"); }
        }

        public BitmapImage Image
        {
            get { return image; }
            protected set { image = value; OnPropertyChanged("Image"); }
        }

        public GetTileSource Source
        {
            get { return source; }
            protected set { source = value; OnPropertyChanged("Source"); }
        }

        public TileId TileId { get { return id; } }

        public bool IsCompleted
        {
            get { return isCompleted; }
            protected set { isCompleted = value; OnPropertyChanged("IsCompleted"); }
        }

        public void CancelDownload()
        {
            isCancelled = true;
            if ((object)DownloadTask != null) cancelDownload(DownloadTask);
        }

        protected void OnCompleted()
        {
            if (Completed != null)
            {
                if (syncContext != null) syncContext.Post(o => Completed(o, EventArgs.Empty), this);
                else Completed(this, EventArgs.Empty);
            }
        }

        private void Complete(BitmapImage image, Exception e)
        {
            this.Image = image;
            this.Exception = e;
            this.IsCompleted = true;
            OnCompleted();
        }

        private void downloadTask_Completed(object sender, EventArgs e)
        {
            downloadTask.Completed -= new EventHandler(downloadTask_Completed);
            if (downloadTask.Exception != null) Complete(null, downloadTask.Exception);
            else Complete(ReadBitmap(downloadTask.Data), null);
        }

        private void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                if (syncContext != null) syncContext.Post(o => PropertyChanged(o, new PropertyChangedEventArgs(propertyName)), this);
                else PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private BitmapImage ReadBitmap(byte[] data)
        {
            var stream = new System.IO.MemoryStream(data);
            var image = new BitmapImage();
            image.BeginInit();
            image.StreamSource = stream;
            image.CacheOption = BitmapCacheOption.OnLoad;
            image.EndInit();

            image.Freeze();
            return image;
        }
    }

    public enum GetTileSource
    {
        Network = 1,
        Cache = 2,
        RamCache = 3
    }
}