namespace Brjnk.Maps.Sources
{
    using System;
    using System.Collections.Generic;
    using System.Windows;

    /// <summary>
    /// </summary>
    public class OsmMapSource : WebMapSource
    {
        private string name;

        private string uniqueName;

        private string pattern;

        private int maxZoom;

        public OsmMapSource(string name, string uniqueName, int maxZoom, string urlPattern)
        {
            this.name = name;
            this.uniqueName = uniqueName;
            pattern = urlPattern.ToLower();
            this.maxZoom = maxZoom;
            GenerateZoomLevels();
        }

        private void GenerateZoomLevels()
        {
            for (int i = 0; i <= maxZoom; i++)
            {
                double s = Math.Pow(2, -i);
                zoomLevels.Add(new ZoomLevel(i, new Size(s, s)));
            }
        }

        private List<ZoomLevel> zoomLevels = new List<ZoomLevel>();

        //                       {new ZoomLevel(0, new Size(1,1)),
        //                        new ZoomLevel(1, new Size (0.5,0.5)),
        //                        new ZoomLevel(2, new Size (0.25,0.25)),
        //                        new ZoomLevel(3, new Size (0.125,0.125)),
        //                        new ZoomLevel(4, new Size (0.0625,0.0625)),
        //                        new ZoomLevel(5, new Size (0.03125,0.03125)),
        //                        new ZoomLevel(6, new Size (0.015625,0.015625)),
        //                        new ZoomLevel(7, new Size (0.0078125,0.0078125)),
        //                        new ZoomLevel(8, new Size (0.00390625,0.00390625)),
        //                        new ZoomLevel(9, new Size (0.001953125,0.001953125)),
        //                        new ZoomLevel(10, new Size(0.0009765625,0.0009765625)),
        //                        new ZoomLevel(11, new Size(0.00048828125,0.00048828125)),
        //                        new ZoomLevel(12, new Size(0.000244140625,0.000244140625)),
        //                        new ZoomLevel(13, new Size(0.0001220703125,0.0001220703125)),
        //                        new ZoomLevel(14, new Size(0.00006103515625,0.00006103515625)),
        //                        new ZoomLevel(15, new Size(0.000030517578125,0.000030517578125)),
        //                        new ZoomLevel(16, new Size(0.0000152587890625,0.0000152587890625)),
        //                        new ZoomLevel(17, new Size(0.00000762939453125,0.00000762939453125)),
        //                        new ZoomLevel(18, new Size(0.000003814697265625,0.000003814697265625))
        //                         };

        public override System.Windows.Point GetTileCoordinates(System.Windows.Point internalPoint, int zoomLevel)
        {
            return new Point(internalPoint.X * Math.Pow(2, zoomLevel), internalPoint.Y * Math.Pow(2, zoomLevel));
        }

        public override IEnumerable<ZoomLevel> ZoomLevels
        {
            get { return zoomLevels; }
        }

        public override Uri GetTileUrl(TileId tile)
        {
            if (tile.X < 0 || tile.Y < 0 || tile.Zoom < 0) return null;
            if (tile.X > Math.Pow(2, tile.Zoom) || tile.Y > Math.Pow(2, tile.Zoom)) return null;
            if (tile.Zoom > maxZoom) System.Diagnostics.Debug.WriteLine("Nepodporovan stupe piblen");
            var url = GenerateUrl(tile.X, tile.Y, tile.Zoom);
            return new Uri(url);
        }

        private string GenerateUrl(long x, long y, int zoom)
        {
            string retVal = pattern.Replace("{x}", x.ToString());
            retVal = retVal.Replace("{y}", y.ToString());
            retVal = retVal.Replace("{z}", zoom.ToString());
            return retVal;

            // string.Format("http://a.tile.openstreetmap.org/{2}/{0}/{1}.png", tile.X, tile.Y, tile.Zoom);
        }

        public override bool IsTileAvailable(TileId tile)
        {
            if (tile.X < 0 || tile.Y < 0 || tile.Zoom < 0) return false;
            if (tile.X >= Math.Pow(2, tile.Zoom) || tile.Y >= Math.Pow(2, tile.Zoom) || tile.Zoom > maxZoom) return false;
            return true;
        }

        private MercatorProjection proj = new MercatorProjection();

        public override IMapProjection Projection
        {
            get { return proj; }
        }

        public override string SourceName
        {
            get { return name; }
        }

        public override string UniqueName
        {
            get { return name; }
        }

        public override Size TileDIPSize
        {
            get { return new Size(256, 256); }
        }
    }
}