﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using Brjnk.Maps;
using Brjnk.Maps.Downloader;
using MapsTest.TestClasses;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace MapsTest
{
    [TestClass()]
    public class DownloadSchedulerTest
    {
        private TestContext testContextInstance;

        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes

        //
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //

        #endregion Additional test attributes

        [TestMethod()]
        public void DownloadSchedulerConstructorTest()
        {
            //Testuje bez použití OverloadProtection
            ITimingProvider timing = new TestTimingProvider();
            DefaultTimingProvider.SetDefaultTimingProvider(timing);
            IOverloadProtection overloadProtection = null;
            MultiTimer multiTimer = new MultiTimer();
            DownloadScheduler target = new DownloadScheduler(overloadProtection, multiTimer);

            DownloadTaskUpdater updater;
            var t1 = new DownloadTask(new Uri("http://domain1.cz"), DownloadTaskPriority.High, out updater);
            var tX = new DownloadTask(new Uri("http://domainX.cz"), DownloadTaskPriority.High, out updater);
            var t3 = new DownloadTask(new Uri("http://domain3.cz"), DownloadTaskPriority.Normal, out updater);
            var t4 = new DownloadTask(new Uri("http://domain4.cz"), DownloadTaskPriority.Normal, out updater);
            var t2 = new DownloadTask(new Uri("http://domain2.cz"), DownloadTaskPriority.High, out updater);
            target.AddTask(t1, null);
            target.AddTask(tX, null);
            target.AddTask(t2, null);
            target.AddTask(t3, null);
            target.AddTask(t4, null);

            target.RemoveTask(tX);
            Assert.AreSame(t1, target.GetNextTask().First().Task);
            var x = target.GetNextTask().First().Task;
            Assert.AreSame(t2, x);
            Assert.AreSame(t3, target.GetNextTask().First().Task);
            Assert.AreSame(t4, target.GetNextTask().First().Task);

            //Přidání úlohy až po volání GetNextTask()
            System.Threading.ThreadPool.QueueUserWorkItem(new System.Threading.WaitCallback(obj =>
                {
                    System.Threading.Thread.Sleep(500);
                    target.AddTask(t1, null);
                }));

            Assert.AreSame(t1, target.GetNextTask().First().Task);
        }

        [TestMethod()]
        public void DownloadSchedulerConstructorTest2()
        {
            //Testuje s použitím OverloadProtection
            //Spustí Logger() již před přidáním první úlohy (na rozdíl od testu 3)
            TestTimingProvider timing = new TestTimingProvider();
            DefaultTimingProvider.SetDefaultTimingProvider(timing);
            IOverloadProtection overloadProtection = new TestOverloadProtection();
            MultiTimer multiTimer = new MultiTimer();
            DownloadScheduler target = new DownloadScheduler(overloadProtection, multiTimer);

            Thread t = new Thread(new ThreadStart(() => Logger(target)));
            t.Start();

            DownloadTaskUpdater updater;
            var t1 = new DownloadTask(new Uri("http://domain1.cz"), DownloadTaskPriority.High, out updater);
            var tX = new DownloadTask(new Uri("http://domainX.cz"), DownloadTaskPriority.High, out updater);
            var t2 = new DownloadTask(new Uri("http://domain2.cz"), DownloadTaskPriority.Normal, out updater);
            var t3 = new DownloadTask(new Uri("http://domain3.cz"), DownloadTaskPriority.Normal, out updater);
            var t4 = new DownloadTask(new Uri("http://domain4.cz"), DownloadTaskPriority.High, out updater);

            timing.Move(10000);
            Assert.AreEqual(0, logs.Count);
            target.AddTask(t1, null);
            target.AddTask(tX, null);
            target.AddTask(t2, null);

            timing.Move(1500);
            Assert.AreEqual(1, logs.Count);
            Assert.AreSame(t1.Uri, logs[0]);
            target.RemoveTask(tX);

            timing.Move(1000);
            Assert.AreEqual(2, logs.Count);
            Assert.AreEqual(t2.Uri, logs[1]);
            target.AddTask(t3, null);

            timing.Move(300);
            target.AddTask(t4, null);

            timing.Move(1000);
            Assert.AreEqual(3, logs.Count);
            Assert.AreEqual(t4.Uri, logs[2]);

            timing.Move(1000);
            Assert.AreEqual(4, logs.Count);
            Assert.AreEqual(t3.Uri, logs[3]);

            timing.Move(3000);
            Assert.AreEqual(4, logs.Count);
        }

        [TestMethod()]
        public void DownloadSchedulerConstructorTest3()
        {
            //Testuje s použitím OverloadProtection
            //Spustí Logger() až po přidání první úlohy
            TestTimingProvider timing = new TestTimingProvider();
            DefaultTimingProvider.SetDefaultTimingProvider(timing);
            IOverloadProtection overloadProtection = new TestOverloadProtection();
            MultiTimer multiTimer = new MultiTimer();
            DownloadScheduler target = new DownloadScheduler(overloadProtection, multiTimer);

            DownloadTaskUpdater updater;
            var t1 = new DownloadTask(new Uri("http://domain1.cz"), DownloadTaskPriority.Normal, out updater);
            var t2 = new DownloadTask(new Uri("http://domain2.cz"), DownloadTaskPriority.High, out updater);

            target.AddTask(t1, null);
            target.AddTask(t2, null);

            Thread t = new Thread(new ThreadStart(() => Logger(target)));
            t.Start();

            Assert.AreEqual(0, logs.Count);
            timing.Move(1000);
            Assert.AreEqual(1, logs.Count);
            Assert.AreEqual(t2.Uri, logs[0]);
            timing.Move(1000);
            Assert.AreEqual(2, logs.Count);
            Assert.AreEqual(t1.Uri, logs[1]);
        }

        private List<Uri> logs = new List<Uri>();

        //Zaznamenává hodnoty vrácené z GetNextTask v jiném vlákně
        private void Logger(DownloadScheduler sched)
        {
            logs.Clear();
            while (true)
            {
                var tasks = sched.GetNextTask();
                lock (logs)
                {
                    foreach (var task in tasks)
                    {
                        logs.Add(task.Task.Uri);
                        System.Diagnostics.Debug.WriteLine(task.Task.Uri);
                    }
                }
            }
        }
    }
}